/****************************************************************
*	FluLibMath.cpp												*
*	Assembled by SHady el Mously								*
*	This is the mathematical implementation of all the 3d math	*
*	classes and syucts in the library							*
*	all classes defined here are extensivly useful and very		*
*	important in the makking of any 3d application				*
*	a deep understanding of 3d math foundation is required in	*
*	order to really digest what thoise classes can do			*
*	This part of the library is completly indepedant of any		*
*	part of the library											*
*	but all 3d stuff will be needing those classes				*
****************************************************************/
#include "3dmath.h"


/****************************************************************
*	struct point3d implementation								*
*	by shady el mously											*
*	basic vetor object operations and main constants are also	*
*	defined here												*
****************************************************************/


// some stuff realted to the point3d.. defined explicitly

const point3d point3d::Zero( 0.f,0.f,0.f );
const point3d point3d::i( 1.f, 0.f, 0.f );
const point3d point3d::j( 0.f, 1.f, 0.f );
const point3d point3d::k( 0.f, 0.f, 1.f );


// a remaining function for point 3d.. that relates it to point4d..
point3d::point3d( const point4d& in )
{
	/**
	 * If the homogenous coordinate isn't 1, normalize the point.
	 */
	if( in.w != 1.f )
	{
		float inv = 1/in.w;
		x = in.x * inv;
		y = in.y * inv;
		z = in.z * inv;
	}
	else
	{
		x = in.x;
		y = in.y;
		z = in.z;
	}
}


/*----------------end of class point3d implementation---------*/
	// can test lines, polys, and bboxes.
ePListLoc plane3d::testplist( point3d *list, int num ) const
{
	bool allfront=true, allback=true;
	ePointLoc res;
	for( int i=0; i<num; i++ )
	{
		res = testpoint( list[i] );
		if( res == ptBack )
		{
			allfront = false;
		}
		else if( res == ptFront )
		{
			allback = false;
		}
	}
	if( allfront && !allback )
	{
		// All the points were either in front or coplanar
		return plistFront;
	}
	else if( !allfront && allback )
	{
		// All the points were either in back or coplanar
		return plistBack;
	}
	else if( !allfront && !allback )
	{
		// Some were in front, some were in back
		return plistSplit;
	}
	// All were coplanar
	return plistCoplanar;
}


// can test lines, polys, and bboxes.
ePListLoc plane3d::testpoly( const polygon<point3d> &poly ) const
{
	return testplist( poly.plist, poly.numelem );
}

// testing a sphere with the plane..
// front.. back.. or partially here and there
eSphereLoc plane3d::testsphere( const bsphere3d& sphere ) const
{
	float dp = (sphere.m_loc * n) + d;

	if((dp - sphere.m_radius) > EPSILON)
	{
		// Sphere was completely in front
		return sphereFront;
	}
	if((dp + sphere.m_radius) < -EPSILON )
	{
		// Sphere was completely in back
		return sphereBack;
	}
	// Sphere was partially in each
	return sphereCoplanar; 
}

// this will clip a polygon to another one.. in front of the plane
bool plane3d::clip( const polygon<point3d> &in, polygon<point3d> *out ) const
{
	assert( out ); // Make sure our pointer to the out polygon is valid
	assert( in.numelem > 2 ); // Make sure we're not passed a degenerate polygon

	int thisInd=in.numelem-1;
	int nextInd=0;

	ePointLoc thisRes = testpoint( in.plist[thisInd] );
	ePointLoc nextRes;

	out->numelem = 0;

	for( nextInd=0; nextInd<in.numelem; nextInd++ ) 
	{

		nextRes = testpoint( in.plist[nextInd] );

		if( thisRes == ptFront || thisRes == ptCoplanar )
		{
			// Add the point
			out->plist[out->numelem++] = in.plist[thisInd];
		}

	if( ( thisRes == ptBack && nextRes == ptFront ) ||
		( thisRes == ptFront && nextRes == ptBack ) )
		{
			// Add the split point
			out->plist[out->numelem++] = split( in.plist[thisInd], in.plist[nextInd] );
		}

		thisInd = nextInd;
		
		thisRes = nextRes;
	}
	if( out->numelem >= 3 )
	{
		return true;
	}
		return false;
}

// this one will split a polygon into 2 polygons.. one in front and one in back..
bool plane3d::split( polygon<point3d> const &in, polygon<point3d> *pFront, polygon<point3d> *pBack ) const
{
	assert( pFront ); // Make sure our pointer to the out polygon is valid
	assert( pBack ); // Make sure our pointer to the out polygon is valid
	assert( in.numelem > 2 ); // Make sure we're not passed a degenerate polygon

	// Start with curr as the last vertex and next as 0. 
	pFront->numelem = 0;
	pBack->numelem = 0;

	int thisInd=in.numelem-1;
	int nextInd=0;

	ePointLoc thisRes = testpoint( in.plist[thisInd] );
	ePointLoc nextRes;

	for( nextInd=0; nextInd<in.numelem; nextInd++) 
	{
		nextRes = testpoint( in.plist[nextInd] );

		if( thisRes == ptFront )
		{
			// Add the point to the front
			pFront->plist[pFront->numelem++] = in.plist[thisInd];
		}

		if( thisRes == ptBack )
		{
			// Add the point to the back
			pBack->plist[pBack->numelem++] = in.plist[thisInd];
		}

		if( thisRes == ptCoplanar ) 
		{ 
			// Add the point to both
			pFront->plist[pFront->numelem++] = in.plist[thisInd];
			pBack->plist[pBack->numelem++] = in.plist[thisInd];
		}

		if( ( thisRes == ptBack && nextRes == ptFront ) ||
			( thisRes == ptFront && nextRes == ptBack ) )
		{ 
			// Add the split point to both
			point3d Split = split( in.plist[thisInd], in.plist[nextInd] );
			pFront->plist[pFront->numelem++] = Split;
			pBack->plist[pBack->numelem++] = Split;
		}
		thisInd = nextInd;
		thisRes = nextRes;
	}
	if( pFront->numelem >= 3 && pBack->numelem >= 3 )
	{ 
		// Nothing ended up degenerate
		return true;
	}
	return false;
}


	// this will split a segment into 2 lines
	// back on front.. return false on failure..
bool plane3d::split( const lineseg3d& in, lineseg3d *front, lineseg3d *back ) const
{
	point3d temp = split( in.v0, in.v1 );

	if( testpoint( in.v0 ) == ptFront )
	{
		front->v0 = in.v0;
		front->v1 = temp;
		back->v0 = temp;
		back->v1 = in.v1;
		return true;
	}
	else 
	{
		front->v0 = temp;
		front->v1 = in.v1;
		back->v0 = in.v0;
		back->v1 = temp;
		return true;
	}
	return false;
}

//	this will will get the reflected vector on the plane
//	from andre lamothe
point3d plane3d::reflect(point3d& I)
{
	point3d N=(-I*n)*n;
	point3d F=2*N+I;
	return F;
}

/****************************************************************
*	struct color3 implementation								*
*	just implementing the constansts in the struct				*
*	the standarda colors..										*
****************************************************************/


const color3 color3::Black( 0.f,0.f,0.f );
const color3 color3::Gray( 0.5f,0.5f,0.5f );
const color3 color3::White( 1.f,1.f,1.f );
const color3 color3::Red( 1.f,0.f,0.f );
const color3 color3::Green( 0.f,1.f,0.f );
const color3 color3::Blue( 0.f,0.f,1.f );
const color3 color3::Magenta( 1.f,0.f,1.f );
const color3 color3::Cyan( 0.f,1.f,1.f );
const color3 color3::Yellow( 1.f,1.f,0.f );

/*------------end of struct color3 implementation--------------*/



/****************************************************************
*	struct color4 implementation								*
*	just like color3, only the standarda colros here are defined*
****************************************************************/
// th color definition for constant colors
const color4 color4::Black( 0.f, 0.f, 0.f, 1.f );
const color4 color4::Gray( 0.5f, 0.5f, 0.5f, 1.f );
const color4 color4::White( 1.f, 1.f, 1.f, 1.f );
const color4 color4::Red( 1.f, 0.f, 0.f, 1.f );
const color4 color4::Green( 0.f, 1.f, 0.f, 1.f );
const color4 color4::Blue( 0.f, 0.f, 1.f, 1.f );
const color4 color4::Magenta( 1.f, 0.f, 1.f, 1.f );
const color4 color4::Cyan( 0.f, 1.f, 1.f, 1.f );
const color4 color4::Yellow( 1.f, 1.f, 0.f, 1.f );


/*--------------end of struct color4 implementation------------*/


/****************************************************************
*	struct matrix4 implementation								*
*	constants and operators										*
****************************************************************/

//returns matrix4*matrix4
matrix4 operator*(matrix4 const &a, matrix4 const &b)
{
    matrix4 out;      // temporary matrix4 for storing result
    for (int j = 0; j < 4; j++)         // transform by columns first
        for (int i = 0; i < 4; i++)     // then by rows
            out.m[i][j] = a.m[i][0] * b.m[0][j] + a.m[i][1] * b.m[1][j] +
                           a.m[i][2] * b.m[2][j] + a.m[i][3] * b.m[3][j];
	return out;
};

// Multiplies two matrices together
void MatMult( const matrix4 &a, const matrix4& b, matrix4* result )
{
    for (int j = 0; j < 4; j++)         // transform by columns first
        for (int i = 0; i < 4; i++)     // then by rows
            result->m[i][j] = a.m[i][0] * b.m[0][j] + a.m[i][1] * b.m[1][j] +
                              a.m[i][2] * b.m[2][j] + a.m[i][3] * b.m[3][j];
}

const matrix4 matrix4::Identity = matrix4(
	1,0,0,0,
	0,1,0,0,
	0,0,1,0,
	0,0,0,1);

const matrix4 matrix4::beziermatrix=matrix4(
	-1.0f,+3.0f,-3.0f,+1.0f,
	+3.0f,-6.0f,+3.0f,+0.0f,
	-3.0f,+3.0f,+0.0f,+0.0f,
	+1.0f,+0.0f,+0.0f,+0.0f
	);

void matrix4::rotate( float yaw, float pitch, float roll )
{
    int i;
    float sinangx,cosangx,sinangy,cosangy,sinangz,cosangz;
    float xold,yold,zold,xnew,ynew,znew;//temporary rotation results to prevent round off error
    sinangx=(float)sin(yaw);
    cosangx=(float)cos(yaw);
    sinangy=(float)sin(pitch);
    cosangy=(float)cos(pitch);
    sinangz=(float)sin(roll);
    cosangz=(float)cos(roll);
    for(i=0; i<3; i++)
    {
////////init points/////////////////////
        xold=m[i][0];
        yold=m[i][1];
        zold=m[i][2];
////////rotate point locally////////////        
        //xrot
        xnew=xold;
        ynew=(yold*cosangx)-(zold*sinangx);
        znew=(yold*sinangx)+(zold*cosangx);
        xold=xnew;
        yold=ynew;
        zold=znew;
        //yrot            
        xnew=((xold*cosangy)+(zold*sinangy));
        ynew=yold;
        znew=((-1*(xold*sinangy))+(zold*cosangy));
        xold=xnew;
        yold=ynew;
        zold=znew;
        //zrot
        xnew=((xold*cosangz)-(yold*sinangz));
        ynew=((yold*cosangz)+(xold*sinangz));
        znew=zold;
        //change object location
        m[i][0]=xnew;
        m[i][1]=ynew;
        m[i][2]=znew;
    }
}

void matrix4::translate( const point3d& amt)
{
    _41 += amt.x;
    _42 += amt.y;
    _43 += amt.z;
}

void matrix4::place( const point3d& loc)
{
    _41 = loc.x;
    _42 = loc.y;
    _43 = loc.z;
}

void matrix4::makeident()
{
	_11 = 1.f;	_22 = 1.f;	_33 = 1.f;	_44 = 1.f;
	_12 = 0.f;  _13 = 0.f;  _14 = 0.f;	_21 = 0.f;
	_23 = 0.f;	_24 = 0.f;	_31 = 0.f;	_32 = 0.f;	
	_34 = 0.f;	_41 = 0.f;	_42 = 0.f;	_43 = 0.f;	
}

//==========--------------------------  

void matrix4::isviewmatrix( const point3d& loc, const point3d& lookAt, const point3d& inUp )
{

    point3d viewVec = lookAt - loc;
 
    float mag = viewVec.mag();
 
    viewVec /= mag;
 
    float fDot = inUp * viewVec;
 
    point3d upVec = inUp - fDot * viewVec;
 
	upVec.normalize();
	
    point3d rightVec = upVec ^ viewVec;


    // Start building the matrix. The first three rows contains the basis
    // vectors used to rotate the view to point at the lookat point
    _11 = rightVec.x;    _21 = upVec.x;    _31 = viewVec.x;
    _12 = rightVec.y;    _22 = upVec.y;    _32 = viewVec.y;
    _13 = rightVec.z;    _23 = upVec.z;    _33 = viewVec.z;
 
    // Do the translation values (rotations are still about the eyepoint)
    _41 = loc.x;
    _42 = loc.y;
    _43 = loc.z;

    _14 = 0;
    _24 = 0;
    _34 = 0;
    _44 = 1;


}

matrix4 matrix4::viewmatrix( 
	const point3d& loc, 
	const point3d& lookAt, 
	const point3d& inUp )
{
	matrix4 out;
	out.isviewmatrix( loc, lookAt, inUp );
	return out;
}

//==========--------------------------  

void matrix4::iscameramatrix( const point3d& loc, const point3d& lookAt, const point3d& inUp )
{

    point3d viewVec = lookAt - loc;
 
    float mag = viewVec.mag();
 
    viewVec /= mag;
 
    float fDot = inUp * viewVec;
 
    point3d upVec = inUp - fDot * viewVec;
 
	upVec.normalize();
	
    point3d rightVec = upVec ^ viewVec;


    // Start building the matrix. The first three rows contains the basis
    // vectors used to rotate the view to point at the lookat point
    _11 = rightVec.x;    _12 = upVec.x;    _13 = viewVec.x;
    _21 = rightVec.y;    _22 = upVec.y;    _23 = viewVec.y;
    _31 = rightVec.z;    _32 = upVec.z;    _33 = viewVec.z;
 
    // Do the translation values (rotations are still about the eyepoint)
    _41 = - (loc * rightVec);
    _42 = - (loc * upVec);
    _43 = - (loc * viewVec);

    _14 = 0;
    _24 = 0;
    _34 = 0;
    _44 = 1;
}


matrix4 matrix4::cameramatrix( 
	const point3d& loc, 
	const point3d& lookAt, 
	const point3d& inUp )
{
	matrix4 out;
	out.iscameramatrix( loc, lookAt, inUp );
	return out;
}

//==========--------------------------  

void matrix4::isXrotation( float theta )
{
	float c = (float) cos(theta);
	float s = (float) sin(theta);
	makeident();
	_22 = c;
	_23 = s;
	_32 = -s;
	_33 = c;
}

matrix4 matrix4::Xrotation( float theta )
{
	matrix4 out;
	out.isXrotation( theta );
	return out;
}

//==========--------------------------  

void matrix4::isYrotation( float theta )
{
	float c = (float) cos(theta);
	float s = (float) sin(theta);
	makeident();
	_11 = c;
	_13 = -s;
	_31 = s;
	_33 = c;
}

matrix4 matrix4::Yrotation( float theta )
{
	matrix4 out;
	out.isYrotation( theta );
	return out;
}

//==========--------------------------  

void matrix4::isZrotation( float theta )
{
	float c = (float) cos(theta);
	float s = (float) sin(theta);
	makeident();
	_11 = c;
	_12 = s;
	_21 = -s;
	_22 = c;
}

matrix4 matrix4::Zrotation( float theta )
{
	matrix4 out;
	out.isZrotation( theta );
	return out;
}

//==========--------------------------  

void matrix4::isaxisrotation( const point3d& inAxis, float angle )
{
	point3d axis = inAxis.normalized();
	float s = (float)sin( angle );
	float c = (float)cos( angle );
	float x = axis.x, y = axis.y, z = axis.z;

	_11 = x*x*(1-c)+c;
	_21 = x*y*(1-c)-(z*s);
	_31 = x*z*(1-c)+(y*s);
	_41 = 0;
	_12 = y*x*(1-c)+(z*s);
	_22 = y*y*(1-c)+c;
	_32 = y*z*(1-c)-(x*s);
	_42 = 0;
	_13 = z*x*(1-c)-(y*s);
	_23 = z*y*(1-c)+(x*s);
	_33 = z*z*(1-c)+c;
	_43 = 0;
	_14 = 0;
	_24 = 0;
	_34 = 0;
	_44 = 1;
}

matrix4 matrix4::axisrotation( const point3d& axis, float angle )
{
	matrix4 out;
	out.isaxisrotation( axis, angle );
	return out;
}


//==========--------------------------  

void matrix4::istranslation( const point3d& p )
{
	makeident();
	_41 = p.x;
	_42 = p.y;
	_43 = p.z;
}

matrix4 matrix4::translation( const point3d& p )
{
	matrix4 out;
	out.istranslation( p );
	return out;
}


//==========--------------------------  

void matrix4::isscale( const point3d& scale )
{
	makeident();
	_11 = scale.x;
	_22 = scale.y;
	_33 = scale.z;
}

matrix4 matrix4::scale( const point3d& scale )
{
	matrix4 out;
	out.isscale( scale );
	return out;
}

//==========--------------------------  

void matrix4::isinverse( const matrix4& in )
{

	// first transpose the rotation matrix
	_11 = in._11;
	_12 = in._21;
	_13 = in._31;
	_21 = in._12;
	_22 = in._22;
	_23 = in._32;
	_31 = in._13;
	_32 = in._23;
	_33 = in._33;

	// fix right column
	_14 = 0;
	_24 = 0;
	_34 = 0;
	_44 = 1;

	// now get the new translation vector
	point3d temp = in.getloc();

	_41 = -(temp.x * in._11 + temp.y * in._12 + temp.z * in._13);
	_42 = -(temp.x * in._21 + temp.y * in._22 + temp.z * in._23);
	_43 = -(temp.x * in._31 + temp.y * in._32 + temp.z * in._33);

}

matrix4 matrix4::inverse( const matrix4& in )
{
	matrix4 out;
	out.isinverse( in );
	return out;
}

void matrix4::flip()
{
	matrix4 old=*this;

	for(int i=0;i<4;i++)
		for(int j=0;j<4;j++)
		{
			
			m[i][j]=old.m[j][i];
			
		}
}
//==========--------------------------  

const point3d matrix4::getloc() const
{
	return point3d( _41, _42, _43 );
}

point3d matrix4::getaxis( int axis ) const
{
	return point3d
	(
		m[axis][0],
		m[axis][1],
		m[axis][2]
	);
}


/*-----------end of struct matrix4 implementation---------------*/



/****************************************************************
*	struct bsptree												*
*	undocmumented ofr fufture use								*
****************************************************************/

cBspTree::cBspTree()
{
}

cBspTree::~cBspTree()
{
	// destroy the tree
}


void cBspTree::AddPolygon( const polygon<point3d>& in )
{
	if( !m_pHead )
	{
		// if there's no tree, make a new one
		m_pHead = new cNode( in );
	}
	else
	{
		// otherwise add it to the tree
		m_pHead->AddPolygon( in );
	}
}

void cBspTree::AddPolygonList( vector< polygon<point3d> >& in )
{
	if( !m_pHead )
	{
		// if there's no tree, make a new one
		m_pHead = new cNode( in );
	}
	else
	{
		/**
		 * Adding a list of polygons to 
		 * an existing tree is unimplemented 
		 * (exercise to the reader)*/
		 
		assert( false );
	}
}


void cBspTree::TraverseTree( vector<polygon<point3d>*>* polyList, const point3d& loc )
{
	if( m_pHead )
	{
		// drop it down
		m_pHead->TraverseTree( polyList, loc );
	}
}


bool cBspTree::LineOfSight( const point3d& a, const point3d& b )
{
	assert( m_pHead ); // make sure there is a tree to test against

	return m_pHead->LineOfSight( a, b );
}


cBspTree::cNode::~cNode()
{
	// we could run into some stack overflow problems, but
	// the tree will most likely not be deep enough for us to worry about it.
	delete m_pFront;
	delete m_pBack;
}


cBspTree::cNode::cNode( bool bIsSolid ) : 
	m_bIsLeaf( true ), 
	m_bIsSolid( bIsSolid ),
	m_pFront( NULL ),
	m_pBack( NULL )
{
	// all done.
}

cBspTree::cNode::cNode( const polygon<point3d>& in ) : 
	m_bIsLeaf( false ), 
	m_poly( in ),
	m_plane( in ),
	m_pFront( new cNode( false ) ),
	m_pBack( new cNode( true ) )
{
	// all done.
}

cBspTree::cNode::cNode( vector< polygon<point3d> >& in ) :
	m_bIsLeaf( false )
{

	// if the list is empty, we're bombing out.
	assert( in.size() );

	// get the best index to use as a splitting plane
	int bestIndex = BestIndex( in );

	// we could remove the index from the vector, but that's slow.  
	// instead we'll just kind of ignore it during the next phase.
	// remove the best index
	polygon<point3d> splitPoly = in[bestIndex];

	m_plane = plane3d( splitPoly );
	m_poly = splitPoly;

	// take the rest of the polygons and divide them.
	vector< polygon<point3d> > frontList, backList;

	int i;
	for( i=0; i<in.size(); i++ )
	{
		// ignore the polygon if it's the one we're using as the splitting plane
		if( i == bestIndex ) continue;

		// test the polygon against this node.
		ePListLoc res = m_plane.testpoly( in[i] );

		polygon<point3d> front(in[i].numelem + 5), back(in[i].numelem + 5); // used in PLIST_SPLIT

		switch( res )
		{
		case plistFront:
			// drop down the front
			frontList.push_back( in[i] );
			break;
		case plistBack:
			// drop down the back
			backList.push_back( in[i] );
			break;
		case plistSplit:
			// split the polygon, drop the halves down.
			m_plane.split( in[i], &front, &back );
			frontList.push_back( front );
			backList.push_back( back );
			break;
		case plistCoplanar:
			// add the polygon to this node's list
			m_coplanarList.push_back( in[i] );
			break;
		}
	}

	// we're done processing the polygon list  Deal with them.
	if( frontList.size() )
	{
		m_pFront = new cNode( frontList );
	}
	else
	{
		m_pFront = new cNode( false );
	}
	if( backList.size() )
	{
		m_pBack = new cNode( backList );
	}
	else
	{
		m_pBack = new cNode( true );
	}

}

void cBspTree::cNode::AddPolygon( const polygon<point3d>& in )
{
	if( m_bIsLeaf )
	{
		// reinitialize ourselves as a node
		*this = cNode( in );
	}
	else
	{
		// test the polygon against this node.
		ePListLoc res = this->m_plane.testpoly( in );

		polygon<point3d> front(in.numelem+1), back(in.numelem+1); // used in PLIST_SPLIT
		switch( res )
		{
		case plistFront:
			// drop down the front
			m_pFront->AddPolygon( in );
			break;
		case plistBack:
			// drop down the back
			m_pBack->AddPolygon( in );
			break;
		case plistSplit:
			// split the polygon, drop the halves down.
			m_plane.split( in, &front, &back );
			m_pFront->AddPolygon( front );
			m_pBack->AddPolygon( back );
			break;
		case plistCoplanar:
			// add the polygon to this node's list
			m_coplanarList.push_back( in );
			break;
		}
	}
}

void cBspTree::cNode::TraverseTree( vector< polygon<point3d>* >* polyList, const point3d& loc )
{
	if( m_bIsLeaf )
	{
		// do nothing.
	}
	else
	{
		// test the loc against the current node
		ePointLoc res = m_plane.testpoint( loc );

		int i;
		switch( res )
		{
		case ptFront:
			// get back, us, front
			m_pBack->TraverseTree( polyList, loc );
			polyList->push_back( &m_poly ); // the poly at this node
			for( i=0; i<m_coplanarList.size(); i++ )
			{
				polyList->push_back( &m_coplanarList[i] );
			}
			m_pFront->TraverseTree( polyList, loc );
			break;

		case ptBack:
			// get front, us, back
			m_pFront->TraverseTree( polyList, loc );
			polyList->push_back( &m_poly ); // the poly at this node
			for( i=0; i<m_coplanarList.size(); i++ )
			{
				polyList->push_back( &m_coplanarList[i] );
			}
			m_pBack->TraverseTree( polyList, loc );
			break;

		case ptCoplanar:
			// get front, back, us
			m_pFront->TraverseTree( polyList, loc );
			m_pBack->TraverseTree( polyList, loc );
			polyList->push_back( &m_poly ); // the poly at this node
			for( i=0; i<m_coplanarList.size(); i++ )
			{
				polyList->push_back( &m_coplanarList[i] );
			}
			break;

		}
	}
}

int cBspTree::cNode::BestIndex( vector< polygon<point3d> >& polyList )
{
	/**
	 * The current hueristic is blind least-split*/
	 
	// run through the list, searching for the best one.
	int maxCheck; // the highest polygon we'll bother testing (10% of total)
	maxCheck = (int)(polyList.size() * percentageToCheck);
	if( !maxCheck ) maxCheck = 1;

	int i, i2;
	int bestSplits = 100000;
	int bestIndex = -1;
	int currSplits;
	plane3d currPlane;
	for( i=0; i<maxCheck; i++ )
	{
		currSplits = 0;
		currPlane = plane3d( polyList[i] );
		ePListLoc res;

		for( i2=0; i2< polyList.size(); i2++ )
		{
			if( i == i2 ) continue;

			res = currPlane.testpoly( polyList[i2] );
			if( res == plistSplit )
				currSplits++;
		}
		if( currSplits < bestSplits )
		{
			bestSplits = currSplits;
			bestIndex = i;
		}
	}
	assert( bestIndex >= 0 );
	return bestIndex;
}


bool cBspTree::cNode::LineOfSight( const point3d& a, const point3d& b )
{
	if( m_bIsLeaf )
	{
		return !m_bIsSolid; // if we land in a solid node, then there is no line of sight
	}

	// FIXME: not correctly handling both verts coplanar
	ePointLoc aLoc = m_plane.testpoint( a );
	ePointLoc bLoc = m_plane.testpoint( b );

	point3d split;

	if( aLoc == ptCoplanar && bLoc == ptCoplanar )
	{
		// for sake of something better to do, be conservative
		//return false;
		return m_pFront->LineOfSight( a, b );
	}

	if( aLoc == ptFront && bLoc == ptBack )
	{
		//split, then return the logical 'or' of both sides
		split = m_plane.split( a, b );

		return m_pFront->LineOfSight( a, split ) && m_pBack->LineOfSight( b, split );

	}

	if( aLoc == ptBack && bLoc == ptFront )
	{
		//split, then return the logical 'or' of both sides
		split = m_plane.split( a, b );

		return m_pFront->LineOfSight( b, split ) && m_pBack->LineOfSight( a, split );
	}


	if( aLoc == ptFront || bLoc == ptFront ) // the other == POINT_COLPLANAR or POINT_FRONT
	{
		// drop down the front
		return m_pFront->LineOfSight( a, b );
	}

	else // they're both on the back side
	{
		// drop down the front
		return m_pBack->LineOfSight( a, b );
	}


	return true;
}


/*---------end of struct bsptree implementation-----------------*/