#ifndef _3DMATH_H
#define _3DMATH_H
/****************************************************************
*	3dmath.h													*
*	this is the header ( definiction) of all the math classes	*
*	and structs that will be uextesivly used in the making of	*
*	any 3d application											*
*	mathematical classes here are vectors(point3d and 4d)		*
*	and trnaformation matrices and bounding spheres				*
*	and also 3 and 4 valued colors								*
*	and it is finished with a binary search path tree			*
*	most the here code is from (advanced 3d game programming)	*
*	i just modified slightly some things to adopt it to			*
*	my library													*
*	Those classes depend to a high extent on the STL			*
*	most classes here have all their methods as inline			*
*	this is just to assure the most optimum in preforamce		*
****************************************************************/

//	STL stuff
#include <algorithm>
#include <iterator>
#include <vector>
#include <list>

#include <assert.h>
#include <math.h>

// some global defines.
#define EPSILON 0.001f
#define PI 3.1415926f
#define SIGN_BIT 0x80000000
#define INV_SIGN_BIT 0x7FFFFFFF

// some fast important functions
// perform absolute value by AND'ing out the sign bit
inline float fastabs( const float &in )
{
	int* temp = (int*)&in;
	int out = *temp & INV_SIGN_BIT;
	return *((float*)&out);
}

// generates a random number between -1 and 1
inline float frand()
{
	return (float)(rand()-RAND_MAX/2) / (RAND_MAX/2);
}



struct point4d;




/****************************************************************
*	struct point3d	declaration									*
*	th main and basic and most funcametnal math class			*
*	this struct represents a vector in 3d space					*
*	many overloads are used with vectors						*
*	such as addition, subtraction and many useful things		*
*	another struct point4d will be defined later				*
*	it will be exactly like this sone but it has a fourth		*
*	coordinate (W) or the homogenous coordiante					*	
*	which facilates the matrix math stuff later					*
*	many opertaors are define explicitly						*
****************************************************************/


struct point3d 
{

	union 
	{
		// This structure here is nameless, which really isn't 
		// legal C++... most compilers will deal with it,
		// however.
		struct 
		{
            float x,y,z; // 3 real components of the vector
        };
        float	v[3]; // Array access useful in loops
    };

	// Default constructor, defined for template classes
	point3d()
	{
	}

	// Construct a point with 3 given inputs
	point3d( float X, float Y, float Z ) : 
		x(X), y(Y), z(Z)
	{
	}

	point3d( const point4d& in );

	// Construct a point using spherical coordinates
	static const point3d spherical( float theta, float rho, float phi )
	{
		return point3d(
			(float)(phi*cos(rho)*sin(theta)), 
			(float)(phi*sin(rho)), 
			(float)(phi*cos(rho)*cos(theta))
		);
	}

	// Reassign a point without making a temporary structure
	void assign( float X, float Y, float Z )
	{
			x=X;	y=Y;	z=Z;
	}

	// getting the magnitude.. or the distance to origin
	float mag() const
	{
		return (float) sqrt(x*x + y*y + z*z);
	}
	// the same.. return the mag squared.. sometimes.. we will need this..
	float magsquared()const
	{
		return x*x + y*y + z*z;
	}
	// normalize..makes the vector a unit vector one.. direction
	void normalize()
	{
		float foo=1/mag();
		x*=foo;
		y*=foo;
		z*=foo;
	}

	// returns a Normalized version of this
	const point3d normalized() const
	{
		point3d temp(x,y,z);
		temp.normalize();
		return temp;
	}

	// Computes the distance between two vectors
	static float dist( const point3d &a, const point3d &b )
	{
		// can't use the subtraction operator (it's not defined yet!)
		point3d distVec( b.x - a.x, b.y - a.y, b.z - a.z );
		return distVec.mag();
	}

	point3d& operator += ( const point3d& in );
	point3d& operator -= ( const point3d& in );
	point3d& operator *= ( const float& in );
	point3d& operator /= ( const float& in );
	point3d  operator -  (); // this is plain old negation
	point3d& operator=(float f)
	{
		x=y=z=f;
		return *this;
	}


	/**
	 * Constant vectors
	 */
	static const point3d Zero; // <0,0,0>
	static const point3d i; // <1,0,0>
	static const point3d j; // <0,1,0>
	static const point3d k; // <0,0,1>
}; 

/*-------------end of struct point3d declaration-----------------------*/


/************************************************************************
*	operator overloaded functions for the point3d struct				*
*	all is inline for speed												*
************************************************************************/

/**
 * Accumulative addition of two vectors
 */
inline point3d& point3d::operator += ( const point3d& in )
{
	x += in.x;   
	y += in.y;   
	z += in.z;
	return *this; // This is so we can chain operations together
}


/**
 * Accumulative subtraction of two vectors
 */
inline point3d& point3d::operator -= ( const point3d& in )
{
	x -= in.x;   
	y -= in.y;   
	z -= in.z;
	return *this; // This is so we can chain operations together
}


/**
 * Accumulative multiplication of a vector by a scalar
 */
inline point3d& point3d::operator *= ( const float& in )
{
	x *= in;   
	y *= in;   
	z *= in;
	return *this; // This is so we can chain operations together
}


/**
 * Accumulative division of a vector by a scalar
 */
inline point3d& point3d::operator /= ( const float& in )
{
	float inv = 1.f / in; // do one (expensive) divide and 3 (cheap) mults
	x *= inv;   
	y *= inv;   
	z *= inv;
	return *this; // This is so we can chain operations together 
}

inline point3d point3d::operator - () 
{
	return point3d(
		-x,
		-y,
		-z
	);
}


// first.. addition and subtraction of vectors////////////////////////
inline point3d operator+(point3d const &a,point3d const &b)
{
	return point3d
		(
		a.x+b.x,
		a.y+b.y,
		a.z+b.z
		);
}



// similarly.. subtraction
inline point3d operator-(point3d const &a,point3d const &b)
{
	return point3d
		(
		a.x-b.x,
		a.y-b.y,
		a.z-b.z
		);
}


// scaling.. the * operator	(by a float)
inline point3d operator*(const point3d &a,const float &s)
{
	return point3d
		(
		a.x*s,
		a.y*s,
		a.z*s
		);
}

// scaling.. the * operator	(by a float)
inline point3d operator*(const float &s,const point3d &a)
{
	return point3d
		(
		a.x*s,
		a.y*s,
		a.z*s
		);
}


// similarly.. division

inline point3d operator/(point3d const &a,float const &f)
{	
	float s=1.0f/f;
	return point3d
		(
		a.x*s,
		a.y*s,
		a.z*s
		);
}



// overloading == eqaulity to check if 2 points are the same
inline bool operator==(point3d const &a,point3d const &b)
{
	if(fabs(a.x-b.x)<EPSILON)
	{
		if(fabs(a.y-b.y)<EPSILON)
		{
			if(fabs(a.z-b.z)<EPSILON)
			{
				return true;
			}
		}
	}
	return false;
}


// we return to a float the dot product of 2 vecotrs..
inline float operator*(const point3d &a,const point3d &b)
{
	return a.x*b.x + a.y*b.y + a.z*b.z;
}


//	to get a vector.. perpendicular on both points.. 
//	we use the ^ operator hich return 
//	the cross product of the 2 vectors.. (vector quantity
inline point3d operator^(point3d const &a,point3d const &b)
{
	return point3d
		(
			(a.y*b.z-a.z*b.y),
			(a.z*b.x-a.x*b.z),
			(a.x*b.y-a.y*b.x)
		);
}
/*---------------------end of point3d operators--------------------*/



/********************************************************************
*	struct point4d	declaration										*
*	completly like the previous point3d struct						*
*	just with a fourth componenet that allows the compatibility		*
*	with the matrix coming later									*
********************************************************************/



struct point4d 
{
	union 
	{
        struct {float x,y,z,w;};
		float	v[4];// for looping
    };

	point4d(){}

	point4d( float X, float Y, float Z, float W ) :
		x(X), y(Y), z(Z), w(W)
	{
	
	}

	point4d( point3d& in ) :
		x(in.x), y(in.y), z(in.z), w(1.f)
	{
	}

	void assign( float X, float Y, float Z, float W )
	{
		x=X;	y=Y;	z=Z;	w=W;
	}

	void negate()
	{
		x=-x;	y=-y;	z=-z;	w=-w;
	}

	float mag()
	{
		return (float)sqrt( x*x + y*y + z*z + w*w );
	}

	void normalize()
	{
		float foo=1/mag();	x*=foo;	y*=foo;	z*=foo; w*=foo;
	}

}; 

/*-------------end of struct point4d implementation--------------*/




/********************************************************************
*	operator overloaded fcuntions for the point4d sructure			*
*	all are inline for assuring speed								*
********************************************************************/

//returns point4+point4
inline const point4d operator+(point4d const &a, point4d const &b)
{
	return point4d(a.x+b.x,a.y+b.y,a.z+b.z,a.w+b.w);
}; 


//returns point4-point4
inline const point4d operator-(point4d const &a, point4d const &b)
{
	return point4d(a.x-b.x,a.y-b.y,a.z-b.z,a.w-b.w);
}; 
//*

//returns point4*float
inline const point4d operator*(point4d const &a, float const &b)
{
	return point4d(a.x*b,a.y*b,a.z*b,a.w*b);
}; 


//returns float*point4
inline const point4d operator*(float  const &a, point4d const &b)
{
	return point4d(a*b.x,a*b.y,a*b.z,a*b.w);
}; 


//returns point4/float
inline const point4d operator/(point4d const &a, float const &b)
{
	return point4d(a.x/b,a.y/b,a.z/b,a.w/b);
}; 

//returns true if point4==point4
inline bool operator==(point4d const &a, point4d const &b)
{
	if(fabs(a.x-b.x)<EPSILON)
		if(fabs(a.y-b.y)<EPSILON)
			if(fabs(a.z-b.z)<EPSILON)
				if(fabs(a.w-b.w)<EPSILON)
					return true;
	return false;
};

// returns a dot b
inline float operator*(point4d const &a, point4d const &b)
{
	return a.x*b.x + a.y*b.y + a.z*b.z + a.w*b.w;
} 

inline point3d operator-(point3d const &a,float i)
{
	return point3d(a.x+i,a.y+i,a.z+i);
}


/*------------ end of point4d operators functions--------------*/



/********************************************************************
*	lineseg3d struct												*
*	This is a very simple line in 3d space... made from 2 points	*
*	from type point3d, useful later									*
********************************************************************/

// a line segment..simply 2 points...

struct lineseg3d
{
	point3d v0,v1;

	lineseg3d( const point3d &a, const point3d &b )
	{
		v0 = a; v1 = b;
	}
	lineseg3d(){}
};

/*------------------end of struct lineseg3d-----------------------*/





/********************************************************************
*	struct polygon													*
*	This struct represent a polygon in 3d space						*
*	and add functionality to split a poly on a plane				*
*	it is a template class, polygns can be made from point3d		*
*	or from point4d structs											*
********************************************************************/

template <class type>
struct polygon 
{
	int numelem; // number of elements in the polygon
	int maxelem; // maximaum elemts the polygon can have

	type *plist;

	polygon()
	{
		numelem = 0;
		maxelem = 0;
		plist = NULL;
	}

	polygon( int maxSize )
	{
		maxelem = maxSize;
		plist = new type[maxSize];
	}

	polygon( const polygon &in )
	{
		clonedata( in );
	}

	~polygon()
	{
		destroydata();
	}

	void clonedata( const polygon &in )
	{
		if( !in.plist ) 
			return;

		plist = new type[in.maxelem];
		maxelem = in.maxelem;
		numelem = in.numelem;
		for( int i=0; i<in.numelem; i++ )
		{
			plist[i] = in.plist[i];
		}
	}

	void destroydata( )
	{
		if( plist )
		{
			delete[] plist;
		}
		plist = NULL;
	}

	polygon& operator=( const polygon<type> &in )
	{
		if( &in != this )
		{
			destroydata();

			clonedata( in );
		}
		
		return *this;
	}
};

/*----------------end of class polygon-------------------------*/




/****************************************************************
*	struct tri													*
*	a triangle, justa special case of the more general polygon	*
*	this triangle is the most basic drawn 3d primitive			*
*	it is also a templatized class								*
****************************************************************/

// a simple triangle..also a template class
template <class type>
struct tri 
{

    type v[3]; // Array access useful in for loops
	
	tri()
	{
		// nothing
	}

	tri( type v0, type v1, type v2 )
	{
		v[0] = v0;
		v[1] = v1;
		v[2] = v2;
	}
};


/*--------------end of struct triangle-------------------------*/

/****************************************************************
*	struct quad													*
*	a triangle, justa special case of the more general polygon	*
*	this triangle is the most basic drawn 3d primitive			*
*	it is also a templatized class								*
****************************************************************/

// a simple triangle..also a template class
template <class type>
struct quad
{

    type v[4]; // Array access useful in for loops
	
	quad()
	{
		// nothing
	}

	quad( type v0, type v1, type v2 ,type v3)
	{
		v[0] = v0;
		v[1] = v1;
		v[2] = v2;
		v[3] = v3;
	}
};
/*--------------end of struct triangle-------------------------*/



/****************************************************************
*	struct bspher3d												*
*	this class represnt a sphere with radius in 3d space		*
*	spheres can be checked fro collision detection, also		*
*	this class may be a base class for all physics and also		*
*	artificial intelligence classes defined int future modules	*
*	of Physics and of Artificial intelligence					*
****************************************************************/


struct bsphere3d
{
	// the data is a 3d point.. and a radius
	float m_radius;
	point3d m_loc;

	bsphere3d():m_radius(1.0f),m_loc(point3d::Zero)
	{
			// all is set
	}

	// construct from a point3d and a float
	bsphere3d( float radius, point3d loc ) :
		m_radius( radius ), m_loc( loc )
	{
			// all is set
	}


	// construct.. using a point as a center.. and a list of point ..( a polygon..
	// the algorithm will pass through al the points in the list and find the rauid needed
	// to represent this object with a sphere
	bsphere3d( point3d loc, int nVerts, point3d* pList )
	{
		m_loc = loc;
		m_radius = 0.f;
		float currRad;
		for( int i=0; i< nVerts; i++ )
		{
			currRad = pList[i].mag();
			if( currRad > m_radius )
			{
				m_radius = currRad;
			}
		}
	}

	// template constructor..
	// using this stuff
	// a boundinjg sphere is made.. from a point
	// and two other primitives.. point3d or point4d
	template< class iter >
	bsphere3d( point3d loc, iter& begin, iter& end )
	{
		iter i = begin;
		m_loc = loc;
		m_radius = 0.f;
		float currRad;
		while( i != end )
		{
			currRad = (*i).Mag();
			if( currRad > m_radius )
			{
				m_radius = currRad;
			}
			i++;
		}
	}

	bsphere3d& operator += ( const point3d& in );
	bsphere3d& operator -= ( const point3d& in );

	// this will return true if two spheres intersect..
	static bool intersect( bsphere3d& a, bsphere3d& b )
	{
		// avoid a square root by squaring both sides of the equation
		float magSqrd = a.m_radius * a.m_radius + b.m_radius * b.m_radius;
		if( (b.m_loc - a.m_loc).magsquared() > magSqrd )
		{
			return false;
		}
		return true;
	}

	// same but to test with he current calling object
	bool hit( bsphere3d& a)
	{
		return intersect(*this,a);
	}
};


/*--------------end of struct bsphere3d declaration--------------------*/



/************************************************************************
*	operator overloads for the bounding sphere struct					*
************************************************************************/

inline bsphere3d& bsphere3d::operator += ( const point3d& in )
{
	m_loc += in;
	return *this;
}

inline bsphere3d& bsphere3d::operator -= ( const point3d& in )
{
	m_loc -= in;
	return *this;
}

inline const bsphere3d operator+(bsphere3d const &a, point3d const &b)
{
	return bsphere3d( a.m_radius, b+a.m_loc ); 
}

inline const bsphere3d operator-(bsphere3d const &a, point3d const &b)
{
	return bsphere3d( a.m_radius, b-a.m_loc ); 
}



/*-----------end of operator overloads for the bsphere3d struct----*/



/********************************************************************
*	handline planes, i will define now a class for represeting		*
*	3d planes, this class will have functions which will define		*
*	the locality between points, lines, spheres and a plan			*
*	so i begin by enumarting those stuff							*
********************************************************************/

// Defines the three possible locations of a point in 
// relation to a plane
enum ePointLoc
{
	ptFront,
	ptBack,
	ptCoplanar
};

// Defines the four possible locations of a point list in 
// relation to a plane.  A point list is a more general
// example of a polygon.
enum ePListLoc
{
	plistFront,
	plistBack,
	plistSplit,
	plistCoplanar
};

// same idea but for bounsinf spheres
enum eSphereLoc
{
	sphereFront, // Sphere was entirely in front
	sphereBack, // Sphere was entirely in back
	sphereCoplanar // Sphere was partially on either side
};


/********************************************************************
*	struct plane3d declaration										*
*	any plane can be defined in 3d space by a normal vector to its	*
*	surface and a distnace to the origin							*
*	the struct here will handle many important things				*
********************************************************************/

struct plane3d 
{

	point3d n; // Normal of the plane
	float d; // Distance along the normal to the origin 

	plane3d( float nX, float nY, float nZ, float D) : n( nX, nY, nZ ), d( D )
	{
		// All done.
	}
	plane3d( const point3d& N, float D) : n( N ), d( D )
	{
		// All done.
	}

	// Construct a plane from three 3-D points
	plane3d( const point3d& a, const point3d& b, const point3d& c)
	{
		n = (b-a)^(c-a);
		n.normalize();
		d = -(n*a);
	}


	// Construct a plane from a normal direction and a point on the plane
	plane3d( const point3d& norm, const point3d& loc):    n( norm ), d( -(norm*loc) )
	{
		// All done
	}

	// Construct a plane from a polygon
	plane3d( const polygon<point3d>& poly )
	{
		point3d a = poly.plist[0];
		point3d b = poly.plist[1];
		point3d c = poly.plist[2];

		n = (b-a)^(c-a);
		n.normalize();
		d = -(n*a);
	}

	plane3d()
	{
		// Do nothing
	}

	// all constructors defined..


	// now for some functions which relates planes to other things
	
	// tests a point against this plane
	ePointLoc testpoint( const point3d& point ) const
	{
		float dp = (point * n) + d;

		if(dp > EPSILON)
		{
			return ptFront;
		}
		if(dp < -EPSILON )
		{
			return ptBack;
		}
		return ptCoplanar; // it was between EP and -EP
	}

	// can test lines, polys, and bboxes.
	ePListLoc testpoly( const polygon<point3d> &poly ) const;

	// can test lines, polys, and bboxes.
	ePListLoc testplist( point3d *list, int num ) const;

	// now for some function to cut things.. between sides of a plane..
	// first one is to fint the intersection point if we link 2 points on both sides of a plane
	const point3d split( const point3d &a, const point3d &b ) const
	{
		float aDot = (a * n);
		float bDot = (b * n);
		float scale = ( -d - aDot ) / ( bDot - aDot );
		return a + (scale * (b - a));
	}

	// this will bring the point to the plane..
	const point3d bringtoplane( const point3d& pt )
	{
		float distToPlane = (pt * n);
		return point3d( pt - (d + distToPlane)*n );
	}


	
	// this will clip a polygon to another one.. in front of the plane
	bool clip( const polygon<point3d> &in, polygon<point3d> *out ) const;


	// this one will split a polygon into 2 polygons.. one in front and one in back..
	bool split( polygon<point3d> const &in, polygon<point3d> *pFront, polygon<point3d> *pBack ) const;


	// this will flip the orientation of the plane
	void Flip()
	{
		n = -n;
		d = -d;
	}
	
	// this will split a segment into 2 lines
	// back on front.. return false on failure..
	bool split( const lineseg3d& in, lineseg3d *front, lineseg3d *back ) const;


	// testing a sphere with the plane..
	// front.. back.. or partially here and there
	eSphereLoc testsphere( const bsphere3d& sphere ) const;
	//	this will will get the reflected vector on the plane
	//	from andre lamothe
	point3d reflect(point3d& I);
};


// this small overload to test equality of planes
inline bool operator==(plane3d const &a, plane3d const &b)
{
	if( fabs(a.d-b.d) < EPSILON )
		if( a.n == b.n )
			return true;
	return false;
};

/*---------------end of class plane3d declalration--------------*/


/********************************************************************
*	struct color3													*
*	this struct represents a simple RGB color						*
*	all channel values are in a float, the channel is full			*
*	when the value is 1.0 and empty at 0.0							*
********************************************************************/

struct color3
{
	// the data is 3 floats.. red, green and blue..
	// value of on1 is full and zero is empty
	// using unions enable the accessing as arrays also..
	union {
		struct 
		{
			float r, g, b; // Red, Green, and Blue color data
        };
		float c[3];
	};

	// empty constructor
	color3(){}

	// construst usin 3 floats
	color3( float inR, float inG, float inB ) :
		r( inR ), g( inG ), b( inB )
	{
	}

	// give some calues if needed
	void assign( float inR, float inG, float inB )
	{
		r = inR;
		g = inG;
		b = inB;
	}
	// give some calues if needed
	void assignrgb( int inR, int inG, int inB )
	{
		r = inR/255.0f;
		g = inG/255.0f;
		b = inB/255.0f;
	}

	// this will return the value of the coor as a dword..
	/// (like 0xff00ff00 per example.; usefeul if i wanna use the coor in other standard
	/// gdi or any other calls
	unsigned long makeword()
	{
		unsigned long iR = (int)(r * 255.f ) << 16;
		unsigned long iG = (int)(g * 255.f ) << 8;
		unsigned long iB = (int)(b * 255.f );
		return 0xff000000 | iR | iG | iB;
	}

	// same as make word
	unsigned long makewordsafe()
	{
		color3 temp = *this;
		temp.sat();
		return temp.makeword();
	}


	// this will bound the color channels value between 0 and 1 as is expected..
	// if any of the values are >1, cap them.
	void sat()
	{
		if( r > 1 ) 
			r = 1.f;
		if( g > 1 ) 
			g = 1.f;
		if( b > 1 ) 
			b = 1.f;
		if( r < 0 ) 
			r = 0.f;
		if( g < 0 ) 
			g = 0.f;
		if( b < 0 ) 
			b = 0.f;
	}

	// operators for adding and subtracting and dividing and multiplying colors
	color3& operator += ( const color3& in );
	color3& operator -= ( const color3& in );
	color3& operator *= ( const float& in );
	color3& operator /= ( const float& in );


	// some basic colors.(static)
	static const color3 Black;
	static const color3 Gray;
	static const color3 White;
	static const color3 Red;
	static const color3 Green;
	static const color3 Blue;
	static const color3 Magenta;
	static const color3 Cyan;
	static const color3 Yellow;
};
/*-------------end of struct color3 operators------------------*/


/****************************************************************
*	operator overloads for struct color3						*
****************************************************************/

/**
 * Accumulative addition of two colors
 */
inline color3& color3::operator += ( const color3& in )
{
	r += in.r;   
	g += in.g;   
	b += in.b;
	return *this;
}


/**
 * Accumulative subtraction of two colors
 */
inline color3& color3::operator -= ( const color3& in )
{
	r -= in.r;   
	g -= in.g;   
	b -= in.b;
	return *this;
}


/**
 * Accumulative multiplication of a color by a scalar
 */
inline color3& color3::operator *= ( const float& in )
{
	r *= in;   
	g *= in;   
	b *= in;
	return *this;
}


/**
 * Accumulative division of a color by a scalar
 */
inline color3& color3::operator /= ( const float& in )
{
	float inv = 1.f / in;
	r *= inv;   
	g *= inv;   
	b *= inv;
	return *this;
}


/**
 * Adds two colors together: ret = a + b
 */
inline color3 operator+(color3 const &a, color3 const &b)
{
	return color3
	(
		a.r+b.r,
		a.g+b.g,
		a.b+b.b
	);
}; 
 

/**
 * Subtracts two colors : ret = a - b
 */
inline color3 operator-(color3 const &a, color3 const &b)
{
	return color3
	(
		a.r-b.r,
		a.g-b.g,
		a.b-b.b
	);
}; 


/**
 * Scales a color by a float : ret = a * b
 */
inline color3 operator*(color3 const &a, float const &b)
{
	return color3
	(
		a.r*b,
		a.g*b,
		a.b*b
	);
}; 


/**
 * Scales a color by a float : ret = a * b
 */
inline color3 operator*(float  const &a, color3 const &b)
{
	return color3
	(
		a*b.r,
		a*b.g,
		a*b.b
	);
}; 


/**
 * Divides a color by a float : ret = a / b
 */
inline color3 operator/(color3 const &a, float const &b)
{
	float inv = 1.f / b;
	return color3
	(
		a.r*inv,
		a.g*inv,
		a.b*inv
	);
}; 


/**
 * color Equality, epsilon used due to numerical imprecision
 */
inline bool operator==(color3 const &a, color3 const &b)
{
	if(fastabs(a.r-b.r)<EPSILON)
		if(fastabs(a.g-b.g)<EPSILON)
			if(fastabs(a.b-b.b)<EPSILON)
				return true;
	return false;
};


/*-----------end of struct color3 overloads--------------------*/


/****************************************************************
*	struct point4d												*
*	same as point3d but a\with the alpha componenet				*
*	this values identifies the transparency						*
****************************************************************/

struct color4
{
	union {
		struct 
		{
			float r, g, b, a; // Red, Green, and Blue color data
        };
		float c[4];
	};

	// empty constructor
	color4(){}

	// construct using 4 floating point values
	color4( float inR, float inG, float inB, float inA ) :
		r( inR ), g( inG ), b( inB ), a( inA )
	{
	}

	// construct using a color and a alpha value
	color4( const color3& in, float alpha = 1.f )
	{
		r = in.r;
		g = in.g;
		b = in.b;
		a = alpha;
	}

	// using a format like 0xff00ff00.. construct
	color4( unsigned long color )
	{
		b = (float)(color&255) / 255.f;
		color >>= 8;
		g = (float)(color&255) / 255.f;
		color >>= 8;
		r = (float)(color&255) / 255.f;
		color >>= 8;
		a = (float)(color&255) / 255.f;
	}


	// set value of a color4 value
	void assign( float inR, float inG, float inB, float inA )
	{
		r = inR;
		g = inG;
		b = inB;
		a = inA;
	}
	// give some calues if needed
	void assignrgb( int inR, int inG, int inB ,int inA)
	{
		r = inR/255.0f;
		g = inG/255.0f;
		b = inB/255.0f;
		a=	inA/255.0f;
	}

	// same idea.. return the word 0xaarrggbb
	unsigned long makeword()
	{
		unsigned long iA = (int)(a * 255.f ) << 24;
		unsigned long iR = (int)(r * 255.f ) << 16;
		unsigned long iG = (int)(g * 255.f ) << 8;
		unsigned long iB = (int)(b * 255.f );
		return iA | iR | iG | iB;
	}

	// return 0xaarrggbb safely
	unsigned long makewordsafe()
	{
		color4 temp = *this;
		temp.sat();
		return temp.makeword();
	}

	// if any of the values are >1, cap them.
	void sat()
	{
		if( r > 1 ) 
			r = 1.f;
		if( g > 1 ) 
			g = 1.f;
		if( b > 1 ) 
			b = 1.f;
		if( a > 1 ) 
			a = 1.f;
		if( r < 0 ) 
			r = 0.f;
		if( g < 0 ) 
			g = 0.f;
		if( b < 0 ) 
			b = 0.f;
		if( a < 0 ) 
			a = 0.f;
	}

	color4& operator += ( const color4& in );
	color4& operator -= ( const color4& in );
	color4& operator *= ( const color4& in );
	color4& operator /= ( const color4& in );
	color4& operator *= ( const float& in );
	color4& operator /= ( const float& in );

	// some basic colors.
	static const color4 Black;
	static const color4 Gray;
	static const color4 White;
	static const color4 Red;
	static const color4 Green;
	static const color4 Blue;
	static const color4 Magenta;
	static const color4 Cyan;
	static const color4 Yellow;

};
/*----------------end of struct color4 declaration------------*/

/****************************************************************
*	operator overload for the color4 struct						*
****************************************************************/


/**
 * Accumulative addition of two colors
 */
inline color4& color4::operator += ( const color4& in )
{
	r += in.r;   
	g += in.g;   
	b += in.b;
	a += in.a;
	return *this;
}


/**
 * Accumulative subtraction of two colors
 */
inline color4& color4::operator -= ( const color4& in )
{
	r -= in.r;   
	g -= in.g;   
	b -= in.b;
	a -= in.a;
	return *this;
}


/**
 * Accumulative multiplication of a color by a color
 */
inline color4& color4::operator *= ( const color4& in )
{
	r *= in.r;   
	g *= in.g;   
	b *= in.b;
	a *= in.a;
	return *this;
}


/**
 * Accumulative division of a color by a color
 */
inline color4& color4::operator /= ( const color4& in )
{
	r /= in.r;   
	g /= in.g;   
	b /= in.b;
	a /= in.a;
	return *this;
}


/**
 * Accumulative multiplication of a color by a scalar
 */
inline color4& color4::operator *= ( const float& in )
{
	r *= in;   
	g *= in;   
	b *= in;
	a *= in;
	return *this;
}


/**
 * Accumulative division of a color by a scalar
 */
inline color4& color4::operator /= ( const float& in )
{
	float inv = 1.f / in;
	r *= inv;   
	g *= inv;   
	b *= inv;
	a *= inv;
	return *this;
}


/**
 * Adds two colors together: ret = a + b
 */
inline color4 operator+(color4 const &a, color4 const &b)
{
	return color4
	(
		a.r+b.r,
		a.g+b.g,
		a.b+b.b,
		a.a+b.a
	);
}; 
 

/**
 * Subtracts two colors : ret = a - b
 */
inline color4 operator-(color4 const &a, color4 const &b)
{
	return color4
	(
		a.r-b.r,
		a.g-b.g,
		a.b-b.b,
		a.a-b.a
	);
}; 


/**
 * Scales a color by a float : ret = a * b
 */
inline color4 operator*(color4 const &a, float const &b)
{
	return color4
	(
		a.r*b,
		a.g*b,
		a.b*b,
		a.a*b
	);
}; 


/**
 * Scales a color by a float : ret = a * b
 */
inline color4 operator*(float  const &a, color4 const &b)
{
	return color4
	(
		a*b.r,
		a*b.g,
		a*b.b,
		a*b.a
	);
}; 


/**
 * Divides a color by a float : ret = a / b
 */
inline color4 operator/(color4 const &a, float const &b)
{
	float inv = 1.f / b;
	return color4
	(
		a.r*inv,
		a.g*inv,
		a.b*inv,
		a.a*inv
	);
}; 

/**
 * Divides a color by a color (piecewise): ret = a / b
 */
inline color4 operator/(color4 const &a, color4 const &b)
{
	return color4
	(
		a.r / b.r,
		a.g / b.g,
		a.b / b.b,
		a.a / b.a
	);
}; 

/**
 * Multiplies a color by a color (piecewise): ret = a / b
 */
inline color4 operator*(color4 const &a, color4 const &b)
{
	return color4
	(
		a.r * b.r,
		a.g * b.g,
		a.b * b.b,
		a.a * b.a
	);
}; 


/**
 * color Equality, epsilon used due to numerical imprecision
 */
inline bool operator==(color4 const &a, color4 const &b)
{
	if(fastabs(a.r-b.r)<EPSILON)
		if(fastabs(a.g-b.g)<EPSILON)
			if(fastabs(a.b-b.b)<EPSILON)
				if(fastabs(a.a-b.a)<EPSILON)
					return true;
	return false;
};


/*----------end of struct color4 overloads---------------------*/



/****************************************************************
*	struct matrix4												*
*	the most important structre in 3d math						*
*	here inside the matrix, all the magic is hidden				*
*	matrices can represnt anything can be imagines in 3d space	*
*	from the tarnaformation, rotaion, translation, scaling		*
*	reflection, bezier curves, spline curves					*
*	all matrices are 4*4 matrices, this is the reason that		*
*	i made the struct point4d which will facilate the			*
*	multipliication of apoint with amatrix to apply the matrix	*
*	effect , using operator overloading							*
*	matrices may be multiplied togetehr							*
****************************************************************/
struct matrix4  
{

	/**
	 * we're using m[y][x] as our notation.
	 */
    union 
	{
        struct 
		{
            float	_11, _12, _13, _14;
            float	_21, _22, _23, _24;
            float	_31, _32, _33, _34;
            float	_41, _42, _43, _44;
        };
        float	m[4][4];		//	can be accesses as a 3d array 4*4 (familar with direct3d
		float   d[16];			//	can be accessed as a single 16 floats arrays (OpenGL familiar)
    };

	/**
	 * justification for a function this ugly:
	 * provides an easy way to initialize static matrix variables 
	 * like base matrices for bezier curves and the identity
	 */
	matrix4(
		float in11, float in12, float in13, float in14,
        float in21, float in22, float in23, float in24,
        float in31, float in32, float in33, float in34,
        float in41, float in42, float in43, float in44 )
	{
        _11 = in11; _12 = in12; _13 = in13; _14 = in14;
        _21 = in21; _22 = in22; _23 = in23; _24 = in24;
        _31 = in31; _32 = in32; _33 = in33; _34 = in34;
        _41 = in41; _42 = in42; _43 = in43; _44 = in44;
	}

	matrix4()
	{
		// Do nothing.  
	}

	// each basic tranforms have 2 function..
	// one chnges the current matrix.. othr return the required

	// rotation about the x-axis
	void isXrotation( float theta );
	static matrix4 Xrotation( float theta );

	// rotation about the y-axis
	void isYrotation( float theta );
	static matrix4 Yrotation( float theta );

	// rotation about the z-axis
	void isZrotation( float theta );
	static matrix4 Zrotation( float theta );

	// rotation about any axe in space.. very useful
	void isaxisrotation( const point3d& inAxis, float angle );
	static matrix4 axisrotation( const point3d& inAxis, float angle );


	/**
	 * Do not confuse with Translate()
	 * Translation clears the matrix, making it a Translation matrix
	 */

	// transate by a vector
	void istranslation( const point3d& p );
	static matrix4 translation( const point3d& p );

	
	/**
	 * Note that scale is not a rigid-body transform
	 */
	// scale by a ceratin vector
	void isscale( const point3d& scale );
	static matrix4 scale( const point3d& scale );

	

	// this will view matrices.. and set the current a s aview matrix
	// the inup vector is 0,1,0 by default

	void isviewmatrix( 
		const point3d& loc, 
		const point3d& lookAt, 
		const point3d& inUp = point3d(0,1,0) );

	// this will view matrices.. and set the current a s aview matrix
	// the inup vector is 0,1,0 by default
	static matrix4 viewmatrix( 
		const point3d& loc, 
		const point3d& lookAt, 
		const point3d& inUp = point3d(0,1,0) );

	
	// like the same but the inverted matrix is used.. 

	void iscameramatrix( 
		const point3d& loc, 
		const point3d& lookAt, 
		const point3d& inUp = point3d(0,1,0) );
	static matrix4 cameramatrix( 
		const point3d& loc, 
		const point3d& lookAt, 
		const point3d& inUp = point3d(0,1,0) );

	/**
	 * Computes the inverse of a rigid-body transform
	 */
	void isinverse( const matrix4& in );
	static matrix4 inverse( const matrix4& in );

	/**
	 * Make the matrix the identity matrix
	 */
	void makeident();

	/**
	 * Rotate an existing matrix by a given yaw, pitch,
	 * and roll.
	 */
	// this is very userful also..

	void rotate( float yaw, float pitch, float roll );
	
	/**
	 * These functions are incremental; 
	 * they take an existing matrix and modify it.
	 * rotation information stays.
	 */
	void translate( const point3d& amt );
	void place( const point3d& loc );

	//	flipping a matrix coverts rows to columns and columns to rows..
	void flip();

	const point3d getloc() const;

	/**
	 * GetAxis only returns the correct axis if we're talking about a matrix that
	 * transforms 'up' towards worldspace.  For example, this will return the 
	 * correct axis if called on a object->world matrix, but incorrect one if
	 * called on the world->camera matrix.  (If one needs the axes for such a 
	 * matrix, invert the world->camera transform to get a camera->world 
	 * transform)
	 */
	point3d getaxis( int axis ) const;

	operator point3d(){return point3d(_41,_42,_43);}

	static const matrix4 beziermatrix;

	static const matrix4 Identity;
};


/*---------end of struct matrix4 declaration-----------------------*/






/********************************************************************
*	operator overloads for struct matrix4							*
********************************************************************/

/**
 * returns matrix4*matrix4
 * not inlined due to the size.
 */
matrix4 operator*(matrix4 const &a, matrix4 const &b);
void MatMult( const matrix4 &a, const matrix4& b, matrix4* result );


/**
 * Specialization for multiplication by a point3 structure.
 * in a mathematically perfect world, we would construct
 * a 1x4 matrix for the point, and then multiply the 1x4
 * and the 4x4 matrices toegether.  However, this performs
 * much better.
 */
inline const point3d operator*( const matrix4 &a, const point3d &b)
{
	return point3d(
		b.x*a._11 + b.y*a._21 + b.z*a._31 + a._41,
		b.x*a._12 + b.y*a._22 + b.z*a._32 + a._42,
		b.x*a._13 + b.y*a._23 + b.z*a._33 + a._43
	);
};

inline const point3d operator*( const point3d &a, const matrix4 &b)
{
	return b*a;
};


/**
 * Specialization for multiplication by a point4 structure.
 */
inline const point4d operator*( const matrix4 &a, const point4d &b)
{
	return point4d(
		b.x*a._11 + b.y*a._21 + b.z*a._31 + b.w*a._41,
		b.x*a._12 + b.y*a._22 + b.z*a._32 + b.w*a._42,
		b.x*a._13 + b.y*a._23 + b.z*a._33 + b.w*a._43,
		b.x*a._14 + b.y*a._24 + b.z*a._34 + b.w*a._44
	);
};

inline const point4d operator*( const point4d &a, const matrix4 &b)
{
	return b*a;
};


//returns matrix4*lineSeg3
inline lineseg3d operator*( lineseg3d const &b, matrix4 const &a )
{
	return lineseg3d( b.v0 * a, b.v1 * a );
};
//returns matrix4*lineSeg3
inline lineseg3d operator*( matrix4 const &a, lineseg3d const &b )
{
	return lineseg3d( b.v0 * a, b.v1 * a );
};




/*---------end of operator overloads of struct matrix4-------------*/
/********************************************************************
*	classes used for interpolation methods							*
********************************************************************/

template <typename T>
class cInterpolation
{
public:

	
	virtual bool	Setup(T&,T&,float)=0;
	virtual void	Interpolate()=0;
	virtual T&		GetValue()
	{	return _value;	}
	virtual bool	Done()=0;


protected:

	T _value;
	T _target;
	float _divisor;



};

template<typename T>
class cEaseOutDivideInterpolation:public cInterpolation<T>
{
public:


	bool Setup(T& from,T& to,float divisor)
	{
		if(divisor<=0)			return false;
		_value		=	from;
		_target		=	to;
		_divisor	=	divisor;
		return true;
	}


	void Interpolate()
	{
		if(Done())	return;
		if(_divisor>0)
		{
			_value=(_value*(_divisor-1.0f)+_target)/_divisor;
		}
	}

	bool Done()
	{
		return _value==_target;
	}


};


class cEaseOutShiftInterpolation
: public cInterpolation<int>
{
private:

	int _shift;

public:

	bool Setup(int from,int to, int shift)
	{
		if(shift<=0)
			return false;

		_value	=	from;
		_target	=	to;
		_shift	=	shift;

		return true;
	}

	void Interpolate()
	{
		if(_shift>0)
		{
			_value=(_value*((1<<_shift)-1)+_target)>>_shift;
		}
	}

	bool Done()
	{	return _value==_target;}

};

template <typename T>
class cLinearInterpolation :
public cInterpolation<T>
{
private:

	T	_step;
	float	_remainingtime;

public:

	bool Setup(T& from,T& to,float time)
	{
		if(time<0)
			return false;

		_remainingtime	=	time;
		_value			=	from;
		_step			=	(to-from)/time;		//	calculate distance per second
		return true;
	}


	//	return true when target is reached or passed
	void Interpolate()
	{
		Interpolate(1.0f);
	}

	void Interpolate(float dtime)
	{
		_remainingtime-=dtime;
		_value+=_step*dtime;
	}

	bool Done()
	{	return (_remainingtime<=0);	}



};

template <typename T>
class cEaseInOutInterpolation
: public cInterpolation<T>
{
private:

	float	_remainingtime;
	float	_totaltime;
	T	_speed;
	T	_acceleration;

public:

	bool Setup(T& from,T& to,float time)
	{
		if(time<=0)
			return false;
		_value	=	from;
		_target	=	to;
		_speed	=	0.0f;

		_acceleration	=	(to-from)/(time*time/4);
		_remainingtime	=	_totaltime	=	time;
		return true;
	}

	void	Interpolate()
	{
		Interpolate(1);
	}

	void	Interpolate(float dtime)
	{
		_remainingtime	-=	dtime;
		if(_remainingtime<_totaltime /2)
		{
			_speed	-=	_acceleration	*	dtime;
		}
		else
		{
			_speed	+=	_acceleration	*	dtime;
		}
		_value	+=	_speed	*	dtime;

	}

	bool Done()
	{	return (_remainingtime<=0);	}
};

/********************************************************************
*	class cBezier													*
*	used to interpolate between 4 points in space..					*
********************************************************************/

class cBezier
{
	int	m_i;			//	current step
	int	m_nSteps;		//	number of steps..
	point3d	m_p[4];		//	the 4 control points
	point3d	m_Q;		//	the current point
	point3d m_dQ;		//	first derivative
	point3d m_ddQ;		//	second derivative
	point3d	m_dddQ;		//	third derivative ( constant )

public:

	cBezier()
	{}

	cBezier(int nSteps,point3d& p1,point3d& p2,point3d& p3,point3d& p4)
	{
		Set(nSteps,p1,p2,p3,p4);
	}

	void Set(int nSteps,point3d& p1,point3d& p2,point3d& p3,point3d& p4)
	{
		m_nSteps	=	nSteps;
		m_p[0]		=	p1;
		m_p[1]		=	p2;
		m_p[2]		=	p3;
		m_p[3]		=	p4;
	}


	//	always start the interpolation with this function
	void Start()
	{
		m_i	=	0;
		float d=1.0f/(m_nSteps-1);
		float d2= d*d;
		float d3= d*d2;

		point4d px(m_p[0].x,m_p[1].x,m_p[2].x,m_p[3].x);
		point4d py(m_p[0].y,m_p[1].y,m_p[2].y,m_p[3].y);
		point4d pz(m_p[0].z,m_p[1].z,m_p[2].z,m_p[3].z);

		point4d cvec[3];
		cvec[0]=px* matrix4::beziermatrix;
		cvec[1]=py* matrix4::beziermatrix;
		cvec[2]=pz* matrix4::beziermatrix;

		m_Q	=	m_p[0];

		//	do the work for each component...
		int i=3;
		while(i--)
		{
			float a=cvec[i].v[0];
			float b=cvec[i].v[1];
			float c=cvec[i].v[2];

			m_dQ.v[i]	=	a*d3+b*d2+c*d;
			m_ddQ.v[i]	=	6*a*d3+2*b*d2;
			m_dddQ.v[i]	=	6*a*d3;
		}
	}

	bool Done()
	{	return !(m_i<m_nSteps);	}

	point3d& GetCurr()
	{
		return m_Q;
	}

	operator point3d&()
	{
		return m_Q;
	}

	void Next()
	{
		m_Q		+=	m_dQ;
		m_dQ	+=	m_ddQ;
		m_ddQ	+=	m_dddQ;
		m_i++;
	}

};


/*---------------------end of class bezier----------------------*/

/********************************************************************
*	class bsptree													*
*	this class can repsresnt a whole whole in 3d					*
* and can be sued to check if a point in this world lies inside		*
*	matter or not													*
*	i still dont fully digest this class and i havent even used it	*
********************************************************************/

using std::vector;

const float percentageToCheck = .1f; // 10%


/**
 * This code expects the set of polygons we're giving it to be
 * closed, forming a continuous skin.  If it's not, weird things
 * may happen.
 */
class cBspTree
{
public:

	// construction/destruction
	cBspTree();
	~cBspTree();

	// we need to handle copying
	cBspTree( const cBspTree &in );
	cBspTree& operator=( const cBspTree &in );

	// add a polygon to the tree
	void AddPolygon( const polygon<point3d>& in );
	void AddPolygonList( vector< polygon<point3d> >& in );

	void TraverseTree( vector< polygon<point3d>* >* polyList, const point3d& loc );

	// will return true idf the two points can see each other
	bool LineOfSight( const point3d& a, const point3d& b );

protected:

private:


	class cNode
	{

		cNode*		m_pFront; // pointer to front subtree
		cNode*		m_pBack; // pointer to back subtree

		polygon<point3d>	m_poly; // 
		plane3d		m_plane;
		bool		m_bIsLeaf;
		bool		m_bIsSolid;
		
		// fixme: should be 2 lists, parallel and anti-parallel
		vector< polygon<point3d> >	m_coplanarList; // list of coplanar polygons

		static int BestIndex( vector< polygon<point3d> >& polyList );

	public:
		cNode( bool bIsSolid ); // leaf constructor
		cNode( const polygon<point3d>& in ); // node constructor
		cNode( vector< polygon<point3d> >& in ); // node constructor
		~cNode();

		// we need to handle copying
		cNode( const cNode &in );
		cNode& operator=( const cNode &in );

		void AddPolygon( const polygon<point3d>& in );

		void TraverseTree( vector< polygon<point3d>* >* polyList, const point3d& loc );

		bool IsLeaf()
		{
			return m_bIsLeaf;
		}

		bool LineOfSight( const point3d& a, const point3d& b );
	};

	cNode* m_pHead; // root node of the tree

};


inline cBspTree::cBspTree( const cBspTree &in )
{
	// clone the tree
	if( in.m_pHead )
		m_pHead = new cNode( *in.m_pHead );
	else
		m_pHead = NULL;
}

inline cBspTree& cBspTree::operator=( const cBspTree &in )
{
	if( &in != this )
	{
		// delete the tree if we have one already
		if( m_pHead )
			delete m_pHead;

		// clone the tree
		if( in.m_pHead )
			m_pHead = new cNode( *in.m_pHead );
		else
			m_pHead = NULL;
	}
		
	return *this;
}


inline cBspTree::cNode::cNode( const cNode &in )
{
	m_poly = in.m_poly;
	m_plane = in.m_plane;
	m_bIsLeaf = in.m_bIsLeaf;
	m_bIsSolid = in.m_bIsSolid;

	// clone the trees
	m_pFront = NULL;
	if( in.m_pFront )
		m_pFront = new cNode( *in.m_pFront );

	m_pBack = NULL;
	if( in.m_pBack )
		m_pBack = new cNode( *in.m_pBack );
}


inline cBspTree::cNode& cBspTree::cNode::operator=( const cNode &in )
{
	if( &in != this )
	{
		// delete the subtrees if we have them already
		if( m_pFront )
			delete m_pFront;
		if( m_pBack )
			delete m_pBack;

		// copy all the data over
		m_poly = in.m_poly;
		m_plane = in.m_plane;
		m_bIsLeaf = in.m_bIsLeaf;
		m_bIsSolid = in.m_bIsSolid;

		// clone the trees
		m_pFront = NULL;
		if( in.m_pFront )
			m_pFront = new cNode( *in.m_pFront );

		m_pBack = NULL;
		if( in.m_pBack )
			m_pBack = new cNode( *in.m_pBack );
	}
	return *this;
}



/*------------------class bsptree declaration------------------*/

typedef tri<unsigned short> stri;
typedef quad<unsigned short> squad;

struct vertex
{
	point3d loc;
	point3d norm;
	float tex[2][2];
};

	
//	for direct3d, this is a lit vertex
struct slvertex
{
	point3d loc;
	unsigned long reserved;
	unsigned long diffuse;
	unsigned long specular;
	float u,v;
};

struct texcoord
{
	float s,t;
};


#endif //_FLULIBMATH_H