/****************************************************************
*	flulibai.cpp												*
*	By Shady El Mously											*
*	Part of the flulib class library							*
*	this file is the implementation of							*
*	the artificial intelligence portion fo the system			*
*includes many intelligent algorithms for steering, motivation	*
****************************************************************/


#include "ai.h"
#include "md2mesh.h"

using namespace std;
#include <string>		//	the neural network needs this...

/****************************************************************
*	class cSmartObject implementation							*
*	basic steering algorithms are implemented here				*
*	a smart object is basically a rigid body					*
*	but it can now manage itself..								*
****************************************************************/

const float cSmartObject::m_PotFuncConstant=0.02f;

//	initialization function
void cSmartObject::Init()
{
	cRigidBody::Init();
	m_Speed		=	0.05f	;
}

//	terminator chasing algorithm ..just move into the other
void cSmartObject::Chase(cRigidBody& other)
{
//	if(hit(other))			return;
	//	generate a direction vector between the 2 objects
	point3d dir=other.m_Bounder->m_Position-m_Bounder->m_Position;
	//	normalize it
	dir.normalize();
	//	now translate it..
	Translate(dir.x*m_Speed,dir.y*m_Speed,dir.z*m_Speed);
}

//	rabbit , just go away from it.. very simple
void cSmartObject::Evade(cRigidBody& other)
{
//	if(hit(other))			return;
	//	generate a diorection vector between the 2 objects
	point3d dir=other.m_Bounder->m_Position-m_Bounder->m_Position;

	//	normalize it
	dir.normalize();

	//	now update the velocity
	Translate(-dir.x*m_Speed,-dir.y*m_Speed,-dir.z*m_Speed);
}

/*-------------end of class smart implementation--------------*/


cCharacter::cCharacter(char* modelFileName,char* TextureFileName)
{
	SAFE_DELETE(m_Mesh);
	m_State = MODEL_IDLE;
	m_Mesh	=	new sMD2Mesh();
	((sMD2Mesh*)m_Mesh)->Load(modelFileName,TextureFileName);
}

// SetState()
// desc: set the model state
int cCharacter::SetState(CharacterState state)
{
	m_State = state;
	return 0;
}

// GetState()
// desc: retrieve the model state
CharacterState cCharacter::GetState()
{
	return m_State;
}


void cCharacter::Draw()
{
	glPushMatrix();											//	save current modelview matrix
	glMultMatrixf(m_WorldMatrix.d);							//	multiply by the world matrix 

	sMD2Mesh*	thismesh=(sMD2Mesh*)m_Mesh;

	switch(m_State)
	{
	case MODEL_IDLE:
		thismesh->Animate(0,39,0.1f);
		break;
	case MODEL_RUN:
		thismesh->Animate(40,46,0.1f);
		break;
	case MODEL_CROUCH:
		thismesh->Animate(136,154,0.1f);
		break;
	case MODEL_JUMP:
		thismesh->Animate(67,73,0.1f);
		break;
	default:
		break;
	}
	glPopMatrix();
}

/********************************************************************
*	helper classes implementation									*
*	here class String, window and all the globaly visible			*
*	classes are implemented											*
********************************************************************/

cFile::cFile() :
	m_fp( NULL )
{
	// nothing to do
}

cFile::~cFile()
{
	// double check to make sure the user closed the file.
	if( m_fp )
	{
		Close();
	}
}

void cFile::Create( const char* filename )
{
	// Create the file handle
	m_fp = fopen( filename, "w+" );

	// make sure everything went well
	if( m_fp == NULL )
		Application()->ReportError("Could Not Create File");

	m_filename = string( filename );
}

void cFile::Append( const char* filename )
{
	// Create the file handle
	m_fp = fopen( filename, "ab" );

	// make sure everything went well
	if( m_fp == NULL )
		Application()->ReportError("Could Not Create File for appending");

	m_filename = string( filename );
}

void cFile::Open( const char* filename )
{
	// Create the file handle
	m_fp = fopen( filename, "r+b" );

	// make sure everything went well
	if( m_fp == NULL )
		Application()->ReportError("Could Not find file");

	m_filename = string( filename );
}


void cFile::Close()
{
	if( m_fp == NULL )
		return; // the file is already closed

	if( 0 != fclose( m_fp ) )
		Application()->ReportError("Error closing file");

	m_fp = NULL;
}



bool cFile::Exists( const char* filename )
{
	// try to open the file.  if we can't, it must not exist
	cFile temp;

	temp.m_fp = fopen( filename, "r" );

	// make sure everything went well
	if( temp.m_fp == NULL )
	{
		return false;
	}
	temp.Close();
	return true;
}


bool cFile::ReadBuff( void* pBuffer, int buffSize )
{
	if( m_fp == NULL )		return false; // error

	if(feof(m_fp) )			return false;

	//int numRead = fread( pBuffer, buffSize, 1, m_fp );
	int numRead = fread( pBuffer, 1, buffSize, m_fp );

	if( numRead != buffSize )
	{
		if( ferror( m_fp ) )
		{
			Application()->ReportError("Error reading data from file");
		}
		else
		{
			int foo = 0;
		}
	}
	return true;
}


void cFile::WriteBuff( void* pBuffer, int buffSize )
{
	if( m_fp == NULL )
		return; // error

	int numWritten = fwrite( pBuffer, buffSize, 1, m_fp );

	if( numWritten != buffSize )
	{
		Application()->ReportError("Error writing data from file");
	}
}


/**
 * Not overrun safe
 */
void cFile::ReadLine( char* pBuffer )
{
	char currChar;
	bool done = false;

	int nRead = 0;
	while( !done )
	{
		if(!ReadBuff( &currChar, 1 ))break;;
		nRead++;
		if( currChar == '\0' || currChar == '\n' )			break;
		*pBuffer++ = currChar;
	}
	// end the string with a \n\0
	*pBuffer++ = '\n';
	*pBuffer++ = '\0';
}

/**
 * Overrun safe
 */
string cFile::ReadLine()
{
	char currChar;
	bool done = false;

	int nRead = 0;

	string out;

	while( !done )
	{
		if(!ReadBuff( &currChar, 1 ))break;;
		nRead++;

		if( currChar == '\0' || currChar == '\n' )
		{
			break;
		}

		out += currChar;
	}
	return out;
}


void cFile::ReadNonCommentedLine( char* pBuffer, char commentChar )
{
	char buff[1024];
	buff[0] = 0;
	while( 1 )
	{
		ReadLine( buff );
		if( buff[0] != commentChar )
			break;
	}
	strcpy( pBuffer, buff );
}


void cFile::TokenizeNextNCLine( queue< string >* pList, char commentChar )
{
	string str;
	while(1)
	{
		str = ReadLine();
		if( str[0] != commentChar )
			break;
	}

	// now curr has our string.
	// first, strip off any comments on the end.
	unsigned int commentLoc = str.find( commentChar, 0);
	if( commentLoc != str.npos )
	{
		str = str.erase( commentLoc );
	}

	char sep[] = " \t\n\r";

	unsigned int tokStart = str.find_first_not_of( (char*)sep, 0 );
	unsigned int tokEnd;
	while( tokStart != str.npos )
	{
		tokEnd = str.find_first_of( (char*)sep, tokStart );
		string token = str.substr( tokStart, tokEnd-tokStart );
		pList->push( token );

		tokStart = str.find_first_not_of( (char*)sep, tokEnd );
	}
}

/*---------------end of class file implementation------------------*/

/****************************************************************
*	class neural network implementation							*
****************************************************************/



cNeuralNet::~cNeuralNet()
{
	// delete all of the nodes. (each node will get it's outgoing edges)
	int numLayers = m_nodes.size();
	for( int i=0; i<numLayers; i++ )
	{
		int layerSize = m_nodes[i].size();
		for( int j=0; j<layerSize; j++ )
		{
			delete m_nodes[i][j];
		}
	}
}


cNeuralNet::cNode* cNeuralNet::FindNode( const char *name)
{
	cNode* pCurr;

	// Search for the node.
	int numLayers = m_nodes.size();
	for( int i=0; i<numLayers; i++ )
	{
		int layerSize = m_nodes[i].size();
		for( int j=0; j<layerSize; j++ )
		{
			pCurr = m_nodes[i][j];
			if( 0 == strcmp( pCurr->GetName(), name ) )
				return pCurr;
		}
	}

	char str[200];
	// didn't contain the node (bad)
	Text()->PutString( "ERROR IN NEURAL NET FILE!\n");
	sprintf(str, "Tried to look for node named [%s]\n", name );
	Text()->PutString(str);
	Text()->PutString( "but couldn't find it!\n");
	Application()->ReportError("Error in Neural Net");
	//exit(0);
	return NULL;
}

void cNeuralNet::Clear()
{
	// Call clear on each of the networks.
	cNode* pCurr;

	int numLayers = m_nodes.size();
	for( int i=0; i<numLayers; i++ )
	{
		int layerSize = m_nodes[i].size();
		for( int j=0; j<layerSize; j++ )
		{
			pCurr = m_nodes[i][j];
			pCurr->Clear();
		}
	}
}

void cNeuralNet::Run()
{
	// Run each layer, running each node in each layer.
	int numLayers = m_nodes.size();
	for( int i=0; i<numLayers; i++ )
	{
		int layerSize = m_nodes[i].size();
		for( int j=0; j<layerSize; j++ )
		{
			m_nodes[i][j]->Run();
		}
	}
}

void cNeuralNet::SendInput( const char *inputTarget, float amt)
{
	// Find the node that we're sending the input to, and send it.
	FindNode( inputTarget )->SendInput( amt );
}

void cNeuralNet::cNode::Clear()
{
	// initial total is set to the bias
	m_total = m_weight; 
}

void cNeuralNet::cNode::Run()
{
	// Compute the transfer function
	float output = CalcOutput();

	// Send it to each of our children
	int size = m_outEdges.size();
	for( int i=0; i< size; i++ )
	{
		m_outEdges[i].Fire( output );
	}
}


void cNeuralNet::cNode::Init( const char *name, float weight) 
{
	m_name = string( name );
	m_weight = weight;
}

float cNeuralNet::cNode::CalcOutput()
{
	// This can use an exponential-type function
	// but for simplicity's sake we're just doing
	// flat yes/no.
	if( m_total >= 1.0f )
		return 1.0f;
	else
		return 0.f;
}

void cNeuralNet::cNode::SendInput(float in)
{
	// just add the input to the total for the network.
	m_total += in;
}

void cNeuralNet::cNode::AddOutEdge(cNode *target, float edgeWeight)
{
	// Create an edge structure
	m_outEdges.push_back( cEdge( this, target, edgeWeight) );
}


void cNeuralNet::Load(cFile &netfile)
{
	char buff[1024];
	char* pCurr;
	char delim[] = " \t\n\r";
	while( 1 )
	{
		netfile.ReadNonCommentedLine( buff, '#' );
		pCurr = strtok( buff, delim );
		if( 0 == strcmp( pCurr, "NN_BEGIN" ) )
		{
			// Read in the # of layers
			int nLayers = atoi( strtok( NULL, delim ) );
			for( int i=0; i<nLayers; i++ )
			{
				// populate the vector-of-vectors with vectors.
				vector< cNeuralNet::cNode* > garbage;
				m_nodes.push_back( garbage );
			}
			
		}
		else if( 0 == strcmp( pCurr, "NN_END" ) )
		{
			// We're done loading the network.  break from the loop
			break;
		}
		else if( 0 == strcmp( pCurr, "NEURON" ) )
		{
			int layerNum = atoi( strtok( NULL, delim ) );
			cNode* pNew = new cNode();
			char* name;
			name = strtok( NULL, delim );
			pCurr = strtok( NULL, delim );
			float val;
			sscanf(pCurr,"%f", &val );
			pNew->Init( name, val );

			// done initializing.  add it to the list
			m_nodes[layerNum].push_back( pNew );
		}
		else if( 0 == strcmp( pCurr, "INPUT" ) )
		{
			string nodeName = string( strtok( NULL, delim ) );
			
			char* pRestOfString = strtok( NULL, "\n\r" );

			char* pStrStart = strchr( pRestOfString, '\"' );
			char* pStrEnd = strrchr( pRestOfString, '\"' );

			assert( pStrStart && pStrEnd && (pStrStart!=pStrEnd) );

			m_inputs[ nodeName ] = string( pStrStart+1, pStrEnd );
		}
		else if( 0 == strcmp( pCurr, "OUTPUT" ) )
		{
			string nodeName = string( strtok( NULL, delim ) );
			
			char* pRestOfString = strtok( NULL, "\n\r" );

			char* pStrStart = strchr( pRestOfString, '\"' );
			char* pStrEnd = strrchr( pRestOfString, '\"' );

			assert( pStrStart && pStrEnd && (pStrStart!=pStrEnd) );

			m_outputs[ nodeName ] = string( pStrStart+1, pStrEnd );
		}
		else if( 0 == strcmp( pCurr, "DEFAULTOUT" ) )
		{
			char* pRestOfString = strtok( NULL, "\n\r" );

			char* pStrStart = strchr( pRestOfString, '\"' );
			char* pStrEnd = strrchr( pRestOfString, '\"' );

			assert( pStrStart && pStrEnd && (pStrStart!=pStrEnd) );

			m_defaultOutput = string( pStrStart+1, pStrEnd );
		}
		else if( 0 == strcmp( pCurr, "EDGE" ) )
		{
			char* src;
			char* dst;
			src = strtok( NULL, " \t\n\r" );
			dst = strtok( NULL, " \t\n\r" );

			pCurr = strtok( NULL, " \t\n\r" );
			float val;
			sscanf(pCurr,"%f", &val );

			FindNode( src )->AddOutEdge( FindNode( dst ), val );
		}

	}

}

cNeuralNet::cEdge::cEdge( cNode *pSrc, cNode *pDest, float weight)
: m_pSrc( pSrc )
, m_pDest( pDest )
, m_weight( weight)
{
	// all done.
}

void cNeuralNet::cEdge::Fire( float amount )
{
	// Send the signal, multiplied by the weight, 
	// to the destination node.
	m_pDest->SendInput( amount * m_weight );
}

//==========--------------------------  

void cNeuralNet::ListInputs()
{
	char	str[200];
	Text()->PutString("\n");
	Text()->PutString("Neural Net Inputs:\n");
	Text()->PutString("------------------\n");

	map<string, string>::iterator iter;
	for( iter = m_inputs.begin(); iter != m_inputs.end(); iter++ )
	{
		sprintf(str,"%s\n", (*iter).second.c_str() );
		Text()->PutString(str);
	}
}


void cNeuralNet::GetInputs()
{
	char str[200];
	Text()->PutString("\n");
	Text()->PutString("Enter Inputs:\n");
	Text()->PutString("-------------\n");

	map<string, string>::iterator iter;
	for( iter = m_inputs.begin(); iter != m_inputs.end(); iter++ )
	{
		sprintf(str,"Enter floating point input for [%s]\n", (*iter).second.c_str() );
		Text()->PutString(str);
		char buff[80];
		Text()->GetString( buff );
		float value = atof( buff );

		cNode* pNode = FindNode( (*iter).first.c_str() );

		pNode->SendInput( value );
	}
}


string cNeuralNet::GetOutput()
{
	char str[200];
	map<string, string>::iterator iter;

	float fBest = 1.f;
	map<string, string>::iterator best = m_outputs.end();

	for( iter = m_outputs.begin(); iter != m_outputs.end(); iter++ )
	{
		// check the output.  is it the best?
		cNode* pNode = FindNode( (*iter).first.c_str() );
		if( pNode->GetTotal() > fBest )
		{
			// new best.
			fBest = pNode->GetTotal();
			best = iter;
			sprintf(str,"new total: %f\n", fBest );
			Text()->PutString(str);
		}
	}
	if( best == m_outputs.end() )
	{
		return m_defaultOutput;
	}
	else
	{
		return (*best).second.c_str();
	}
}

/*---------end of class neural network implementation----------*/