#ifndef _BASE_H
#define _BASE_H

/********************************************************************
*	base.h															*
*	By Shady El Mously												*
*	This file contains the base and most useful						*
*	classes that will be used everywhere throughout the library		*
********************************************************************/

//	disable warning 4786
#pragma warning( disable : 4786 )

//	Very Important General Macros to be used every where
#define KEYDOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEYUP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)
#define KEYHIT(c)	KEYDOWN(c)&&Application()->m_KeyPressed==FALSE
#define SAFE_RELEASE(x) if (x) {x->Release(); x=NULL;}
#define SAFE_DELETE(p)  if(p) {delete (p);(p)=0; }
#define SAFE_FREE(p)	if(p) {delete [](p);(p)=0; }

//	Say NO to MFCz !!
#define WIN32_LEAN_AND_MEAN

//	Windows API Includes
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>

//	STL stuff
#include <cassert>			//	used in Singletton
#include <queue>			//	for class cFile

#include <fstream>
using namespace std;


using std::string;			//	use to inherit and create a string class
using std::vector;			//	used for creating infinite arrays
using std::queue;			//	for the class cFile

//	a typedef for a VoidFunction variable type
typedef void (__cdecl *VoidFunction)();

//	this is an empty function to be assigned initially for VoidFunction variables
inline void DoNothing(void){}


/********************************************************************
*	Helper classes													*
*	those classes are singleton,String,error,vector2d				*
*	and window														*
*	they repsent many useful object used later..					*
********************************************************************/


/********************************************************************
*	class singleton													*
*	this magical templatized class from game programming gems		*
*	this class will be a base class when i need to create			*
*	any singleton class, i just have to inherit from it				*
*	it will create a static pointer to an object from the same		*
*	type of its class which i can create and destroy any time		*
********************************************************************/
template <typename T> class singleton
{
	static T* ms_Singleton;

public:

	singleton(void)
	{
		assert(!ms_Singleton);
		int offset=(int)(T*)1-(int)(singleton<T>*)(T*)1;
		ms_Singleton=(T*)((int)this+offset);
	}

	~singleton(void)
	{assert(ms_Singleton);ms_Singleton=0;}

	static T& GetSingleton(void)
	{assert(ms_Singleton);return (*ms_Singleton);}
	
	static T*& GetSingletonPtr(void)
	{return ms_Singleton;}

};
//	and this is the static pointer to object
template <typename T> T* singleton<T>::ms_Singleton=0;



/*---------------------end of class singleton----------------------*/
/********************************************************************
*	class error														*
*	By Shady el mawsselly											*
*	this class is used for error handling							*
*	an object of the type of this class								*
*	will be thrown in the case of an error							*
*	an object oif this class will be thrown with an error msg		*
*	it just contain a string with the message specified				*
********************************************************************/
class cError
{
public:
	char * m_Message;

	cError(char *ErrorMessage)
	{m_Message=ErrorMessage;}
};
/*---------------------end of class error--------------------------*/


/********************************************************************
*	class cWindow													*
*	By SHady el Mously												*
*	this class encapsulates the functionlity of a window			*
*	windows can be created using this class							*
*	and they may be created with a default message handler			*
*	or a defined message handler									*
*	This Code Try To encapsulate as much as possible				*
*	The internal complexities associated with creating a window		*
*	and changing its settings										*
*	and handling windows messages									*
********************************************************************/
class cWindow
{
private:

	HWND		m_Hwnd			;		//	this is a handle to the main window object
	LPSTR		m_ClassName		;		//	name of class to be registered in the registry
	WNDPROC		m_fWindowProc	;		//	a message handler

	//	this class abstract a window...
	//	first this is a default message handler.. 
	//	very basic, it will used by default 
	//	if no the message specific handler is sent
	static HRESULT CALLBACK DefaultWinProc(HWND hwnd,UINT message,WPARAM wparam,LPARAM lparam);

	// fill a window class and register it...
	void Register(WNDPROC MessageHandler);

public:

	//		a default window constructor
	cWindow();
	//	initializing a window
	//	set a default class name of WINCLASS
	//	u must send the hinstance for which the window will be created
	//	also a message handler function may be sent here
	cWindow(LPSTR ClassName,WNDPROC WinProc=DefaultWinProc);
	// upon destruction.. unregister the class and destroy the window
	~cWindow()
	{	DestroyWindow(m_Hwnd);	}

	//	This will create a window and shows it
	//	i must send the width and height of the client rect required
	//	and the window style and label get default value if
	//	nothing was sent
	void Create(int width,int height,
				DWORD WinStyle=WS_VISIBLE|WS_SYSMENU,
				LPSTR label="");

	//this will set the window label
	void SetLabel(LPSTR label)
	{SetWindowText(m_Hwnd,label);}

	//	gets the handle to the window
	HWND GetHwnd()
	{	return m_Hwnd;	}
};	//	end of class Window
/*-----------------------end of class cWindow----------------------*/




/********************************************************************
*	Class thread													*
*	By Shady El Mously												*
*	this class is used to create any threas in the application		*
*	such as the Shell and a thread to hide the silly windows		*
*	messages														*
********************************************************************/

class cThread
{
private:


	HANDLE		m_ThreadHandle;			//	handle to the thread when it runs.
	DWORD		m_ThreadID;				//	id of the thread
	static int	s_NumActiveThreads;		//	this is the active number of threads
	bool		m_bInThread;			//	is the thread running ? private  coz only base class can modify it..
										//	it is a read only variable for child classes through a function (InThread)

	CRITICAL_SECTION m_CriticalSection;	//	a crtitical section struct, used for handling unsafe sections of code

	
	static void StartAsThread(cThread* thread)
	{
		try{thread->Run();}
		catch(cError&){}
		thread->StopThread();
	}	//	static function to be called for each thred..

	// this is the actual thread function of the console.., must look like this..
	static DWORD WINAPI ThreadConsole(LPVOID pThread)
	{
		StartAsThread((cThread*)pThread);
		return ((DWORD)0);
	}



	//	the default commands remain here
	//	this will stop the threaded module
	void StopThread();

protected:

	//	constructor and destructor are made protected to prevent instances from the class..
	//	i should inheret always...
	cThread(): m_bInThread(false)	
	{
		InitializeCriticalSection(&m_CriticalSection);
	}

	virtual ~cThread()
	{
		DeleteCriticalSection(&m_CriticalSection);
		StopThread(); 	
	}


public:

	//	this will start the shell as a thread (independant) function
	void Start(void);

	//	this will main thread function, i can inheret and rewrite it
	//	it is pure virtual so it must be written in all children classes
	virtual void Run(void){}

	//	to check if the thread is on/off..
	bool InThread(){return m_bInThread;}

	//	call this before doing anything critical
	void EnterCriticalSection()
	{
		::EnterCriticalSection(&m_CriticalSection);
	}

	//	call this when done with critical section
	void LeaveCriticalSection()
	{
		::LeaveCriticalSection(&m_CriticalSection);
	}
};
/*------------------end of class thread declaration----------------*/






/********************************************************************
*	class  cString													*
*	By Shady el Mously												*
*	a simple string class.. inhertied from the std::string			*
*	has support for splitting strings and reading them from files	*
*	which will be useful in text processing later					*
********************************************************************/
class cString : public string
{
public:
	//	the static tokens
	static vector<cString> arg_v;
	//	empty constructor
	cString(){}
	//	constructor with a c string
	cString(char * str) : string(str){}
	//	operator=
	operator=(char *str){string::operator=(str);}
	//	check strings equlity
	bool operator==(char *str){return strcmp(c_str(),str)==0 ? true : false ;}
	//	erase last charcater in the string
	void operator--(int){erase(size()-1,size()-1);}
	//	typecasting operators
	operator const char *(){return c_str();}
	operator char *(){return (char*)c_str();}
	void Split(){Split((char*)c_str());}
	
	//	this will split the string into the static tokens
	static void Split(char * command);
	static void ReadLine(FILE * file,char * string);
	static void ReadTokenLine(FILE * file);

};
/*------------------end of class cString---------------------------*/


/********************************************************************
*	Struct Screen Mode												*
*	used to hold information about a screen mode.					*
********************************************************************/
struct sScreenMode
{
	int Width;
	int Height;
	int BitDepth;
};
/*-------------------end if struct screenmode----------------------*/

/********************************************************************
*	struct vector2d													*
*	this is a trivially simple class represeting a 2 dimensional	*
*	integer value, it will be very helpful later					*
********************************************************************/
typedef struct vector2d
{
	union
	{
		int x;
		int dx;
		int width;
	};
	union
	{
		int y;
		int dy;
		int height;
	};
}point2d;
/*----------end of struct vector2d---------------------------------*/


/********************************************************************
*	struct mouseinfo												*
*	a singleton class represeting the simple object					*
*	holding information about the mouse								*
*	which is the position and the buttons state						*
********************************************************************/
struct sMouseInfo: public singleton<sMouseInfo>
{
	//	this is the only one
	point2d		m_Loc;
	bool		m_bLButtonDown	;		//	left	button is	down
	bool		m_bRButtonDown	;		//	right	button is	down
	bool		m_bMButtonDown	;		//	middle	button is	down
	float		m_DeltaX		;		//	deltax of the mouse from last frame ,changes every frame
	float		m_DeltaY		;		//	deltay of the mouse from last frame ,changes every frame
	float		m_DeltaZ		;		//	deltaz of the mouse from last frame ,changes every frame
	float		m_Xpos			;		//	3d xposition
	float		m_Ypos			;		//	3d yposition
	float		m_Zpos			;		//	3d zposition


	sMouseInfo()
	{
		memset(this,0,sizeof(*this));
	}
};


//	access all mouse data from this fucntion.
inline sMouseInfo	*& Mouse(){return sMouseInfo::GetSingletonPtr();}
/*---------------------end of class mouseinfo----------------------*/


/********************************************************************
*	enumeration for languages										*
********************************************************************/

enum eLanguage { ENGLISH , ARABIC , FRENCH };

/*-------------------end of languages enumeration------------------*/


#endif // _FLULIBMISC_H