#include "converters.h"

#define D3D_OVERLOADS
#include <d3d.h>
#include <d3dtypes.h>
#include "d3dmath.h"


//	this function will take CD3DFileObject, and extract the mesh of all 
//	its children and next meshes into a single big mesh
//	this is recursive function
//	i must use a single array of vertices and a single array of indexes, 
//	the best way to do this recusrive algorithm is using a 2 STL vectors
//	a vector for vertices
//	and a vector for indeces
//	the vectors are sent by reference so recursive calls will continue
//	pushing back into the same vectors sent first
void GetAllMeshes(
				  CD3DFileObject*			mesh	,
				  vector	<D3DVERTEX>&	verts	,
				  vector	<WORD>&			indeces	,
				  matrix4	CurrentMatrix	=	matrix4::Identity	)
{
	//	 read a new mesh in those local arrays
	D3DVERTEX *		newverts;
	WORD *			newindeces;
	DWORD			newnumverts;
	DWORD			newnumindeces;

	//	read the mesh of the current object into the arrays
	mesh->GetMeshGeometry(&newverts,&newnumverts,&newindeces,&newnumindeces);

	//	make the stl vectors ready to accept new data
	verts.reserve(newnumverts);
	indeces.reserve(newnumindeces);

	int vertcount=verts.size();

	//	before pushing into the big mesh, i should transform each point
	//	using the matrix of this object

	//	a working point to use while transforming
	point3d workp;
	matrix4 NewMatrix;

	//	get the object matrix into the matrix4
	memcpy(&NewMatrix,&mesh->m_mat,sizeof(matrix4));

	//	conactenate it with the previous one if sent,
	//	it is identity if nothing is sent
	NewMatrix=NewMatrix*CurrentMatrix;


	//	save here the original vertex value
	D3DVERTEX	originalvertex;

	//	now let each vertex pass by the matrix and be transformed,
	//	then pushed back in into the STL vector
	for (int i=0;i<newnumverts;i++)
	{
		originalvertex	=	newverts[i];

		workp.x=newverts[i].x;
		workp.y=newverts[i].y;
		workp.z=-newverts[i].z;
		workp=workp*NewMatrix;
		newverts[i].x=-workp.x;
		newverts[i].y=workp.y;
		newverts[i].z=workp.z;
		verts.push_back(newverts[i]);

		//	fix this back...
		newverts[i]	=	originalvertex;	
		
	}

	//	then push back the indices7
	//	add the offset of the total number of points inside the STL vector
	for (i=0;i<newnumindeces;i++)
	{
		indeces.push_back(newindeces[i]+vertcount);
	}

	//	are there any children? use the new matrix and recurse
	if(mesh->m_pChild)
		GetAllMeshes(mesh->m_pChild,verts,indeces,NewMatrix);

	//	any next meshes?..get using the current matrix and recurse
	if(mesh->m_pNext)
		GetAllMeshes(mesh->m_pNext,verts,indeces,CurrentMatrix);
}



//	this will get the average vertex of a mesh..
//	this could be subtracted from all vertices to take everything about he origin
point3d GetAverageVertex(vector<D3DVERTEX>& Verts)
{
	// get center of the object and adjust it around it
	point3d AverageVertex(0,0,0);//

	// add all the vertices
	for(int i=0;i<Verts.size();i++)
		AverageVertex+=point3d(Verts[i].x,Verts[i].y,Verts[i].z);

	// and get their average point
	AverageVertex/=Verts.size();

	return AverageVertex;
};


//	thos one gets the maximum vertex..
//	used to scale objects ..
point3d GetMaxVertex(vector<D3DVERTEX>& Verts)
{
	//	this will make sure the object is inside a unit circle in its space
	point3d	MaxVertex(0,0,0);

	// now find the farthest point in the object... to scale by 1/ it
	point3d tp;

	for(int i=0;i<Verts.size();i++)
	{
		tp.assign(Verts[i].x,Verts[i].y,Verts[i].z);
		if (tp.mag()>MaxVertex.mag())
			MaxVertex=tp;	
	}
	return MaxVertex;
}


//	save single mesh data in a binary file as pure binary data
void SaveSingleMeshAsBinary(
					FILE*		output,
					D3DVERTEX * pNewVerts,
					WORD *		pNewIndeces,
					DWORD		NumVerts,
					DWORD		NumIndeces,
					bool		revcull,
					int			NumVertexFields,
					char*		MeshName,
					char*		TextureFileName
					)
{
	int i;

	//	write the mesh header
	fprintf(output,"%s %u %u %u %u %s\n",MeshName,NumVertexFields,0,NumVerts,NumIndeces/3,TextureFileName);

	//	write vertices
	for(i=0;i<NumVerts;i++)
	{
		switch(NumVertexFields)
		{
		case 3:
			
			//	first 3 values are always guarantueed
			fwrite(&pNewVerts[i].x,sizeof(float),1,output);
			fwrite(&pNewVerts[i].y,sizeof(float),1,output);
			fwrite(&pNewVerts[i].z,sizeof(float),1,output);
			
			break;
		case 5:

			//	first 3 values are always guarantueed
			fwrite(&pNewVerts[i].x,sizeof(float),1,output);
			fwrite(&pNewVerts[i].y,sizeof(float),1,output);
			fwrite(&pNewVerts[i].z,sizeof(float),1,output);

			fwrite(&pNewVerts[i].tu,sizeof(float),1,output);
			fwrite(&pNewVerts[i].tv,sizeof(float),1,output);
				
			break;

		case 6:
			//	first 3 values are always guarantueed
			fwrite(&pNewVerts[i].x,sizeof(float),1,output);
			fwrite(&pNewVerts[i].y,sizeof(float),1,output);
			fwrite(&pNewVerts[i].z,sizeof(float),1,output);

			fwrite(&pNewVerts[i].nx,sizeof(float),1,output);
			fwrite(&pNewVerts[i].ny,sizeof(float),1,output);
			fwrite(&pNewVerts[i].nz,sizeof(float),1,output);

			break;

		case 8:
			
			//	first 3 values are always guarantueed
			fwrite(&pNewVerts[i].x,sizeof(float),1,output);
			fwrite(&pNewVerts[i].y,sizeof(float),1,output);
			fwrite(&pNewVerts[i].z,sizeof(float),1,output);

			fwrite(&pNewVerts[i].nx,sizeof(float),1,output);
			fwrite(&pNewVerts[i].ny,sizeof(float),1,output);
			fwrite(&pNewVerts[i].nz,sizeof(float),1,output);

			fwrite(&pNewVerts[i].tu,sizeof(float),1,output);
			fwrite(&pNewVerts[i].tv,sizeof(float),1,output);
		
			break;
	
		default:
			break;
		}	//	end of switch
	}//	end of vertices loop

	for(i=0;i<NumIndeces;i=i+3)
	{
		if(revcull)
		{
			fwrite(&pNewIndeces[i+2],sizeof(unsigned short),1,output);
			fwrite(&pNewIndeces[i+1],sizeof(unsigned short),1,output);
			fwrite(&pNewIndeces[i+0],sizeof(unsigned short),1,output);
		}
		else
		{
			fwrite(&pNewIndeces[i+0],sizeof(unsigned short),1,output);
			fwrite(&pNewIndeces[i+1],sizeof(unsigned short),1,output);
			fwrite(&pNewIndeces[i+2],sizeof(unsigned short),1,output);
		}
	}
}

//	save single mesh data in a text file as pure text data
void SaveSingleMeshAsText(
					FILE*		output,
					D3DVERTEX * pNewVerts,
					WORD *		pNewIndeces,
					DWORD		NumVerts,
					DWORD		NumIndeces,
					bool		revcull,
					int			NumVertexFields,
					char*		MeshName,
					char*		TextureFileName
					)
{
	int i;

	//	write the mesh header
	fprintf(output,"%s %u %u %u %u %s\n",MeshName,NumVertexFields,0,NumVerts,NumIndeces/3,TextureFileName);

	// write vertices
	switch(NumVertexFields)
	{
	case 3:

		for(i=0;i<NumVerts;i++)fprintf(output,"%f %f %f\n",pNewVerts[i].x,pNewVerts[i].y,pNewVerts[i].z);
		break;

	case 5:
		for(i=0;i<NumVerts;i++)fprintf(output,"%f %f %f %f %f\n",pNewVerts[i].x,pNewVerts[i].y,pNewVerts[i].z,pNewVerts[i].tu,pNewVerts[i].tv);
		break;

	case 6:
		for(i=0;i<NumVerts;i++)fprintf(output,"%f %f %f %f %f %f\n",pNewVerts[i].x,pNewVerts[i].y,pNewVerts[i].z,pNewVerts[i].nx,pNewVerts[i].ny,pNewVerts[i].nz);
		break;

	case 8:
		for(i=0;i<NumVerts;i++)fprintf(output,"%f %f %f %f %f %f %f %f\n",pNewVerts[i].x,pNewVerts[i].y,pNewVerts[i].z,pNewVerts[i].nx,pNewVerts[i].ny,pNewVerts[i].nz,pNewVerts[i].tu,pNewVerts[i].tv);
		break;

	default:break;
	}

	//	then write the triangle indices
	if(revcull)
	{
		for(i=0;i<NumIndeces;i=i+3)
			fprintf(output,"%u %u %u\n",pNewIndeces[i+2],pNewIndeces[i+1],pNewIndeces[i+0]);
	}
	else
	{
		for(i=0;i<NumIndeces;i=i+3)
			fprintf(output,"%u %u %u\n",pNewIndeces[i+0],pNewIndeces[i+1],pNewIndeces[i+2]);
	}
}

//	this function will convert an x file into an o3d file
//	it can make both binary and text formats for.o3d files..
//	it also can reverse the culling order if required
//	and in text mode, it can only generate the reuired info
//	if the filename is terminated by a letter t
//	then generate a text file
void x2o3d(char * infile,char * outfile,bool revcull,int numfields,point3d axeloc,char* Texfturefilename)
{
	int i;// looping 

	bool binary=true;

	if(outfile[strlen(outfile)-1]=='t')
		binary=false;
	else
		binary=true;

	// object file to convert
	CD3DFile test;

	//	load the xfile int the x fileobject..
	test.Load(infile);
		

	//	data of the mesh
	ULONG numverts=0;					// number of vertices
	ULONG numtriangles=0;				// number of triangles * 3

	vector<D3DVERTEX>	tverts;		//	this vector will contain all the vertices
	vector<WORD>		tris;		//	all triangle indices

	//	get everything in one mesh.. no hierachical info
	GetAllMeshes(test.m_pRoot,tverts,tris);

	//	use vector sizes to adjust the numverts and numtrinagles
	numverts=tverts.size();
	numtriangles=tris.size();

	// get center of the object and adjust it around it
	point3d AvVertex(0,0,0);//

	//	get the average vertex
	AvVertex=GetAverageVertex(tverts);

	// now subtract it from all the vertices
	for(i=0;i<numverts;i++)
	{
		tverts[i].x-=AvVertex.x;
		tverts[i].y-=AvVertex.y;
		tverts[i].z-=AvVertex.z;
	}
	//	this will make sure the object is inside a unit circle in its space
	point3d	MaxVertex=GetMaxVertex(tverts);

	// we have the biggest point, now divide all by it
	for(i=0;i<numverts;i++)
	{
		tverts[i].x/=MaxVertex.mag();
		tverts[i].y/=MaxVertex.mag();
		tverts[i].z/=MaxVertex.mag();
	}

	// and finally add the center axe location that was sent
	for(i=0;i<numverts;i++)
	{
		tverts[i].x+=axeloc.x;
		tverts[i].y+=axeloc.y;
		tverts[i].z+=axeloc.z;
	}

	// the o3d file to be created
	FILE * file3d;


	// write the first line
	// name of object, number of vertex info filed ( 3 or 5 or 8 or 6)
	// numver of vertices
	// number of triangles

	if(binary)
	{
		// create and open a o3d file
		file3d=fopen(outfile,"wb");
		SaveSingleMeshAsBinary(file3d,&tverts[0],&tris[0],numverts,numtriangles,revcull,numfields,outfile,Texfturefilename);

	}//end of binary mesh generation
	else
	{
		// create and open a o3d file
		file3d=fopen(outfile,"wt");
		SaveSingleMeshAsText(file3d,&tverts[0],&tris[0],numverts,numtriangles,revcull,numfields,outfile,Texfturefilename);
	}//	end of text file generation
	// file is done.. close the buffer and clean everything before leaving here
	fclose(file3d);
}



//	recursive function that will mov through a hierachy and build the 
//	3d file describing it
void WriteHierachicalMesh(CD3DFileObject*	mesh,FILE* output,bool revcull,bool includenormals,bool binary)
{
	int i;
	//	 read a new mesh in those local arrays
	D3DMATRIX		CurrentMatrix;

	D3DVERTEX * pNewVerts;
	WORD *		pNewIndeces;

	DWORD		NumVerts;
	DWORD		NumIndeces;

	//	number of vertex fields
	int			NumVertexFields	=	3;

	//	read the mesh of the current object into the arrays
	mesh->GetMeshGeometry(&pNewVerts,&NumVerts,&pNewIndeces,&NumIndeces);

	//	and read the matrix of the current mesh
	//	get the object matrix into the matrix4
	memcpy(&CurrentMatrix,mesh->GetMatrix(),sizeof(matrix4));


	//	texturefilename should be a bmp file
	char TextureFileName[80];

	sprintf(TextureFileName,"%s",mesh->m_Material->m_strTexture);

	//	now make sure last 3 letters are "bmp"
	if(strlen(mesh->m_Material->m_strTexture)>3)
	{
		strcpy(&TextureFileName[strlen(TextureFileName)-3],"bmp");
		NumVertexFields	+=	2;
	}

	if(includenormals)
		NumVertexFields	+=	3;

	for(i=0;i<NumVerts;i++)
	{
		//	invert all z values
		pNewVerts[i].z=-pNewVerts[i].z;
	}

	D3DMATRIX& NewMatrix=CurrentMatrix;

	NewMatrix._13=NewMatrix._13;
	NewMatrix._23=NewMatrix._23;
	NewMatrix._33=NewMatrix._33;
	NewMatrix._43=NewMatrix._43;

	bool	hasmaterial=false;
	bool	hasalpha=false;;

	if(mesh->m_bHasAlpha)	hasalpha=true;

	if(
		mesh->m_Material->m_mtrl.diffuse.r	||
		mesh->m_Material->m_mtrl.diffuse.g	||
		mesh->m_Material->m_mtrl.diffuse.b	||
		mesh->m_Material->m_mtrl.ambient.r	||
		mesh->m_Material->m_mtrl.ambient.g	||
		mesh->m_Material->m_mtrl.ambient.b	||
		mesh->m_Material->m_mtrl.specular.r	||
		mesh->m_Material->m_mtrl.specular.g	||
		mesh->m_Material->m_mtrl.specular.b
		)
		hasmaterial=true;


	if(binary)
	{
		SaveSingleMeshAsBinary(output,pNewVerts,pNewIndeces,NumVerts,NumIndeces,revcull,NumVertexFields,mesh->m_strName,TextureFileName);
		//write matrices
		for(i=0;i<4;i++)
		{
			fwrite(&CurrentMatrix.m[i][0],sizeof(float),1,output);
			fwrite(&CurrentMatrix.m[i][1],sizeof(float),1,output);
			fwrite(&CurrentMatrix.m[i][2],sizeof(float),1,output);
			fwrite(&CurrentMatrix.m[i][3],sizeof(float),1,output);
		}
		//	write material
		if(hasmaterial)
		{
			fprintf(output,"y");

			//	write an extra yes if the mesh has alpha blending
			if(hasalpha)
				fprintf(output," y");

			fprintf(output,"\n");

			//	write material is there were vertices only
			if(NumVerts>0)
			{
				fwrite(&mesh->m_Material->m_mtrl,sizeof(D3DMATERIAL7),1,output);
			}
		}
		else
			fprintf(output,"n\n");

	}
	else
	{
		SaveSingleMeshAsText(output,pNewVerts,pNewIndeces,NumVerts,NumIndeces,revcull,NumVertexFields,mesh->m_strName,TextureFileName);
		//	write matrices
		for(i=0;i<4;i++)
		{
			fprintf(output,"%f %f %f %f\n",CurrentMatrix.m[i][0],CurrentMatrix.m[i][1],CurrentMatrix.m[i][2],CurrentMatrix.m[i][3]);
		}

		if(hasmaterial)
		{
			fprintf(output,"y");

			//	write an extra yes if the mesh has alpha blending
			if(hasalpha)
				fprintf(output," y");

			fprintf(output,"\n");

			//	write material is there were vertices only
			if(NumVerts>0)
			{
				fprintf(output,"%f %f %f %f\n",mesh->m_Material->m_mtrl.diffuse.r,mesh->m_Material->m_mtrl.diffuse.g,mesh->m_Material->m_mtrl.diffuse.b,mesh->m_Material->m_mtrl.diffuse.a);
				fprintf(output,"%f %f %f %f\n",mesh->m_Material->m_mtrl.ambient.r,mesh->m_Material->m_mtrl.ambient.g,mesh->m_Material->m_mtrl.ambient.b,mesh->m_Material->m_mtrl.ambient.b);
				fprintf(output,"%f %f %f %f\n",mesh->m_Material->m_mtrl.specular.r,mesh->m_Material->m_mtrl.specular.g,mesh->m_Material->m_mtrl.specular.b,mesh->m_Material->m_mtrl.specular.a);
				fprintf(output,"%f %f %f %f\n",mesh->m_Material->m_mtrl.emissive.r,mesh->m_Material->m_mtrl.emissive.g,mesh->m_Material->m_mtrl.emissive.b,mesh->m_Material->m_mtrl.emissive.a);
				fprintf(output,"%f\n",mesh->m_Material->m_mtrl.power);
			}
		}
		else
			fprintf(output,"n\n");
	}


	//	before getting deeper into the hierrachy
	//	i should know if the coming is a next or is a child 

	//	0 means no child nor next
	if(!mesh->m_pChild&&!mesh->m_pNext)
	{
		fprintf(output,"0 (no further hierarchical info)\n");
	}
	else
	//	1 means a child only
	if(mesh->m_pChild&&!mesh->m_pNext)
	{
		fprintf(output,"1 (mesh has a child)\n");
	}
	else	
	//	2 means a next only
	if(mesh->m_pNext&&!mesh->m_pChild)
	{
		fprintf(output,"2 (mesh has a next mesh)\n");
	}
	else
	//	and 3 means both next and child
	if(mesh->m_pNext&&mesh->m_pChild)
	{
		fprintf(output,"3 (mesh have a child and a next)\n");
	}


	//	are there any children? use the new matrix and recurse
	if(mesh->m_pChild)
		WriteHierachicalMesh(mesh->m_pChild,output,revcull,includenormals,binary);

	//	any next meshes?..get using the current matrix and recurse
	if(mesh->m_pNext)
		WriteHierachicalMesh(mesh->m_pNext,output,revcull,includenormals,binary);
}


void ScaleHierarchy(
					CD3DFileObject*	mesh	,
					float factor
					)
{
	//	 read a new mesh in those local arrays
	D3DVERTEX *		newverts;
	WORD *			newindeces;
	DWORD			newnumverts;
	DWORD			newnumindeces;

	int i;

	//	first get all current data...

	//	read the mesh of the current object into the arrays
	mesh->GetMeshGeometry(&newverts,&newnumverts,&newindeces,&newnumindeces);


	//	loop on all vertices factoring them
	for(i=0;i<newnumverts;i++)
	{
		newverts[i].x/=factor;
		newverts[i].y/=factor;
		newverts[i].z/=factor;
	}

	//	also factor the matrix..
	mesh->m_mat._41/=factor;
	mesh->m_mat._42/=factor;
	mesh->m_mat._43/=factor;


	//	if there is anext.. recurse
	if(mesh->m_pNext)
		ScaleHierarchy(mesh->m_pNext,factor);

	// if there is a child.. recurse..
	if(mesh->m_pChild)
		ScaleHierarchy(mesh->m_pChild,factor);
}




//	creates a hierarchycal object file..
//	still under test...
void x2h3d(char* xfilename,char* h3dfilename,bool revcull,bool normals)
{
	// object file to convert
	CD3DFile test;

	//	load the xfile int the x fileobject..
	test.Load(xfilename);

	//	here is should adjust the geometry before beggining to create the file..
	vector<D3DVERTEX>	AllVertices;		//	this vector will contain all the vertices
	vector<WORD>		AllIndeces;		//	all triangle indices

	//	get everything in one mesh.. no hierachical info
	GetAllMeshes(test.m_pRoot,AllVertices,AllIndeces);

	//	then find the points to adjust with.. theavergae vertex first...
	point3d AverageVertex=GetAverageVertex(AllVertices);

	//	now that i have the avergae point i have to modify the d3dfileobject..
	//	by finding the first matrix with a non identity matrix
	//	and translating back this matrix with this vector
	test.m_pRoot->m_mat._41+=AverageVertex.x;
	test.m_pRoot->m_mat._42-=AverageVertex.y;
	test.m_pRoot->m_mat._43-=AverageVertex.z;


	//	now find the maximum point..
	//	this should be used to scale everything in the object

	//	get everything in one mesh.. no hierachical info
	GetAllMeshes(test.m_pRoot,AllVertices,AllIndeces);

	point3d MaxPoint=GetMaxVertex(AllVertices);

	ScaleHierarchy(test.m_pRoot,MaxPoint.mag());


	//	now that the file is loaded
	//	here is what i should do while reading the object loaded..
	//	i should start from the root mesh first..
	FILE * tfile;

	bool binary=true;

	if(h3dfilename[strlen(h3dfilename)-1]=='t')
	{
		tfile=fopen(h3dfilename,"w+t");
		binary=false;
	}
	else
	{
		tfile=fopen(h3dfilename,"w+b");
		binary=true;
	}
	
	WriteHierachicalMesh(test.m_pRoot,tfile,!revcull,normals,binary);
	fclose(tfile);
}