#ifndef _GRAPHICS_H
#define _GRAPHICS_H

/****************************************************************
*	Graphics.h													*
*	Main Graphics Subsystem header for the interface class for	*
*	Graphics													*
****************************************************************/

#include "base.h"		//	to get the singletton class
#include "3dmath.h"		//	3d math classes


/****************************************************************
*	3D State Machine flags										*
*	could be get/set by the graphics layer						*
****************************************************************/

/*---------------end of 3d states enumeration------------------*/

/****************************************************************
*	The Graphics family of classes								*
*	the base class is called cGraphicsSubSystem,				*
*	it is completly basic										*
*	it does nothing, the derived classes						*
*	cOpenGLSubSystem		will use OpenGL						*
****************************************************************/
class cGraphicsSubSystem	:	public singleton<cGraphicsSubSystem>
{
public:

	//	assume width and height of text buffer is zero initially.. for safety
	cGraphicsSubSystem();

	//	must be virtual..
	virtual ~cGraphicsSubSystem();


	//	use to allocate the text buffer..
	void AllocateTextBuffer(int NumRows,int NumColumns);

	
	int				m_NumLines;				//	width	of screen	(in char)
	int				m_NumCharsPerLine;		//	height	of screen	(in char)
	
	int				m_CurLine;				//	the line which the cursor is located
	int				m_CurCharPos;			//	the char position in which the charcater is located
	
	char **			m_ScreenText;			//	a 2D array of characters containing the screen buffer

	int				m_FontWidth;				//	width and height in pixels of each character
	int				m_FontHeight;



	//	some important device capabilits
	bool			m_bCanMultiTexture		;	//	can do multitextures?
	bool			m_bCanLockArrays		;	//	can lock arrays?
	bool			m_bSupportTexEnvCombine	;	//	texture environment combine
	bool			m_bSupportReflectionMap	;	//	support for reflection mapping
	int				m_MaxTextureUnits		;	//	maximum number of texure units


	void	ClearText(void);



	//	all functions here are very generic functions, they should be implemented in both of the APIz

	//	text writing functions
	virtual void	PutText(char* str,int,int)								{}
	virtual	void	BlitText(char*,int,int)									{}
	virtual	void	WriteText(int x,int y,int color,const char *,...)		{}
	virtual void	WriteMode()												{}
	virtual void	WriteTime()												{}
	virtual void	WriteFPS()												{}
	virtual	void	DrawTextBuffer()										{}
	virtual void	DrawText3D(char*,float=0.0f)							{}

	//	frame management functions
	virtual	void	InitDrawing2D(void)										{}
	virtual	void	EndDrawing2D(void)										{}
	virtual	void	UpdateRects()											{}
	virtual void	Flip()													{}
	virtual void	Clear()													{}

	//	GDI text functions
	virtual void WriteGDIText(char * str,int color,int x,int y)				{}

	HDC			m_hDC			;		//	device context for graphics subsystem

};

//	public access to graphics susbystem from here..
inline cGraphicsSubSystem	*&	Graphics		(void){return cGraphicsSubSystem::GetSingletonPtr();}


/*---------end of class cGraphicsSubSystem declaration------------------*/
#endif