#ifndef _HELMET_H
#define _HELMET_H
#include "application.h"

/****************************************************************
*	helmet.h													*
*	by Shady el Mously											*
*	this file is the header declaration for the classes used to	*
*	get data from the special input devices used in our project	*
*	the devices which will be decalred here are the tracker of	*
*	the head mounted display unit and the data gloves			*
****************************************************************/


/****************************************************************
*	class cViewer												*
*	represents an input device used to control the camera		*
*	this is a base class for two classes used to control		*
*	the view, one will be with mouse and the other from			*
*	the tracker fo the head mounted device						*
****************************************************************/
class cViewer
{
protected:



public:

	float	m_Yaw;		//	yaw angle	( with x axis)
	float	m_Pitch;	//	pitch angle ( with y axis)
	float	m_Roll;		//	roll angle	( with z axis)


	//	consructor
	cViewer()
	{
		m_Yaw	=	m_Pitch	=	m_Roll	=	0;
	}

	//	empty destructor
	virtual ~cViewer()
	{}

	//	Get The data..pure virtual here needed in children
	virtual void GetData()=0;
};
/*---------------end of class cViewer declaration--------------*/

/****************************************************************
*	cMouseViewer												*
*	makes the camera controlled by the x-y axis of the mouse	*
****************************************************************/
class cMouseViewer : public cViewer
{
public:



	//	update the angles from the mouse data
	void GetData()
	{
		m_Yaw	+=(Mouse()->m_DeltaX/100.0f);
		m_Pitch	-=(Mouse()->m_DeltaY/100.0f);
		m_Roll	+=(Mouse()->m_DeltaZ/100.0f);
	}
};




/*---------------end of										*/
/****************************************************************
*	class cHeadDisplay											*
*	represents the head mounted display							*
*	which has three inputs, the three orientation angles		*
*	yaw, pitch and roll											*
*	the head display can be requers ted for those				*
****************************************************************/
class cHelmetViewer : public cViewer
{
public:

	//	constructor creates the device
	cHelmetViewer();

	//	destructor closes it
	~cHelmetViewer();

	//	this will catch the data 
	void GetData();

};
/*--------------end of class cHeadDisplay declaration-----------*/
#endif //	_HELMET_H	