/****************************************************************
*		flulibinput.cpp											*
*	By Shady el Mously											*
*	this is the implementation file for the class _INPUT		*
*	for handling direct input in the engine						*
****************************************************************/


#include "input.h"			//	class definition file
#include "application.h"	//	For Report error and success


//	default constructor set nulls and report success
cInputSubSystem::cInputSubSystem()
{
	m_lpdi		=	NULL;
	m_lpdikey	=	NULL;
	m_lpdimouse	=	NULL;
	Init();
	Application()->m_ReportFile<<"DirectInput subsystem Successfully Initialized"<<endl;
}

//	destructor will release all direct input stuff and report success
cInputSubSystem::~cInputSubSystem()
{
	EndDMouse();
	EndDKey();
	EndDI();
	Application()->m_ReportFile<<"DirectInput subsystem Successfully ShutDown   "<<endl;
}



/// a function that initializes all direct input stuff
void cInputSubSystem::Init()
{
	InitDI();
	InitDKey();
	InitDMouse();
}


//	this will initialize direct input
void cInputSubSystem::InitDI()
{
	if (FAILED(DirectInput8Create(Application()->m_Instance,DIRECTINPUT_VERSION,IID_IDirectInput8,(void**)&m_lpdi,NULL)))
	{
		Application()->ReportError("Error Creating main direct input object");
	}
}



//	use this to start using direct keyoard input
void cInputSubSystem::InitDKey()
{
	if (FAILED(m_lpdi->CreateDevice(GUID_SysKeyboard,&m_lpdikey,NULL)))
	{
		Application()->ReportError("Error initializing keyboard device");
	}
	if(FAILED(m_lpdikey->SetCooperativeLevel(NULL,DISCL_BACKGROUND|DISCL_NONEXCLUSIVE)))
	{
		Application()->ReportError("Error Setting keyboard cooperative level");
	}
	
	if(FAILED(m_lpdikey->SetDataFormat(&c_dfDIKeyboard)))
	{
		Application()->ReportError("Error Setting keyboad data format");
	}
	if(FAILED(m_lpdikey->Acquire()))
	{
		Application()->ReportError("error aquiring keyboard device");
	}
}


//	this will let you start using drect mouse
void cInputSubSystem::InitDMouse()
{
	if(FAILED(m_lpdi->CreateDevice(GUID_SysMouse,&m_lpdimouse,NULL)))
	{Application()->ReportError("Error Creating mouse device");}
		if(FAILED(m_lpdimouse->SetCooperativeLevel(NULL,DISCL_BACKGROUND|DISCL_NONEXCLUSIVE)))
	{Application()->ReportError("Error Setting mouse cooperative level");}
		if(FAILED(m_lpdimouse->SetDataFormat(&c_dfDIMouse)))
	{Application()->ReportError("Error setting mouse data fromat");}
		if(FAILED(m_lpdimouse->Acquire()))
	{Application()->ReportError("error aquiring mouse");}
}



//	this will catch all input keyboard and mouse
//	i may try to make a thread which continuously catches input
void cInputSubSystem::Catch()
{
	if(FAILED(m_lpdikey->GetDeviceState(sizeof(UCHAR[256]),(LPVOID)m_keystate)))
	{Application()->ReportError("Error reading keyboard");}
	if(FAILED(m_lpdimouse->GetDeviceState(sizeof(DIMOUSESTATE),(LPVOID)&m_MouseState)))
	{Application()->ReportError("Error reading mouse");}

	//	update mouse buttons states
	Mouse()->m_bLButtonDown=(m_MouseState.rgbButtons[0] & 0x80) ? true : false;
	Mouse()->m_bRButtonDown=(m_MouseState.rgbButtons[1] & 0x80) ? true : false;
	Mouse()->m_bMButtonDown=(m_MouseState.rgbButtons[2] & 0x80) ? true : false;

	//	update mouse delta values
	Mouse()->m_DeltaX	=	m_MouseState.lX;
	Mouse()->m_DeltaY	=	m_MouseState.lY;
	Mouse()->m_DeltaZ	=	m_MouseState.lZ;
}


//	this abstarcts an input device..and it will have its guid to enable creating it..
struct sInputDevice
{
	char	m_DeviceName[256]	;	//	name of the device
	char	m_ProductName[256]	;	//	name of the product
	
	GUID	m_Guid				;	//	guid of the device to create it later
	int		m_NumButtons		;	//	number of buttons in the device
	int		m_NumAxes			;	//	number of axes in the device
	int		m_NumPOVs			;	//	number of points of view of the device


	char	m_MajorType[256]	;	//	the device major type
	char	m_SubType[256]		;	//	the device subtype


	sInputDevice(REFGUID DeviceID):m_Guid(DeviceID)
	{
		m_NumButtons=m_NumAxes=m_NumPOVs=0;
	}
};


BOOL CALLBACK cInputSubSystem::MyDIDevicesCallback(LPCDIDEVICEINSTANCE FoundDevice,LPVOID pvref)
{
	//	get the main DirectInput Interface here
	
	//	temporary device..created for testing..
	LPDIRECTINPUTDEVICE8 TempDevice=NULL;

	//	Device capabilites
	DIDEVCAPS			DeviceCaps;			//	device capabities structure

	//	setup the device capabilities structure
	memset(&DeviceCaps,0,sizeof(DIDEVCAPS));
	DeviceCaps.dwSize=sizeof(DIDEVCAPS);

	//	recast the pointer to the array of input devices
	vector<sInputDevice>*	pAllDevices=(vector<sInputDevice>*)pvref;

	//	create a new device desription
	sInputDevice NewDev(FoundDevice->guidInstance);
	strcpy(NewDev.m_DeviceName,FoundDevice->tszInstanceName);
	strcpy(NewDev.m_ProductName,FoundDevice->tszProductName);


	//	now create the device temporarly
	Input()->m_lpdi->CreateDevice(FoundDevice->guidInstance,&TempDevice,NULL);

	//	get its capabilities
	TempDevice->GetCapabilities(&DeviceCaps);

	//	number of axes
	NewDev.m_NumAxes=DeviceCaps.dwAxes;

	//	number of buttons
	NewDev.m_NumButtons=DeviceCaps.dwButtons;

	//	number of point of view controllers
	NewDev.m_NumPOVs	=	DeviceCaps.dwPOVs;


	DWORD MajorType	=	DeviceCaps.dwDevType&0x00FF;
	DWORD SubType	=	DeviceCaps.dwDevType&0xFF000000;

	strcpy(NewDev.m_SubType,"Unknown Subtype");

	//	now switch on all possible device types
	switch(MajorType)
	{
	case DI8DEVTYPE_1STPERSON:
		strcpy(NewDev.m_MajorType,"First Person");
		{
			switch(SubType)
			{
			case DI8DEVTYPE1STPERSON_LIMITED:
				strcpy(NewDev.m_SubType,"Limited 1st person device");
				break;
			case DI8DEVTYPE1STPERSON_SHOOTER:
				strcpy(NewDev.m_SubType,"For 1st person shooter");
				break;
			case DI8DEVTYPE1STPERSON_SIXDOF:
				strcpy(NewDev.m_SubType,"Has six degrees of freedom");
				break;
			}
		}
		break;

	case DI8DEVTYPE_DRIVING:
		strcpy(NewDev.m_MajorType,"Driving device");
		{
			switch(SubType)
			{
			case DI8DEVTYPEDRIVING_COMBINEDPEDALS:
				strcpy(NewDev.m_SubType,"Combined pedals (Treat acceleration and bracking as single axe)");
				break;
			case DI8DEVTYPEDRIVING_DUALPEDALS:
				strcpy(NewDev.m_SubType,"Dual pedals (acceleration and bracking are separate axes)");
				break;
			case DI8DEVTYPEDRIVING_HANDHELD:
				strcpy(NewDev.m_SubType,"hand held");
				break;
			case DI8DEVTYPEDRIVING_LIMITED:
				strcpy(NewDev.m_SubType,"limited (Doesnt have enough device objects)");
				break;
			case DI8DEVTYPEDRIVING_THREEPEDALS:
				strcpy(NewDev.m_SubType,"three pedals (Acceleration, bracking and glutching)");
				break;
			}
		}
		break;

	case DI8DEVTYPE_FLIGHT:
		strcpy(NewDev.m_MajorType,"Flight device");
		{
			switch(SubType)
			{
			case DI8DEVTYPEFLIGHT_LIMITED:
				strcpy(NewDev.m_SubType,"limited (doesnt have enough device objects)");
				break;
			case DI8DEVTYPEFLIGHT_RC:
				strcpy(NewDev.m_SubType,"RC (based on model airplane remote control)");
				break;
			case DI8DEVTYPEFLIGHT_STICK:
				strcpy(NewDev.m_SubType,"Flight joystick");
				break;
			case DI8DEVTYPEFLIGHT_YOKE:
				strcpy(NewDev.m_SubType,"Flight yoke");
				break;
			}
		}
		break;

	case DI8DEVTYPE_GAMEPAD:
		{
			switch(SubType)
			{
				case DI8DEVTYPEGAMEPAD_LIMITED:
					strcpy(NewDev.m_SubType,"limited (doesnt have enough device objects)");
					break;
				case DI8DEVTYPEGAMEPAD_STANDARD:
					strcpy(NewDev.m_SubType,"Standard gamepad");
					break;
				case DI8DEVTYPEGAMEPAD_TILT:
					strcpy(NewDev.m_SubType,"tilt (Can report x,y axis data when tilted");
					break;
			}
		}
		strcpy(NewDev.m_MajorType,"GamePad device");
		break;

	case DI8DEVTYPE_JOYSTICK:
		{
			switch(SubType)
			{
				case DI8DEVTYPEJOYSTICK_LIMITED:
					strcpy(NewDev.m_SubType,"limited (doesnt have enough device objects)");
					break;
				case DI8DEVTYPEJOYSTICK_STANDARD:
					strcpy(NewDev.m_SubType,"Standard joystick");
					break;
			}
		}
		strcpy(NewDev.m_MajorType,"Joystick ");
		break;

	case DI8DEVTYPE_KEYBOARD:
		{
			switch(SubType)
			{
				case DI8DEVTYPEKEYBOARD_UNKNOWN:
					strcpy(NewDev.m_SubType,"unknown keyboard type");
					break;
				case DI8DEVTYPEKEYBOARD_PCXT:
					strcpy(NewDev.m_SubType,"PC XT keyboard");
					break;
				case DI8DEVTYPEKEYBOARD_PCAT:
					strcpy(NewDev.m_SubType,"PC AT keyboard");
					break;
				case DI8DEVTYPEKEYBOARD_OLIVETTI:
					strcpy(NewDev.m_SubType,"olivetti 102-key keyboard");
					break;
				case DI8DEVTYPEKEYBOARD_PCENH:
					strcpy(NewDev.m_SubType,"IBM PC Enhanced 101/102");
					break;
			}
		}
		strcpy(NewDev.m_MajorType,"Keyboard ");
		break;

	case DI8DEVTYPE_MOUSE:
		{
			switch(SubType)
			{
			case DI8DEVTYPEMOUSE_ABSOLUTE:
				strcpy(NewDev.m_SubType,"Mouse that returns absolute axis data");
				break;
			case DI8DEVTYPEMOUSE_FINGERSTICK:
				strcpy(NewDev.m_SubType,"Fingerstick");
				break;
			case DI8DEVTYPEMOUSE_TOUCHPAD:
				strcpy(NewDev.m_SubType,"Touchpad");
				break;
			case DI8DEVTYPEMOUSE_TRACKBALL:
				strcpy(NewDev.m_SubType,"TrackBall");
				break;
			case DI8DEVTYPEMOUSE_TRADITIONAL:
				strcpy(NewDev.m_SubType,"Traditional");
				break;
			case DI8DEVTYPEMOUSE_UNKNOWN:
				strcpy(NewDev.m_SubType,"unknown mouse type");
				break;
			}
		}
		strcpy(NewDev.m_MajorType,"Mouse ");
		break;

	case DI8DEVTYPE_SCREENPOINTER:
		{
			switch(SubType)
			{
			case DI8DEVTYPESCREENPTR_UNKNOWN:
				strcpy(NewDev.m_SubType,"unknown subtype");
				break;
			case DI8DEVTYPESCREENPTR_LIGHTGUN:
				strcpy(NewDev.m_SubType,"light gun");
				break;
			case DI8DEVTYPESCREENPTR_LIGHTPEN:
				strcpy(NewDev.m_SubType,"light pen");
				break;
			case DI8DEVTYPESCREENPTR_TOUCH:
				strcpy(NewDev.m_SubType,"touch screen");
				break;
			}
		}
		strcpy(NewDev.m_MajorType,"Screen pointer ");
		break;

	case DI8DEVTYPE_SUPPLEMENTAL:
		{
			switch(SubType)
			{
			case DI8DEVTYPESUPPLEMENTAL_2NDHANDCONTROLLER:
				strcpy(NewDev.m_SubType,"second hand controller");
				break;
			case DI8DEVTYPESUPPLEMENTAL_COMBINEDPEDALS:
				strcpy(NewDev.m_SubType,"combined pedals(use a single axe for acceleration and bracking");
				break;
			case DI8DEVTYPESUPPLEMENTAL_DUALPEDALS:
				strcpy(NewDev.m_SubType,"dual pedals(separate axes for acceleration and bracking");
				break;
			case DI8DEVTYPESUPPLEMENTAL_HANDTRACKER:
				strcpy(NewDev.m_SubType,"hand tracker (tracks hand movement)");
				break;
			case DI8DEVTYPESUPPLEMENTAL_HEADTRACKER:
				strcpy(NewDev.m_SubType,"head tracker (tracks head movement)");
				break;
			case DI8DEVTYPESUPPLEMENTAL_RUDDERPEDALS:
				strcpy(NewDev.m_SubType,"rudder pedals");
				break;
			case DI8DEVTYPESUPPLEMENTAL_SHIFTER:
				strcpy(NewDev.m_SubType,"shifter (reports gear selection from an axis");
				break;
			case DI8DEVTYPESUPPLEMENTAL_SHIFTSTICKGATE:
				strcpy(NewDev.m_SubType,"shift stick gate (report gear selection from buttons");
				break;
			case DI8DEVTYPESUPPLEMENTAL_SPLITTHROTTLE:
				strcpy(NewDev.m_SubType,"split throttle (report two or more throttle values)");
				break;
			case DI8DEVTYPESUPPLEMENTAL_THREEPEDALS:
				strcpy(NewDev.m_SubType,"three pedals");
				break;
			case DI8DEVTYPESUPPLEMENTAL_THROTTLE:
				strcpy(NewDev.m_SubType,"throttle");
				break;
			case DI8DEVTYPESUPPLEMENTAL_UNKNOWN:
				strcpy(NewDev.m_SubType,"Uknown supplemental subtype");
				break;

			}
		}
		strcpy(NewDev.m_MajorType,"Supplemental device ");
		break;

	default:
		strcpy(NewDev.m_MajorType,"Unknown type!");
		break;
	}




	//	and release it
	SAFE_RELEASE(TempDevice);
	
	//	add it to the vector
	pAllDevices->push_back(NewDev);

	return DIENUM_CONTINUE;
}

void cInputSubSystem::DetectDevices()
{
	//	fill this array with data about all devices
	vector<sInputDevice>	AllDevices;

	//	call the enumeration function which will fill the vector
	m_lpdi->EnumDevices(DI8DEVCLASS_ALL,MyDIDevicesCallback,(void*)&AllDevices,DIEDFL_ALLDEVICES);


	//	open a file to write the report about the input devices found
	ofstream outfile;
	outfile.open("inputinfo.txt");

	outfile<<"Found "<<AllDevices.size()<<" devices."<<endl<<endl;

	//	loop on devices found
	for(int i=0;i<AllDevices.size();i++)
	{
		outfile<<"====================================================="<<endl;
		outfile<<"Device  Name             :	"<<AllDevices[i].m_DeviceName<<endl;
		outfile<<"Product Name             :	"<<AllDevices[i].m_ProductName<<endl;
		outfile<<"Number of axes           :	"<<AllDevices[i].m_NumAxes<<endl;
		outfile<<"Number of buttons        :	"<<AllDevices[i].m_NumButtons<<endl;
		outfile<<"Number of points of view :	"<<AllDevices[i].m_NumPOVs<<endl;
		outfile<<"Major Type               :	"<<AllDevices[i].m_MajorType<<endl;
		outfile<<"Sub Type                 :	"<<AllDevices[i].m_SubType<<endl;
	}
	outfile.close();
}


/*-------------------------------------------------------------*/
//				End of _INPUT implementation
