#include "md2mesh.h"



//	calculates the normal for a single vertex
void	CalculateNormal(float* p1,float* p2,float* p3);


// sMD2Mesh constructor
sMD2Mesh::sMD2Mesh()
{
	numVertices		=	0			;// vertices
	numTriangles	=	0			;// triangles
	numFrames		=	0			;// frames
	numST			=	0			;// texture coordinates
	frameSize		=	0			;// needed?
	currentFrame	=	0			;// current keyframe 
	nextFrame		=	1			;// next keyframe
	interpol		=	0.0			;// interpolation percent
	triIndex		=	NULL		;// triangle indices
}    

// sMD2Mesh destructor
sMD2Mesh::~sMD2Mesh()
{
	Unload();
}


// sMD2Mesh::Load()
// access: public
// desc: loads model and skin
int sMD2Mesh::Load(char *modelFile, char *skinFile)
{
	FILE *filePtr;                          // file pointer
	int fileLen;                            // length of model file
	char *buffer;                           // file buffer
     
	modelHeader_t *modelHeader;             // model header
     
	stIndex_t *stPtr;                       // texture data
	frame_t *frame;                            // frame data
	point3d *vertexListPtr;                // index variable
	mesh_t *bufIndexPtr;					// index variables
	int i, j;                               // index variables
     
	// open the model file
	filePtr = fopen(modelFile, "rb");
	if (filePtr == NULL)
		return FALSE;
     
	// find length of file
	fseek(filePtr, 0, SEEK_END);
	fileLen = ftell(filePtr);
	fseek(filePtr, 0, SEEK_SET);
     
	// read entire file into buffer
	buffer = new char [fileLen+1];
	fread(buffer, sizeof(char), fileLen, filePtr);
     
	// extract model file header from buffer
	modelHeader = (modelHeader_t*)buffer;

	m_Verts.clear();
	m_Verts.reserve(modelHeader->numXYZ * modelHeader->numFrames);

     
	numVertices	=	modelHeader->numXYZ;
	numFrames		=	modelHeader->numFrames;
	frameSize		=	modelHeader->framesize;
     
	for (j = 0; j < numFrames; j++)
	{
		frame = (frame_t*)&buffer[modelHeader->offsetFrames + frameSize * j];
		vertexListPtr = (point3d*)&m_Verts[numVertices * j];
		for (i = 0; i < numVertices; i++)
		{
			vertexListPtr[i].v[0] = frame->scale[0] * frame->fp[i].v[0] + frame->translate[0];
			vertexListPtr[i].v[1] = frame->scale[2] * frame->fp[i].v[2] + frame->translate[2];
			vertexListPtr[i].v[2] = frame->scale[1] * frame->fp[i].v[1] + frame->translate[1];
		}
	}

	numST = modelHeader->numST;
	m_TexCoords.clear();
	m_TexCoords.reserve(numST);
	texcoord newTexCoord;


	if(skinFile)	
	{
		m_Texture=new cTexture(skinFile);
	    stPtr = (stIndex_t*)&buffer[modelHeader->offsetST];
		for (i = 0; i < numST; i++)
		{
			newTexCoord.s = (float)stPtr[i].s / (float)m_Texture->m_Width;
			newTexCoord.t = (float)stPtr[i].t / (float)m_Texture->m_Height;
			m_TexCoords.push_back(newTexCoord);
		}
	}
	else
	{
		stPtr = (stIndex_t*)&buffer[modelHeader->offsetST];
		for (i = 0; i < numST; i++)
		{
			newTexCoord.s = 0.0f;
			newTexCoord.t = 0.0f;
			m_TexCoords.push_back(newTexCoord);
		}
	}



	numTriangles = modelHeader->numTris;
	triIndex = new mesh_t [numTriangles];
 
	// point to triangle indexes in buffer
	bufIndexPtr = (mesh_t*)&buffer[modelHeader->offsetTris];
     
	// create a mesh (triangle) list
	for (j = 0; j < numFrames; j++)         
	{
		// for all triangles in each frame
		for(i = 0; i < numTriangles; i++)
		{
			triIndex[i].meshIndex[0] = bufIndexPtr[i].meshIndex[0];
			triIndex[i].meshIndex[1] = bufIndexPtr[i].meshIndex[1];
			triIndex[i].meshIndex[2] = bufIndexPtr[i].meshIndex[2];
			triIndex[i].stIndex[0] = bufIndexPtr[i].stIndex[0];
			triIndex[i].stIndex[1] = bufIndexPtr[i].stIndex[1];
			triIndex[i].stIndex[2] = bufIndexPtr[i].stIndex[2];
		}
	}

	// close file and free memory
	fclose(filePtr);
	delete [] buffer;
     
	currentFrame = 0;
	nextFrame = 1;
	interpol = 0.0;
     
	return TRUE;
}


// sMD2Mesh::LoadSkin()
// access: public
// desc: loads a skin for the model
int sMD2Mesh::LoadSkin(char *skinFile)
{
     int i;
     
	m_Texture=new cTexture(skinFile);
     
     
     for (i = 0; i < numST; i++)
     {
          m_TexCoords[i].s /= (float)m_Texture->m_Width;
          m_TexCoords[i].t /= (float)m_Texture->m_Height;
     }
     
     return 0;
}


// sMD2Mesh::Animate()
// access: public
// desc: animates the model between the keyframes startFrame and endFrame
int sMD2Mesh::Animate(int startFrame, int endFrame, float percent)
{
	point3d *vList;			// current frame vertices
	point3d *nextVList;		// next frame vertices
	int i;					// index counter
	float x1, y1, z1;		// current frame point values
	float x2, y2, z2;		// next frame point values
	
	point3d vertex[3]; 

	if((startFrame > currentFrame))
		currentFrame = startFrame;
     
	if((startFrame < 0) || (endFrame < 0))
		return -1;
     
	if((startFrame >= numFrames) || (endFrame >= numFrames))
		return -1;
     
	if(interpol >= 1.0)
	{
		interpol = 0.0f;
		currentFrame++;
		if(currentFrame >= endFrame)
			currentFrame = startFrame;
        nextFrame = currentFrame + 1;
		if (nextFrame >= endFrame)
			nextFrame = startFrame;
	}
	vList = &m_Verts[numVertices*currentFrame];
	nextVList = &m_Verts[numVertices*nextFrame];

	
	    
	if(m_Texture)m_Texture->Activate();

	glBegin(GL_TRIANGLES);
	for (i = 0; i < numTriangles; i++)
	{
		// get first points of each frame
		x1 = vList[triIndex[i].meshIndex[0]].v[0];
		y1 = vList[triIndex[i].meshIndex[0]].v[1];
		z1 = vList[triIndex[i].meshIndex[0]].v[2];
		x2 = nextVList[triIndex[i].meshIndex[0]].v[0];
		y2 = nextVList[triIndex[i].meshIndex[0]].v[1];
		z2 = nextVList[triIndex[i].meshIndex[0]].v[2];

		// store first interpolated vertex of triangle
		vertex[0].v[0] = x1 + interpol * (x2 - x1);
		vertex[0].v[1] = y1 + interpol * (y2 - y1);
		vertex[0].v[2] = z1 + interpol * (z2 - z1);
          
		// get second points of each frame
		x1 = vList[triIndex[i].meshIndex[2]].v[0];
		y1 = vList[triIndex[i].meshIndex[2]].v[1];
		z1 = vList[triIndex[i].meshIndex[2]].v[2];
		x2 = nextVList[triIndex[i].meshIndex[2]].v[0];
		y2 = nextVList[triIndex[i].meshIndex[2]].v[1];
		z2 = nextVList[triIndex[i].meshIndex[2]].v[2];
          
		// store second interpolated vertex of triangle
		vertex[2].v[0] = x1 + interpol * (x2 - x1);
		vertex[2].v[1] = y1 + interpol * (y2 - y1);
		vertex[2].v[2] = z1 + interpol * (z2 - z1);   
          
		// get third points of each frame
		x1 = vList[triIndex[i].meshIndex[1]].v[0];
		y1 = vList[triIndex[i].meshIndex[1]].v[1];
		z1 = vList[triIndex[i].meshIndex[1]].v[2];
		x2 = nextVList[triIndex[i].meshIndex[1]].v[0];
		y2 = nextVList[triIndex[i].meshIndex[1]].v[1];
		z2 = nextVList[triIndex[i].meshIndex[1]].v[2];
          
		// store third interpolated vertex of triangle
		vertex[1].v[0] = x1 + interpol * (x2 - x1);
		vertex[1].v[1] = y1 + interpol * (y2 - y1);
		vertex[1].v[2] = z1 + interpol * (z2 - z1);

		   
		// calculate the normal of the triangle
		CalculateNormal(vertex[0].v, vertex[2].v, vertex[1].v);
		// render properly textured triangle
		glTexCoord2f(m_TexCoords[triIndex[i].stIndex[0]].s,m_TexCoords[triIndex[i].stIndex[0]].t);
		glVertex3fv(vertex[0].v);
		glTexCoord2f(m_TexCoords[triIndex[i].stIndex[1]].s,m_TexCoords[triIndex[i].stIndex[1]].t);
		glVertex3fv(vertex[1].v);
		glTexCoord2f(m_TexCoords[triIndex[i].stIndex[2]].s,m_TexCoords[triIndex[i].stIndex[2]].t);
		glVertex3fv(vertex[2].v);
	}
	glEnd();
	interpol += percent;  // increase percentage of interpolation between frames
	m_Texture->Deactivate();

	
	return 0;
}

// RenderFrame()
// desc: renders a single key frame
int sMD2Mesh::RenderFrame(int keyFrame)
{
	point3d *vList;
	int i;
     
	// create a pointer to the frame we want to show
	vList = &m_Verts[numVertices * keyFrame];
     
	if(m_Texture)m_Texture->Activate();
     
	// display the textured model with proper lighting normals
	glBegin(GL_TRIANGLES);
	for(i = 0; i < numTriangles; i++)
	{
		CalculateNormal(vList[triIndex[i].meshIndex[0]].v,vList[triIndex[i].meshIndex[2]].v,vList[triIndex[i].meshIndex[1]].v);
		glTexCoord2f(m_TexCoords[triIndex[i].stIndex[0]].s,m_TexCoords[triIndex[i].stIndex[0]].t);
		glVertex3fv(vList[triIndex[i].meshIndex[0]].v);
		glTexCoord2f(m_TexCoords[triIndex[i].stIndex[1]].s,m_TexCoords[triIndex[i].stIndex[1]].t);
		glVertex3fv(vList[triIndex[i].meshIndex[1]].v);
		glTexCoord2f(m_TexCoords[triIndex[i].stIndex[2]].s,m_TexCoords[triIndex[i].stIndex[2]].t);
		glVertex3fv(vList[triIndex[i].meshIndex[2]].v);
	}
	glEnd();
	if(m_Texture)m_Texture->Deactivate();
    return 0;
}

// Unload()
// desc: unloads model data from memory
int sMD2Mesh::Unload()
{
	if (triIndex)
		delete[]triIndex;
	return 0;
}


//	calculates the normal for a single vertex
void	CalculateNormal(float* p1,float* p2,float* p3)
{
	float a[3],b[3],result[3];
	float length;
	
	a[0]=p1[0]-p2[0];
	a[1]=p1[1]-p2[1];
	a[2]=p1[2]-p2[2];

	b[0]=p1[0]-p3[0];
	b[1]=p1[1]-p3[1];
	b[2]=p1[2]-p3[2];

	result[0]=a[1]*b[2]-b[1]*a[2];
	result[1]=a[0]*b[2]-b[0]*a[2];
	result[2]=a[0]*b[1]-b[0]*a[1];

	//	calculate the length of the normal
	length=(float)sqrt(result[0]*result[0]+result[1]*result[1]*result[2]*result[2]);

	//	normalize and specify the normal
	glNormal3f(result[0]/length,result[1]/length,result[2]/length);
}


