#ifndef _MD2MESH_H
#define _MD2MESH_H

#include "meshes.h"

#define MAX_FRAMES 512



/* 
	MD2 Model Functionality
*/


// texture coordinate index
typedef struct
{
   short s;
   short t;
} stIndex_t;

// info for a single frame point
typedef struct
{
   unsigned char v[3];
   unsigned char normalIndex;	// not used
} framePoint_t;

// information for a single frame
typedef struct
{
   float scale[3];
   float translate[3];
   char name[16];
   framePoint_t fp[1];
} frame_t;

// data for a single triangle
typedef struct
{
   unsigned short meshIndex[3];		// vertex indices
   unsigned short stIndex[3];		// texture coordinate indices
} mesh_t;

typedef struct
{
   int ident;		 // identifies as MD2 file "IDP2"
   int version;		 // mine is 8
   int skinwidth;    // width of texture
   int skinheight;   // height of texture
   int framesize;    // number of bytes per frame
   int numSkins;     // number of textures
   int numXYZ;       // number of points
   int numST;        // number of texture
   int numTris;      // number of triangles
   int numGLcmds;
   int numFrames;    // total number of frames
   int offsetSkins;  // offset to skin names (64 bytes each)
   int offsetST;     // offset of texture s-t values
   int offsetTris;   // offset of triangle mesh
   int offsetFrames; // offset of frame data (points)
   int offsetGLcmds; // type of OpenGL commands to use
   int offsetEnd;    // end of file
} modelHeader_t;


class sMD2Mesh : public sMesh
{
private:

     int numFrames;				// number of model frames
     int numVertices;			// number of vertices
     int numTriangles;			// number of triangles
     int numST;					// number of skins
     int frameSize;				// size of each frame in bytes
     int currentFrame;			// current frame # in animation
     int nextFrame;				// next frame # in animation
     float interpol;			// percent through current frame
     mesh_t *triIndex;			// triangle list

public:

	sMD2Mesh();        // constructor
	~sMD2Mesh();       // destructor

	// load model and skin/texture at the same time
	int Load(char *modelFile, char *skinFile=NULL);

	// load skin only
	int LoadSkin(char *skinFile);

	// render model with interpolation to get animation
	int Animate(int startFrame, int endFrame, float percent);

	// render a single frame
	int RenderFrame(int keyFrame);

	// free memory of model
	int Unload();


};

#endif	//_MD2MESH_H