#include "mesh3ds.h"
#include "graphics.h"

/****************************************************************
*	mesh3ds.cpp													*
*	by Shady el Mously											*
*	part of the flulib 3d engin									*
*	this file is the implementation of the classes responsible	*
*	of loading and processing 3ds studio files					*
*	still under development										*
****************************************************************/


//------ Primary chunk

#define MAIN3DS       0x4D4D

//------ Main Chunks

#define EDIT3DS       0x3D3D  // this is the start of the editor config
#define KEYF3DS       0xB000  // this is the start of the keyframer config

//------ sub defines of EDIT3DS

#define EDIT_MATERIAL 0xAFFF
#define EDIT_CONFIG1  0x0100
#define EDIT_CONFIG2  0x3E3D
#define EDIT_VIEW_P1  0x7012
#define EDIT_VIEW_P2  0x7011
#define EDIT_VIEW_P3  0x7020
#define EDIT_VIEW1    0x7001
#define EDIT_BACKGR   0x1200
#define EDIT_AMBIENT  0x2100
#define EDIT_OBJECT   0x4000

#define EDIT_UNKNW01  0x1100
#define EDIT_UNKNW02  0x1201
#define EDIT_UNKNW03  0x1300
#define EDIT_UNKNW04  0x1400
#define EDIT_UNKNW05  0x1420
#define EDIT_UNKNW06  0x1450
#define EDIT_UNKNW07  0x1500
#define EDIT_UNKNW08  0x2200
#define EDIT_UNKNW09  0x2201
#define EDIT_UNKNW10  0x2210
#define EDIT_UNKNW11  0x2300
#define EDIT_UNKNW12  0x2302 
#define EDIT_UNKNW13  0x3000
#define EDIT_UNKNW14  0xAFFF

//------ sub defines of EDIT_OBJECT
#define OBJ_TRIMESH   0x4100
#define OBJ_LIGHT     0x4600
#define OBJ_CAMERA    0x4700

#define OBJ_UNKNWN01  0x4010
#define OBJ_UNKNWN02  0x4012 //---- Could be shadow

//------ sub defines of OBJ_CAMERA
#define CAM_UNKNWN01  0x4710 
#define CAM_UNKNWN02  0x4720 

//------ sub defines of OBJ_LIGHT
#define LIT_OFF       0x4620
#define LIT_SPOT      0x4610
#define LIT_UNKNWN01  0x465A
//------ sub defines of OBJ_TRIMESH
#define TRI_VERTEXL			0x4110
#define TRI_FACEL2			0x4111 
#define TRI_FACEL1			0x4120
#define	TRI_MATERIAL		0x4130
#define	TRI_MAPPINGCOORS    0x4140
#define TRI_SMOOTH			0x4150
#define TRI_LOCAL			0x4160
#define	TRI_MAPPINGSTANDARD 0x4170
#define TRI_VISIBLE			0x4165
//------ sub defs of KEYF3DS

#define KEYF_UNKNWN01 0xB009
#define KEYF_UNKNWN02 0xB00A
#define KEYF_FRAMES   0xB008
#define KEYF_OBJDES   0xB002




//------  these define the different color chunk types
#define COL_RGB  0x0010
#define COL_TRU  0x0011
#define COL_UNK  0x0013

//------ defines for viewport chunks

#define TOP           0x0001
#define BOTTOM        0x0002
#define LEFT          0x0003
#define RIGHT         0x0004
#define FRONT         0x0005
#define BACK          0x0006
#define USER          0x0007
#define CAMERA        0x0008 // 0xFFFF is the actual code read from file
#define LIGHT         0x0009
#define DISABLED      0x0010
#define BOGUS         0x0011





/****************************************************************
*	class cMesh3ds implementation								*
*	derived from sIndexedMesh									*
*	represents a single mesh inside a 3ds file					*
****************************************************************/


//	constructor initializes everything
cMesh3ds::cMesh3ds()
{
	lpTexture	=	NULL	;
	m_pMaterial	=	NULL	;
	dwNumVerts	=	0		;
	dwNumFaces	=	0		;
}


//	draw a single mesh using all the internal data
void cMesh3ds::Draw()
{
	if(lpTexture)	lpTexture->Activate();

	glPushMatrix();
	glMultMatrixf(Matrix.d);

	m_pMaterial->Activate();

	glVertexPointer(3,GL_FLOAT,0,m_Verts.begin());					//	set vertex pointer to start of vertex array
	glNormalPointer(GL_FLOAT,0,m_Norms.begin());					//	set normals pointer to start of normals array
	glTexCoordPointer(2,GL_FLOAT,0,m_TexCoords.begin());					//	set texture coordinates pointer to start of texcoords array
	glDrawElements(GL_TRIANGLES,m_Tris.size()*3,					//	draw elements using
	GL_UNSIGNED_SHORT,m_Tris.begin());								//	the indexes sent

	m_pMaterial->Deactivate();


	glPopMatrix();
	if(lpTexture)	lpTexture->Deactivate();
}


//	adjust by adding some dummt texture coordiantes
void cMesh3ds::Adjust()
{
	texcoord t;
	while(m_TexCoords.size()<m_Verts.size())
	{
		m_TexCoords.push_back(t);
	}
}
/*-------------end of class cMesh3ds implementation------------*/


/****************************************************************
*	class c3dsObject implementation								*
*	still under development										*
****************************************************************/

//	constructor and destructor :
s3dsMesh::s3dsMesh()	//on creation set all varible to 0 or NULL
{
	bError				= false;
	dwTotalVerts		= 0;
	dwTotalFaces		= 0;
}

s3dsMesh::~s3dsMesh()	//on destruction
{
}



//	3ds specific file format reading functions
unsigned char s3dsMesh::ReadChar()	//reads a 8bit chunk of the file
{
	return fgetc(fle3DS);
}

unsigned int s3dsMesh::ReadInt()		//reads a 16bit chunk of the file
{
	unsigned int temp = ReadChar();
	return (temp | (ReadChar() << 8));	//bind to chars into one int
}

unsigned long s3dsMesh::ReadLong()	//reads a 32bit chunk of the file
{
	unsigned long temp1, temp2;
	temp1 = ReadInt();
	temp2 = ReadInt();

	return (temp1 | (temp2 << 16));		//binds two ints(four chars) into one long
}

//changes the proportion of a byte(0-255) to a float(0.0-1.0) EX:if 127 was input as val, then PropBYTEtoFloat would return 0.49f
float s3dsMesh::PropBytetoFloat(BYTE val)
{
	float newVal = (float)val / 255;
	return newVal;
}



//	load the data from a 3ds file sent
int s3dsMesh::Load3DS(char *fileName)
{
	fle3DS = fopen(fileName, "rb"); //open the specified file as read only and binary

	strcpy(FileName, fileName);

	if(fle3DS == NULL)
	{
		bError = true;
		return ERR3DS_FILELOAD;
	}

	fseek(fle3DS, 0, SEEK_SET);	//set our file posistion at the begining

	if(Read3DSHeader() == false)
	{
		bError = true;
		return ERR3DS_INVALIDVERSION;
	}

	//load the file
	Read3DSData();

	//	calculate the normals
	for(int i=0;i<m_MeshList.size();i++)
		m_MeshList[i].ComputeNormals(m_MeshList[i].m_Verts,m_MeshList[i].m_Norms);

	//	link the materials to their meshes
	LinkMaterials();

	//	adjust around a centralized point
	Adjust();

	return LOAD3DS_SUCCESFUL;
}



//	read the file header and return true only if it a 3ds file
bool s3dsMesh::Read3DSHeader()		//reads the 3ds header mainly to check for the right file version (3DS v.3 or above)
{
	if(ReadInt() == MAIN3DS)
	{
		fseek(fle3DS, 28L, SEEK_SET);

		Version3DS = ReadChar();

		if(Version3DS < 3) //Make sure we are loading a compatible version
		{
			return false;//exit out if wrong version
		}

		fseek(fle3DS, 16, SEEK_SET);
		return true;
	}
	else
		return false;
}


//	this will load the whole mesh and material data from inside the file
//	everything but objects will be skipped
void s3dsMesh::Read3DSData()
{
	WORD wChunkDesc;//tells us what kind of data out file pos is at
	bool done = false;

	while(!done)
	{
		wChunkDesc = ReadInt();//obtain data description
		if(feof(fle3DS))//if the end of the file is reached immediatly step out of loop so function dosent read data that aint there
		{
			done = true;
			break;//immediatly step out of while loop
		}

		Len = ReadLong();

		switch(wChunkDesc)
		{
		case 0xFFFF:	//NULL(no data to read)
			done = true;
			break;

		case EDIT3DS:	//valid object(s)

			ReadObjects();	//function reads all 3DS objects (vertices, faces, lights, materials, etc...)
			fseek(fle3DS, 0, SEEK_END);	//after getting all objects we dont need to access the file anymore
			break;

		default:	//some unknown chunk descriptor
			unsigned long pos;
			pos = ftell(fle3DS);
			pos += (Len - 6);

			if(fseek(fle3DS, pos, SEEK_SET))
				done = true;

			break;
		}
		if(feof(fle3DS))	//make sure we havent reached the end
			done = true;
	}
}

//	read objects from inside the file
//	the objects might be meshes or material data
//	anything else will be skipped for now
void s3dsMesh::ReadObjects()
{
	unsigned long count = ftell(fle3DS) + (Len - 6);	//where we are in the file
	WORD wChunkID;	//tells us what the chunk of data consists of
	DWORD llen;		//the chunk length
	bool IsDone = false;	

	while(!IsDone)
	{
		//	read the cunk id
		wChunkID = ReadInt();
		if(feof(fle3DS))
		{
			IsDone = true;
			break;
		}

		//	read the chunk length
		llen = ReadLong();


		//	check which chunk id is this..
		switch(wChunkID)
		{
		case EDIT_OBJECT:	//if it is a mesh id
			{
				//	create a new mesh object
				cMesh3ds newmesh;

				//get the mesh name
				char ch;
				int charpos = 0;
							
				while((ch = fgetc(fle3DS)) != 0)
				{
					newmesh.mName[charpos] = ch;
					charpos++;
				}
				newmesh.mName[charpos] = '\0';

				//	make the mesh material the generic material initially
				newmesh.m_pMaterial=(sMaterial3ds*)&sMaterial::BRUSHED_METAL;

				//	add the new mesh to the list
				m_MeshList.push_back(newmesh);
	
				//	get all the mesh properties(verts, indicies, material name etc...)
				ReadMesh(llen);
	
				//	if the mesh doesnt have vertices, destroy it (lights or cameras..etc
				if(m_MeshList.back().dwNumVerts==NULL)
				{
					m_MeshList.pop_back();
				}
			}break;	
		case EDIT_MATERIAL://material
			ReadMaterials(llen);

			break;

		default:
			//if we come across a chunk we dont know just skip past it
			unsigned long pos;
			pos = ftell(fle3DS);
			if((pos - 6) >= count)
			{
				fseek(fle3DS, -6, SEEK_CUR);
				IsDone = true;
				break;
			}
			pos += (llen - 6);
			if(fseek(fle3DS, pos, SEEK_SET))
				IsDone = true;
		}
	}
}


//	this will read mesh data
//	which could be vertices data, texture coordiantes, texturew filename
//	tranformation matrix
//	until now, the transformation matrix is not read coorectly
//	and the normals information is not read either
void s3dsMesh::ReadMesh(DWORD Len)
{
	unsigned long count = ftell(fle3DS) + (Len - 6);	//where we are in the file
	WORD wMeshID;	//tells us what the chunk conatins
	DWORD llen;	//length of the chunk
	bool IsDone = false;
	unsigned int i;

	while(IsDone == false)
	{
		wMeshID = ReadInt();	//obtain what the chunk contain
		if(feof(fle3DS))	//make sure we havent reached the end of the file
		{
			IsDone = true;
			break;
		}

		llen = ReadLong();	//get the length of the chunk
		switch(wMeshID)	
		{
		case OBJ_TRIMESH:
			break;
			
		case TRI_VERTEXL:	//vertex info(just the position)
			m_MeshList.back().dwNumVerts = ReadInt();	//get the number of verts
			dwTotalVerts += m_MeshList.back().dwNumVerts;	//add this to the total number of verts in the file
			m_MeshList.back().m_Verts.reserve(m_MeshList.back().dwNumVerts);

			//read the vertices. location
			for(i = 0; i < m_MeshList.back().dwNumVerts; i++)
			{
				point3d NewVertex;
				fread(&NewVertex.x, sizeof(float), 1, fle3DS);
				fread(&NewVertex.y, sizeof(float), 1, fle3DS);
				fread(&NewVertex.z, sizeof(float), 1, fle3DS);
				m_MeshList.back().m_Verts.push_back(NewVertex);
				m_MeshList.back().m_Norms.push_back(NewVertex);
			}
			break;

		case TRI_FACEL1:	//face info(Indices)
			m_MeshList.back().dwNumFaces = ReadInt();	//get the number of indices

			dwTotalFaces += m_MeshList.back().dwNumFaces;	//add this to the total number of indices
			m_MeshList.back().m_Tris.reserve(m_MeshList.back().dwNumFaces/3);

			for(i = 0; i < m_MeshList.back().dwNumFaces; i++)	//get the index values
			{
				stri tri;
				WORD dummy;
				fread(&tri.v[0], sizeof(WORD), 1, fle3DS);
				fread(&tri.v[1], sizeof(WORD), 1, fle3DS);
				fread(&tri.v[2], sizeof(WORD), 1, fle3DS);
				fread(&dummy, sizeof(WORD), 1, fle3DS);	//face flags, not used 
				m_MeshList.back().m_Tris.push_back(tri);
			}
			break;

		case TRI_MATERIAL:	//material mapping info
			char ch;
			i = 0;
			//get the material name that belongs to this mesh
			while((ch = fgetc(fle3DS)) != 0)
			{
				m_MeshList.back().matName[i] = ch;
				i++;
			}
			m_MeshList.back().matName[i] = '\0';

			//the number of verts in the mesh that has this material(not used)
			int NumEntries;
			unsigned long pos;

			NumEntries = ReadInt();
			pos = ftell(fle3DS);
			pos += (sizeof(WORD) * NumEntries);
			fseek(fle3DS,  pos, SEEK_SET);

			break;
		case TRI_MAPPINGCOORS:	//material mapping coords
			if(m_MeshList.size() == NULL)
				break;

			//get the number of pairs of tus and tvs
			DWORD NumMapCoords;
			NumMapCoords = ReadInt();

			m_MeshList.back().m_TexCoords.reserve(NumMapCoords);

			//then set theses to the mesh verts
			DWORD i;
			texcoord newtexcoord;
			for(i = 0; i < NumMapCoords; i++)
			{
				fread(&newtexcoord.s, sizeof(float), 1, fle3DS);
				fread(&newtexcoord.t, sizeof(float), 1, fle3DS);

				m_MeshList.back().m_TexCoords.push_back(newtexcoord);
			}
			break;

			break;
		case TRI_LOCAL:	//transformation matrix
			float tempMat[16];
			if(m_MeshList.size() == NULL)
				break;

			//get the matrix
			for(i = 0; i < 12; i++)
			{
				fread(&tempMat[i], sizeof(float), 1, fle3DS);
			}

			m_MeshList.back().Matrix.makeident();

			//then set it to our model mesh
			//m_MeshList.back().Matrix._11 = tempMat[0];
			//m_MeshList.back().Matrix._12 = tempMat[1];
			//m_MeshList.back().Matrix._13 = tempMat[2];
			
			//m_MeshList.back().Matrix._21 = tempMat[3];
			//m_MeshList.back().Matrix._22 = tempMat[4];
			//m_MeshList.back().Matrix._23 = tempMat[5];
			
			//m_MeshList.back().Matrix._31 = tempMat[6];
			//m_MeshList.back().Matrix._32 = tempMat[7];
			//m_MeshList.back().Matrix._33 = tempMat[8];

			//m_MeshList.back().Matrix._41 = tempMat[9];
			//m_MeshList.back().Matrix._43 = tempMat[10];
			//m_MeshList.back().Matrix._42 = tempMat[11];
			
			//m_MeshList.back().Matrix._14 = m_MeshList.back().Matrix._24 = m_MeshList.back().Matrix._34 = 0.0f;
			//m_MeshList.back().Matrix._44 = 1.0f;

			// normalize the matrix for now until i fix reading it correctly
			//m_MeshList.back().Matrix.makeident();
			m_MeshList.back().Matrix.rotate(-PI/2,0,0);



			break;
		case EDIT_OBJECT:	//object
			fseek(fle3DS, -6, SEEK_CUR);
			IsDone = true;
			break;
		default:
			//skip over any unknown chunks
			pos = ftell(fle3DS);
			if(pos - 6 >= count)
			{
				fseek(fle3DS, -6, SEEK_CUR);
				IsDone = true;
				break;
			}
			pos += (llen - 6);
			if(fseek(fle3DS, pos, SEEK_SET))
			{
				IsDone = true;
			}
		}
	}
}


//	this will read the materials information
//	like diffuse, ambient and specular
//	including some stuff related to the texture transformations
void s3dsMesh::ReadMaterials(DWORD Len)
{
	unsigned long count = ftell(fle3DS) + (Len - 6);	//position in the  file
	WORD wChunkID;	//tells what the chunk contains
	DWORD ChunkLen;	//the chunk length
	bool IsDone = false;
	bool IsAmbient = false, IsDiffuse = false, IsSpecular = false;	//tells us if and when to set color values

	//	make a new material and add it to the materials list
	sMaterial3ds newmaterial;
	m_MaterialList.push_back(newmaterial);

	while(IsDone == false)
	{
		if(feof(fle3DS))	//check for the end of the file
		{
			IsDone = true;
			break;
		}

		wChunkID = ReadInt();	//get the chunk info
		ChunkLen = ReadLong();	//get the chunk length

		switch(wChunkID)
		{
		case 0xA000:	//material name
			int i;
			char ch;
			i = 0;
			while((ch = fgetc(fle3DS)) != 0)
			{
				m_MaterialList.back().mName[i] = ch;
				i++;
			}
			m_MaterialList.back().mName[i] = '\0';
			break;
		case 0xA010:	//ambient flag
			IsAmbient = true;
			IsDiffuse = false;
			IsSpecular = false;

			m_MaterialList.back().HasMaterial = true;
			break;		//diffuse flag
		case 0xA020:
			IsAmbient = false;
			IsDiffuse = true;
			IsSpecular = false;

			m_MaterialList.back().HasMaterial = true;
			break;
		case 0xA030:	//specular flag
			IsAmbient = false;
			IsDiffuse = false;
			IsSpecular = true;
			
			m_MaterialList.back().HasMaterial = true;
			break;

		case 0xA200:	//texture map flag
			m_MaterialList.back().HasTextureMap = true;	//this material has a texture
			m_MaterialList.back().Vs = m_MaterialList.back().Us = m_MaterialList.back().uOff = m_MaterialList.back().vOff = m_MaterialList.back().tRot = 0.0f;
			break;
		case 0xA300:	//texture map filename
			i = 0;
			while((ch = fgetc(fle3DS)) != 0)
			{
				m_MaterialList.back().tFileName[i] = ch;
				i++;
			}
			m_MaterialList.back().tFileName[i] = '\0';
			break;
		case 0xA354:		//V scale
			fread(&(m_MaterialList.back().Vs), sizeof(float), 1, fle3DS);
			break;
		case 0xA356:		//U scale
			fread(&(m_MaterialList.back().Us), sizeof(float), 1, fle3DS);
			break;
		case 0xA358:		//U offset
			fread(&(m_MaterialList.back().uOff), sizeof(float), 1, fle3DS);
			break;
		case 0xA35A:		//V offset
			fread(&(m_MaterialList.back().vOff), sizeof(float), 1, fle3DS);
			break;
		case 0xA35C:		//	rotation of texture
			fread(&(m_MaterialList.back().tRot), sizeof(float), 1, fle3DS);
			break;	

		case 0x0011:		//color data
			int r, g, b;

			if(IsDiffuse == true)
			{
				//read the diffuse from file
				fread(&r, 1, 1, fle3DS);
				fread(&g, 1, 1, fle3DS);
				fread(&b, 1, 1, fle3DS);

				//then scale it to a 0-1.0 based proportion so that D3D materials can use it
				m_MaterialList.back().diffuse.r = PropBytetoFloat(r);
				m_MaterialList.back().diffuse.g = PropBytetoFloat(g);
				m_MaterialList.back().diffuse.b = PropBytetoFloat(b);

				IsDiffuse = false;
			}
			if(IsAmbient == true)
			{
				fread(&r, 1, 1, fle3DS);
				fread(&g, 1, 1, fle3DS);
				fread(&b, 1, 1, fle3DS);
				
				m_MaterialList.back().ambient.r = PropBytetoFloat(r);
				m_MaterialList.back().ambient.g = PropBytetoFloat(g);
				m_MaterialList.back().ambient.b = PropBytetoFloat(b);

				IsAmbient = false;
				
			}
			if(IsSpecular == true)
			{	
				fread(&r, 1, 1, fle3DS);
				fread(&g, 1, 1, fle3DS);
				fread(&b, 1, 1, fle3DS);
				
				m_MaterialList.back().specular.r = PropBytetoFloat(r);
				m_MaterialList.back().specular.g = PropBytetoFloat(g);
				m_MaterialList.back().specular.b = PropBytetoFloat(b);
	
				IsSpecular = false;
			}
			break;

		default:		//unknown(skipps the chunk)
			unsigned long pos;
			pos = ftell(fle3DS);
			if((pos - 6) >= count)
			{
				fseek(fle3DS, -6, SEEK_CUR);
				IsDone = true;
				break;
			}

			pos += (ChunkLen - 6);
			if(fseek(fle3DS, pos, SEEK_SET))
				IsDone = true;
			break;
		}
	}
}



//	this will adjust the object into a good position with a good size for vieweing
//	this works fine now because there is no hierarchy information
//	this function should produce problems and should be fixed when 
//	i support the hierachy
void s3dsMesh::Adjust()
{
	point3d AverageVertex(0,0,0);	//	this is the average vertex of the whole object
	int i,j;

	//	loop on all meshes
	for(i=0;i<m_MeshList.size();i++)
	{
		//	loop on all vertices inside the mesh and add 
		for(j=0;j<m_MeshList[i].m_Verts.size();j++)
			AverageVertex+=m_MeshList[i].m_Verts[j];
	}

	//	compute the average vertex
	AverageVertex=AverageVertex/dwTotalVerts;

	//	now subtract it from all the vertices again
	for(i=0;i<m_MeshList.size();i++)
	{
		//	loop on all vertices inside the mesh and subtract
		for(j=0;j<m_MeshList[i].m_Verts.size();j++)
			m_MeshList[i].m_Verts[j]-=AverageVertex;
	}

	//	now search for the vertex with the maximum magnitude
	float MaxMag=0;
	
	//	now subtract it from all the vertices again
	for(i=0;i<m_MeshList.size();i++)
	{
		//	loop on all vertices inside the mesh and check
		for(j=0;j<m_MeshList[i].m_Verts.size();j++)
		{
			if(m_MeshList[i].m_Verts[j].mag()>MaxMag)
			{
				MaxMag=m_MeshList[i].m_Verts[j].mag();
			}
		}
	}
	//	now scale the whole object by 1/maxmag
	Scale(0.707f/MaxMag);


	//	adjust each object alone
	for(i=0;i<m_MeshList.size();i++)
		m_MeshList[i].Adjust();
}

//	this will scale all the meshes , also should be modified when i support hierahcies
void s3dsMesh::Scale(float amount)
{
	//	loop on all meshes
	for(int i=0;i<m_MeshList.size();i++)
	{
		//	loop on all vertices inside the mesh
		for(int j=0;j<m_MeshList[i].m_Verts.size();j++)
			m_MeshList[i].m_Verts[j]*=amount;
	}
}



//	link the materials into the meshes
//	and also create the textures if required
//	all texture filenames will be converted into bmp
void s3dsMesh::LinkMaterials()
{
	//	destroy the textures if there was any
	m_TextureList.clear();


	//	loop on all meshes
	for(int i=0;i<m_MeshList.size();i++)
	{
		//	loop on all materials
		for(int j=0;j<m_MaterialList.size();j++)
		{
			//	if the names does match, make 
			//	a new texture and let the pointer inside the mesh point to it
			if(strlen(m_MeshList[i].matName)>3&&(strcmp(m_MeshList[i].matName,m_MaterialList[j].mName)==0))
			{
				m_MeshList[i].m_pMaterial=&m_MaterialList[j];

				//	now for any material which has a texture, allocate a texture for the object pointing to it
				if(m_MaterialList[j].HasTextureMap)
				{
					//	make a file name which is the same
					//	but make sure that it has the extension .bmp
					char BMPfilename[256];
					strcpy(BMPfilename,m_MaterialList[j].tFileName);

					int size=strlen(BMPfilename);
					BMPfilename[size-3]='b';
					BMPfilename[size-2]='m';
					BMPfilename[size-1]='p';
					m_TextureList.push_back(* (new cTexture(BMPfilename)));
					m_MeshList[i].lpTexture	=&m_TextureList.back();
				}
			}
		}
	}	
}



//	this will draw the 3ds file data object at the current transform
void s3dsMesh::Draw()
{
	if(bError == true)	//if the file is coorupt or dosent exist exit before we get memory access errors
		return;

	//loop through the meshes and draw them
	for(int i=0;i<m_MeshList.size();i++)
	{
		m_MeshList[i].Draw();
	}
}

//	will write some info about the object 
void s3dsMesh::WriteInfo()
{

	Graphics()->WriteText(0,0,0xffffff,FileName);
	Graphics()->WriteText(0,15,0xffffff,"%d meshes",m_MeshList.size());
	Graphics()->WriteText(0,30,0xffffff,"%d materials",m_MaterialList.size());
	Graphics()->WriteText(0,45,0xffffff,"%d textures",m_TextureList.size());
	Graphics()->WriteText(0,60,0xffffff,"%d vertices",this->dwTotalVerts);
	Graphics()->WriteText(0,75,0xffffff,"%d triangles",this->dwTotalFaces);
}


void s3dsMesh::WriteInfo(char * filename)
{
	ofstream outfile;
	outfile.open(filename);

	int i;

	outfile<<"========================================================"<<endl;

	//	write the file name
	outfile<<"Filename            : "<<FileName<<endl;


	outfile<<"Number of meshes    : "<< m_MeshList.size()<<endl;
	outfile<<"Number of materials : "<< m_MaterialList.size()<<endl;
	outfile<<"Number of textures  : "<< m_TextureList.size()<<endl;
	outfile<<"Number of vertices  : "<< dwTotalVerts<<endl;
	outfile<<"Number of triangles : "<< dwTotalFaces<<endl;
	

	outfile<<"========================================================"<<endl;

	//	now write the info about all the meshes
	for(i=0;i<m_MeshList.size();i++)
	{
		outfile<<"--------------------------------------------------------"<<endl;

		outfile<<"Mesh number         : "<<i+1<<endl;
		outfile<<"Mesh name           : "<<m_MeshList[i].mName<<endl;
		outfile<<"Number of vertices  : "<<m_MeshList[i].m_Verts.size()<<endl;
		outfile<<"Number of triangles : "<<m_MeshList[i].m_Tris.size()<<endl;
		outfile<<"Material Name       : "<<m_MeshList[i].matName<<endl;

		if(m_MeshList[i].lpTexture)
			outfile<<"Has Texture         : YES"<<endl;
		else
			outfile<<"Has Texture         : NO"<<endl;

		outfile<<"--------------------------------------------------------"<<endl;
	}

	//	now write the info about all the materials
	for(i=0;i<m_MaterialList.size();i++)
	{
		outfile<<"********************************************************"<<endl;

		outfile<<"Material number     : "<<i+1<<endl;
		outfile<<"Material name       : "<<m_MaterialList[i].mName<<endl;

		if(m_MaterialList[i].HasTextureMap)
		{
			outfile<<"Has Texture         : YES"<<endl;
			outfile<<"Texture name        : "<<m_MaterialList[i].tFileName<<endl;
		}
		else
			outfile<<"Has Texture         : NO"<<endl;

		outfile<<"********************************************************"<<endl;
	}
}


/*------------end of class c3dsObject implementation-----------*/