#ifndef _OPENGL_H
#define _OPENGL_H
/****************************************************************
*	opengl.h													*
*	OpenGL Header Class for the flulib  library					*
*	adds a Graphics layer based on OpenGL						*
****************************************************************/
#include	"graphics.h"		//	to inheret from the very basic graphics layer interface class
#include	"application.h"

//	includes for OpenGL
#include <gl\gl.h>				//	header file for the opengl32 library
#include <gl\glu.h>
#include <gl\glaux.h>


#include "glext.h"


//	additional functions which use extensions
void	glMultiTexCoord2f(float,float)		;
void	glActiveTexture(int)				;
void	glClientActiveTexture(int)			;
void	glLockArrays(int first,int count)	;
void	glUnlockArrays()					;


/****************************************************************
*	Class cOpenGLSubSystem : Encapsulates the OpenGl Graphics	*
*	class declaration											*
****************************************************************/
class cOpenGLSubSystem	: public cGraphicsSubSystem
{
private:

	void	InitGL();								//	this will start OpenGL
	void	EndGL(void);							//	the shutdown for OpenGL
	void	ReSizeGLScene(int width,int height);	//	used when resizing the window


	//	some variables related to creating and managing fonts
	unsigned int	m_OutLineFontBase;			//	base for the display list holding the outline font
	GLYPHMETRICSFLOAT	m_GMF[256];				//	the 256 3d characters
	float			m_OutLineFontDepth;			//	depth of the outline font

	//	functions to create a destroy bitmap fonts
	void			CreateBitMapFont(char* fontname,int fontsize);
	void			DestroyBitMapFont(){if(m_BitMapFontBase)glDeleteLists(m_BitMapFontBase,96);}



	//	functions to create and destroy 3d outiline fonts
	void			CreateOutLineFont(char* fontname,int fontsize,float depth);
	void			DestroyOutLineFont(){if(m_OutLineFontBase)glDeleteLists(m_OutLineFontBase,256);}
	
	//	this function will setup the 2d array of charcaters of the screen buffer...
	void	DrawTextBuffer();	//	used to draw all the characters of the text array
	void	DrawStr(char* str,int x,int y);//	the internal function that actually draw the strings..

	unsigned int	m_BitMapFontBase;			//	base for the display list holding the bitmap font


	//	font used for gdi text
	HFONT		m_GDIFont;				

	friend class	cTextWindow	;
	friend struct	sTextMesh	;


	//	this function will setup the necessary extensions
	void		CreateExtensions(void);


public:


	HGLRC		m_hRC			;		//	rendering context for OPENGL


	//	writes a 3d string in a 3d location
	void DrawText3D(char* str,float ypos=0.0);

	//	constructor initializes OpenGL
	cOpenGLSubSystem()
	{	InitGL();		Application()->m_ReportFile<<"OpenGL      subsystem Successfully initialized"<<endl;	}

	//	Desctructor shuts down OpenGL
	~cOpenGLSubSystem()
	{	EndGL();		Application()->m_ReportFile<<"OpenGL      subsystem Successfully ShutDown   "<<endl;	}


	//	this function will save a screen shot of the front buffer in a bitmap file
	void	SaveScreen(char* filename);


	//	text writing functions
	void	PutText		(char* str,int x,int y)	{DrawStr(str,x*m_FontWidth,y*m_FontHeight);}

	void	WriteText	(int x,int y,int color,const char *sentence,...);
	void	WriteMode	(void);
	void	WriteTime	(void);
	void	WriteFPS	(void);
	

	//	set the world transform before calling this function
	void	BlitText(char* str,int ix,int iy)	{	DrawStr(str,ix*m_FontWidth,iy*m_FontHeight);	}
	

	//	frame management functions
	void	InitDrawing2D	(void)			;
	void	EndDrawing2D	(void)			;
	void	UpdateRects		(void)			{													}
	void	Flip			(void)			
	{
		if(Application()->m_InThread&&!Application()->m_IsDead)	
			DrawTextBuffer();
		SwapBuffers(m_hDC);
	}
	void	Clear			(void)			{glClear(GL_COLOR_BUFFER_BIT);}


	//	sets the font used for gdi text functions
	void WriteGDIText(char * str,int color,int x,int y);
	//	uses the gdi font to write string ceneterd in the screen
	void WriteTextCentered(char *sentence,int color,int ypos=-1,int numchars=-1);
};
/*------end of class cOpenGLSubSystem declaration--------------------*/

//			end class _Graphics3d implementation
inline cOpenGLSubSystem * GraphicsGL(){ return (cOpenGLSubSystem*)cOpenGLSubSystem::GetSingletonPtr(); }

#endif // _OPENGL_H
