#include "library.h"
#include "sceneload.h"
#include "scenedata.h"
#include "scene.h"

/********************************************************************
*	scenes.cpp														*
*	By Shady El Mously												*
*	this is the implementation for the Scene Descriptors object		*
*	here all the functions required for playing the 3d scene		*
*	are implemeted													*
********************************************************************/



/********************************************************************
*	class cSceneData												*
*	used to describe a 3d scene										*
*	the scene must be described in a text file						*
*	this base class will be used to make simple static scenes		*
*	containing objects, lights,a camera								*
*	the scene ends by pressing SPACE								*
********************************************************************/


//	 constructor always need a file name
cSceneData::cSceneData(char* FileName)
{
	m_pMarquee	=	NULL			;
	m_pMenu		=	NULL			;
	m_pMusic	=	NULL			;
	m_pViewer	=	NULL			;
	m_pFog		=	NULL			;
	m_FrameRate	=	100				;
	m_BackColor	=	color3::Black	;
	strcpy(m_FileName,FileName)		;
	m_bKeyboardControl	=	false;
	//	set view parameters
	m_HeadResponse.SetParams(240,0.9f);
	m_KeysResponse.SetParams(240,0.9f);
}

//	this will create the whole scene descritption from a text file describing it
int cSceneData::Create(c3DScene* pScenePlayer)
{
	//	make a scene loader
	cSceneLoader	SceneLoader(this);
	SceneLoader.Load();

	//	first lock the frame rate
	FrameController()->SetFps(m_FrameRate);

	//	set the clear color
	glClearColor(m_BackColor.r,m_BackColor.g,m_BackColor.b,0);

	//	activate the camera and the view port
	m_Camera.Activate();
	m_ViewPort.Activate();

	int i=0;	//	looping index

	//	set up everything before starting to run the scene
	pScenePlayer->m_NumParticles	=	m_ParticleSystems.size();
	pScenePlayer->m_NumObjects		=	m_ObjectsList.size();
	pScenePlayer->m_NumBounded		=	m_BoundedObjects.size();
	pScenePlayer->m_NumRigidBodies	=	m_RigidBodies.size();
	pScenePlayer->m_NumPaths		=	m_PathsList.size();
	pScenePlayer->m_NumPathLinks	=	m_PathLinksList.size();
	pScenePlayer->m_NumPointLinks	=	m_PointLinksList.size();
	pScenePlayer->m_NumLights		=	m_LightsList.size();

	//	pointer to the particle systems
	pScenePlayer->m_pParticles	=	m_ParticleSystems.begin();

	//	use a pointer to loop on the objects
	pScenePlayer->m_pObjects	=	(c3DObject**)m_ObjectsList.begin();

	//	pointer to bounded objects
	pScenePlayer->m_pBounded	=	m_BoundedObjects.begin();

	//	use a pointer to loop on the rigid bodies
	pScenePlayer->m_pRigidBodies	=	m_RigidBodies.begin();

	//	pointer to paths
	pScenePlayer->m_pPath		=	m_PathsList.begin();

	//	pointer to path links
	pScenePlayer->m_pPathLink	=	m_PathLinksList.begin();

	//	pointer to point links
	pScenePlayer->m_pPointLink=	m_PointLinksList.begin();

	//	pointer to light
	pScenePlayer->m_pLights		=	m_LightsList.begin();

	//	pointer to the camera
	pScenePlayer->m_pCamera	=	&m_Camera;



	//	initialize the particle systems
	for(i=0;i<pScenePlayer->m_NumParticles;i++)
	{
		pScenePlayer->m_pParticles[i]->InitializeSystem();
	}


	//	if there is a music, start playing it
	if(m_pMusic)		m_pMusic->Play();

	//	if there is a marquee..start it
	if(m_pMarquee)		m_pMarquee->Start();


	//	activate the fog of there is one
	if(m_pFog)		m_pFog->Activate();

	//	now it is loaded
	pScenePlayer->m_bIsLoaded	=	true;

	return 1;
}

//	the inverse of create, deallocates all objects, lights and paths of the scene
//	if a menu button is clicked, i should call it
//	the call again Create when back 
void cSceneData::Kill()
{
	int i;	//	loops index
	
	//	destroy the lights
	for(i=0;i<m_LightsList.size();i++)
		SAFE_DELETE(m_LightsList[i]);

	//	destroy the objects
	for(i=0;i<m_ObjectsList.size();i++)
		SAFE_DELETE(m_ObjectsList[i]);

	//	destroy the paths
	for(i=0;i<m_PathsList.size();i++)
		SAFE_DELETE(m_PathsList[i]);

	//	destroy the particle systems
	for(i=0;i<m_ParticleSystems.size();i++)
	{
		m_ParticleSystems[i]->KillSystem();
		SAFE_DELETE(m_ParticleSystems[i]);
	}
		
	//	destroy the marquee text object if it exists
	SAFE_DELETE(m_pMarquee);

	//	destroy the menu if it exists
	SAFE_DELETE(m_pMenu);

	//	destroy the music if it exists
	SAFE_DELETE(m_pMusic);

	//	delete the viewer if exists
	SAFE_DELETE(m_pViewer);

	//	delete the fog if it exists
	SAFE_DELETE(m_pFog);

	//	restore back color to black
	m_BackColor	= color3::Black	;

	//	clear the vectors from the NULLs they have
	m_LightsList.clear();
	m_ObjectsList.clear();
	m_PathsList.clear();
	m_PathLinksList.clear();
	m_VectorsList.clear();
	m_PointLinksList.clear();
	m_ParticleSystems.clear();

	//	clear the other vectors
	m_RigidBodies.clear();
	m_BoundedObjects.clear();

}
/*-------------end of class c3DScene implementation----------------*/

