
#include "sceneload.h"
#include "library.h"


/********************************************************************
*	sceneload.cpp													*
*	By Shady El Mously												*
*	this is the second implementation file for the Scene class		*
*	all functions related to loading the scene from the	description	*
*	file is here													*
********************************************************************/

//	this will load the data into the scene
int cSceneLoader::Load()
{
	//	open the file as read only, text
	m_Fptr	=	fopen(m_pScene->m_FileName,"rt");

	//	if null, return error
	if(!m_Fptr)
		return 0;

	//	now read the file
	if(!ReadData())
		return 0;

	//	and close the file
	fclose(m_Fptr);

	//	clear the names
	ClearNames();

	return 1;
}

//	the general format for the 3d scenes description files is as follows
//	any line that contains just one string, will be a header for the data coming after it
//	any other line will have 3 strings, the first is the name of the attribute
//	the second is always the = sign
//	and the third is the attribute value
//	all attribute names should be in small letters

//	this will prepare a line of text, skipping any empty lines
void cSceneLoader::PrepareLine()
{
	//	only leave here is the file has ended 
	//	only we found something in the line
	bool Done=false;

	while(!Done)
	{
		if(feof(m_Fptr))
			break;

		cString::ReadTokenLine(m_Fptr);

		if(m_LineWords.size())
			Done	=	true;
	}
}


//	clear the names when done reading the file describing the scene
void cSceneLoader::ClearNames()
{
	//	clear the names
	m_ObjectsNames.clear();
	m_PointsNames.clear();
	m_PathsNames.clear();
	m_PathLinksNames.clear();
	m_LightsNames.clear();
	m_PointLinksNames.clear();
	m_ParticlesNames.clear();
}


//	comment detection, always call after prepare 
bool cSceneLoader::IsComment()
{
	//	if the first charcater in the first is '#', return true
	//	this can be removed
	if(m_LineWords.size()==0)
		return true;
	if(((char*)m_LineWords[0])[0]=='#')
			return true;
	return false;
}

//	new section detection
bool cSceneLoader::NewSection()
{
	//	new section means a single string which isnt a part of a comment
	//	i should always call this after making sure this isnt a comment
	if(m_LineWords.size()==1)
		return true;
	return false;
}


//	returns the index of the object name, return -1 if no such name exists
int	cSceneLoader::GetObjectID(char* objectname)
{
	int ID;
	for(ID=0;ID<m_ObjectsNames.size();ID++)
	{
		if(strcmp(m_ObjectsNames[ID].c_str(),objectname)==0)
			return ID;
	}
	return -1;//doesnt exist
}

//	returns the index of the point name, return -1 if no such name exists
int	cSceneLoader::GetPointID(char* pointname)
{
	int ID;

	for(ID=0;ID<m_PointsNames.size();ID++)
	{
		if(strcmp(m_PointsNames[ID].c_str(),pointname)==0)
			return ID;
	}
	return -1;//doesnt exist
}

//	returns the index of the point name, return -1 if no such name exists
int	cSceneLoader::GetPathID(char* pathname)
{
	int ID;

	for(ID=0;ID<m_PathsNames.size();ID++)
	{
		if(strcmp(m_PathsNames[ID].c_str(),pathname)==0)
			return ID;
	}
	return -1;//doesnt exist
}

//	returns the index of the light name, return -1 if no such name exists
int	cSceneLoader::GetLightID(char* pathname)
{
	int ID;

	for(ID=0;ID<m_LightsNames.size();ID++)
	{
		if(strcmp(m_LightsNames[ID].c_str(),pathname)==0)
			return ID;
	}
	return -1;//doesnt exist
}

//	returns the index of the particle system name, return -1 if no such name exists
int	cSceneLoader::GetParticleSystemID(char* particlesystemname)
{
	int ID;

	for(ID=0;ID<m_ParticlesNames.size();ID++)
	{
		if(strcmp(m_ParticlesNames[ID].c_str(),particlesystemname)==0)
			return ID;
	}
	return -1;//doesnt exist
}



//	this will read all the data inside the file
int cSceneLoader::ReadData()
{
	//	we return into this loop again when any new section begins
	bool firsttime=true;

	while(1)
	{
		//	break if the file ended
		if(feof(m_Fptr))
			break;
		// else, prepare a line
		if(firsttime)	PrepareLine();

		//	if this is a comment, skip..
		if(IsComment())
			continue;

		if(NewSection())
		{
			if(m_LineWords[0]=="scene")				ReadScene();
			else
			if(m_LineWords[0]=="object")			ReadObject();
			else
			if(m_LineWords[0]=="light")				ReadLight();
			else
			if(m_LineWords[0]=="camera")			ReadCamera();
			else
			if(m_LineWords[0]=="path")				ReadPath();
			else
			if(m_LineWords[0]=="pathlink")			ReadPathLink();
			else
			if(m_LineWords[0]=="point3d")			ReadPoint3d();
			else
			if(m_LineWords[0]=="pathlink")			ReadPathLink();
			else
			if(m_LineWords[0]=="marquee")			ReadMarquee();
			else
			if(m_LineWords[0]=="pointlink")			ReadPointLink();
			else
			if(m_LineWords[0]=="particles")			ReadParticleSystem();
			else
			if(m_LineWords[0]=="fog")				ReadFog();
			else	//	this is something unknown
				Application()->ReportError("Unknown Section!");
		}
		firsttime	=	false;
	}
	return 1;
}

//	this will read the scene header
int cSceneLoader::ReadScene()
{
	//	until now the only things we can read about the scene as a whole is the
	//	frame rate and wether or not ot displays the frate rate while running
	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())			break;
	
		//	now make a check on the first string
		if(m_LineWords[0]=="framerate")
			m_pScene->m_FrameRate=atoi(m_LineWords[2]);
		else//	else, the show frame rate flag
		if(m_LineWords[0]=="showframerate")
			m_pScene->m_bShowFps	=	(atoi(m_LineWords[2])==0) ? false : true;
		else	//	else, the view port data
		if(m_LineWords[0]=="viewport")
			m_pScene->m_ViewPort.Set(atoi(m_LineWords[2]),atoi(m_LineWords[3]),atoi(m_LineWords[4]),atoi(m_LineWords[5]));
		else	//	else the low bound of world dimensions (boxed world
		if(m_LineWords[0]=="minpoint")
			m_pScene->m_MinPoint	=	point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else	//	else the high bound of world dimensions (boxed world
		if(m_LineWords[0]=="maxpoint")
			m_pScene->m_MaxPoint	=	point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else	//	else the menu name if the scene has a menu
		if(m_LineWords[0]=="menu")
			m_pScene->m_pMenu	=	new c3DMenu(m_LineWords[2]);
		else	//	if there is a next scene in the chain
		if(m_LineWords[0]=="nextscene")
			strcpy(m_pScene->m_NextSceneFilename,m_LineWords[2].c_str());
		else	//	music file name
		if(m_LineWords[0]=="music")
			m_pScene->m_pMusic	=	new cMusic(m_LineWords[2]);
		else	//	backgrounf color
		if(m_LineWords[0]=="backcolor")
			m_pScene->m_BackColor	=	color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else	//	something is wrong
			Application()->ReportError("Unknow scene attribute type!");
	}
	return 1;
}



//	read an object description
int cSceneLoader::ReadObject()
{
	//	reading a new object
	//	we will read line after line
	//	until we find a line with just one string
	//	only then the function should leave

	cRigidBody * pNewObject;	//	pointer to the object which will be created now
	pNewObject	=	new cRigidBody;


	//	in which vector to add the object
	//	can be in normal objects or in rigid bodies
	int AddTo	=	1;	//1 for normal, 2 for rigid bodies, 3 for bounded object

	//	what can be read about the object is
	
	//	first : the object type (should come at first)
	//	which may be one of these :
	//	box , followed by 3 floats describing its dimensions
	//	sphere, followed by a single float which is its radius
	//	water	,	 for a water object, folowed by 2 floats and 2 integers
	//	terrain ,	followed by 3 floats (dimenstions and scale) and a file name 
	//	h3d , name of an h3d file
	//	o3d , name of an o3d file
	//	3ds , name of a 3ds file

	//	additional stuff like :
	//	texture : a texture file name for the object (string)
	//	position : initial position for the object (3 floats)
	//	orientation: initial orintatin angles for the objects (3 floats)

	//	also..
	//	transvel	:	3 floats (translation velocity)
	//	rotvel		:	3 floats (roation velocity)
	//	transacc	:	3 floats (translation acceleration)
	//	rotacc		:	3 floats (roation acceleration)
	
	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())			break;


		//	the controller type (mouse)
		if(m_LineWords[0]=="cursor")
		{
			SAFE_DELETE(pNewObject);

			float omega=200;
			float zeta=0.7f;

			//	if the response parameters are mentioned use them
			if(m_LineWords.size()>4)
			{
				omega	=	atof(m_LineWords[3]);
				zeta	=	atof(m_LineWords[4]);
			}
			pNewObject	=	new c3DCursor(omega,zeta);
			AddTo	=	3;
		}
		else		//	if that was the object type:
		if(m_LineWords[0]=="type")
		{
			//	make a check on the 3rd string

			//	if this is a box
			if(m_LineWords[2]=="box")
				pNewObject->CreateBox(atof(m_LineWords[3]),atof(m_LineWords[4]),atof(m_LineWords[5]));
			else	//	if it is a sphere
			if(m_LineWords[2]=="sphere")
				pNewObject->CreateSphere(atof(m_LineWords[3]));
			else	//	if it is a disk
			if(m_LineWords[2]=="disk")
				pNewObject->CreateDisk(atof(m_LineWords[3]),atof(m_LineWords[4]));
			else	//	if it is a cylinder
			if(m_LineWords[2]=="cylinder")
				pNewObject->CreateCylinder(atof(m_LineWords[3]),atof(m_LineWords[4]),atof(m_LineWords[5]));
			else	//	h3d files
			if(m_LineWords[2]=="h3d")
				pNewObject->CreateObject(m_LineWords[3]);
			else	//	o3d files
			if(m_LineWords[2]=="o3d")
				pNewObject->CreateObject(m_LineWords[3]);
			else	//	3ds files
			if(m_LineWords[2]=="3ds")
				pNewObject->CreateObject(m_LineWords[3]);
			else	//	text object
			if(m_LineWords[2]=="text")
				pNewObject->CreateText(m_LineWords[3]);
			else	//	heightfield terrain
			if(m_LineWords[2]=="terrain")
				pNewObject->CreateTerrain(1,1,m_LineWords[3],atof(m_LineWords[4]));
			else	//	water mesh
			if(m_LineWords[2]=="water")
				pNewObject->CreateWater(atof(m_LineWords[3]),atof(m_LineWords[4]),atoi(m_LineWords[5]),atoi(m_LineWords[6]));
			else	//	stars mesh
			if(m_LineWords[2]=="stars")
				pNewObject->CreateStars(atoi(m_LineWords[3]));
			else
				Application()->ReportError("Unknown object type!");
		}
		else	//	else, the object name
		if(m_LineWords[0]=="name")
			m_ObjectsNames.push_back(m_LineWords[2]);
		else	//	else, if the texture name
		if(m_LineWords[0]=="texture")
			pNewObject->SetTexture(m_LineWords[2]);
		else	//	else if the position
		if(m_LineWords[0]=="position")
			pNewObject->SetPos(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else	//	else if the orientation
		if(m_LineWords[0]=="orientation")
			pNewObject->RotateLocal(atof(m_LineWords[2])*PI/180,atof(m_LineWords[3])*PI/180,atof(m_LineWords[4])*PI/180);
		else//	else if the scale
		if(m_LineWords[0]=="scale")
			pNewObject->Scale(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	else if this is the translation velocity
		if(m_LineWords[0]=="transvel")
		{
			pNewObject->m_TransVel=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
			AddTo	=	2;
		}
		else//	else if this is the translation accelearation
		if(m_LineWords[0]=="transacc")
		{
			pNewObject->m_TransAcc=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
			AddTo	=	2;
		}
		else//	else if this is the rotational velocity
		if(m_LineWords[0]=="rotvel")
		{
			pNewObject->m_RotVel=point3d(atof(m_LineWords[2])*PI/180,atof(m_LineWords[3])*PI/180,atof(m_LineWords[4])*PI/180);
			AddTo	=	2;
		}
		else//	else if this is the rotational acceleration
		if(m_LineWords[0]=="rotacc")
		{
			pNewObject->m_RotAcc=point3d(atof(m_LineWords[2])*PI/180,atof(m_LineWords[3])*PI/180,atof(m_LineWords[4])*PI/180);
			AddTo	=	2;
		}
		else//	the diffuse color
		if(m_LineWords[0]=="diffuse")
			pNewObject->m_Material.diffuse=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	is it bounded ?
		if(m_LineWords[0]=="bounded")
			AddTo	=	3;
		else//	the specular color
		if(m_LineWords[0]=="specular")
			pNewObject->m_Material.specular=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	the ambient color
		if(m_LineWords[0]=="ambient")
			pNewObject->m_Material.ambient=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else
			Application()->ReportError("Unknown object attribute!");
	}

	m_pScene->m_ObjectsList.push_back(pNewObject);

	//	add the new object to the objects list
	//	now check AddTo...
	switch(AddTo)
	{
	case 1:
		
		break;
	case 2:
		m_pScene->m_RigidBodies.push_back(pNewObject);
		break;

	case 3:
		m_pScene->m_RigidBodies.push_back(pNewObject);
		m_pScene->m_BoundedObjects.push_back(pNewObject);
		break;
	}
	return 1;
}

//	read a light description 
int cSceneLoader::ReadLight()
{
	//	reading the lights requires reading first the type which could be
	//	spot, point or directional
	//	other things to read are 
	//	light position, direction
	//	diffuse specular and ambient colors also
	sLight	*	pNewLight;
	

	int LightType=0;	//	0 is directional, 1 is point and 2 is spot
	point3d	LightPos	=	point3d(0,0,0)	;	
	point3d	LightDir	=	point3d(0,0,1)	;	//	this will be the target for spot light
	color3	Diffuse		=	color3::White;
	color3	Specular	=	color3::Black;
	color3	Ambient		=	color3::Black;
	float Theta=0,Phi=PI/4;	//	angles for the spot light 
	float atten0=0.3f,atten1=0.2f,atten2=0.3f;	//	attenuation factors for the light (point source lights)

	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())			break;

		//	if that was the object type:
		if(m_LineWords[0]=="type")
		{
			//	make a check on the 3rd string

			//	if this is a directional light
			if(m_LineWords[2]=="directional")
				LightType=0;
			else	//	if it is a point light
			if(m_LineWords[2]=="point")
				LightType=1;
			else	//	spot light, must be more 2 values for the angles
			if(m_LineWords[2]=="spot")
			{
				LightType=2;
				Theta	=	atof(m_LineWords[3]);
				Phi		=	atof(m_LineWords[4]);
			}
			else
				Application()->ReportError("unknown light type");
		}
		else//	if this is the name
		if(m_LineWords[0]=="name")
			m_LightsNames.push_back(m_LineWords[2]);
		else//	else, this is the light direction
		if(m_LineWords[0]=="direction")
			LightDir=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	else, this is the light position
		if(m_LineWords[0]=="position")
			LightPos=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	else this is the diffuse color
		if(m_LineWords[0]=="diffuse")
			Diffuse=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	else this is the specular
		if(m_LineWords[0]=="specular")
			Specular=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	else this is the ambient
		if(m_LineWords[0]=="ambient")
			Ambient=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	constant attenunation factor
		if(m_LineWords[0]=="attenuation0")
			atten0=atof(m_LineWords[2]);
		else//	linear attenation factor
		if(m_LineWords[0]=="attenuation1")
			atten1=atof(m_LineWords[2]);
		else//	quadratic attenuation factor
		if(m_LineWords[0]=="attenuation2")
			atten2=atof(m_LineWords[2]);
		else//	anything else is an error
			Application()->ReportError("error in scene description file");
	}



	//	now switch on the light type we just read
	switch(LightType)
	{
	case 0:	//	directional
		pNewLight	=	new sDirectionalLight(LightDir,Diffuse,Specular,Ambient);
		break;
	case 1:	//	point source
		pNewLight	=	new sPointLight(LightPos,Diffuse,Specular,Ambient);
		((sPointLight*)pNewLight)->dvAttenuation0=atten0;
		((sPointLight*)pNewLight)->dvAttenuation1=atten1;
		((sPointLight*)pNewLight)->dvAttenuation2=atten2;

		break;
	case 2:	//	spot
		pNewLight	=	new sSpotLight(LightPos,LightDir,Theta,Diffuse,Specular,Ambient);
		((sSpotLight*)pNewLight)->dvAttenuation0=atten0;
		((sSpotLight*)pNewLight)->dvAttenuation1=atten1;
		((sSpotLight*)pNewLight)->dvAttenuation2=atten2;
		break;
	}

	
	//	now add the light to all the lights in the scene
	m_pScene->m_LightsList.push_back(pNewLight);
	return 1;
}


//	read a point3d section
int cSceneLoader::ReadPoint3d()
{
	//	a point3d will make a pointer which will
	//	point to a certain 3d value inside anything
	//	could be inside an object location, diffuse
	//	a file name contaning the path description
	///	and the path type which is one of 4 types
	//	easein,easeinout,linear,bezier
	//	also, the number of steps for the path can be given

#define OBJECT_POSITION		0x0000
#define OBJECT_DIFFUSE		0x0001
#define OBJECT_SPECULAR		0x0002
#define OBJECT_AMBIENT		0x0003
#define OBJECT_ROTVEL		0x0004
#define OBJECT_ROTACC		0x0005
#define OBJECT_TRANSVEL		0x0006
#define OBJECT_TRANSACC		0x0007
#define	PARTICLES_POSITION	0x0008
#define	LIGHT_POSITION		0x0009
#define	LIGHT_DIRECTION		0x000A
#define	LIGHT_DIFFUSE		0x000B
#define	LIGHT_SPECULAR		0x000C
#define	LIGHT_AMBIENT		0x000D
#define	CAMERA_POSITION		0x000E
#define	CAMERA_TARGET		0x000F
#define	CAMERA_UP			0x0010



	int		PointType=OBJECT_POSITION;
		
	
	int	DestID=-1;		//	the destid, could be a number of a light or of an object
	string Name;		//	name of the thing we refer to

	while(1)
	{
		//	if the file ended, leave immediatly
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())			break;

		//	the field name should create a new name for a new point
		if(m_LineWords[0]=="name")
		{
			m_PointsNames.push_back(m_LineWords[2]);
		}
		else
			//	if this is the type of link, may be one of 11 things
			if(m_LineWords[0]=="type")
			{
				//	object location 0
				if(m_LineWords[2]=="objectposition")
					PointType=OBJECT_POSITION;
				else//	object diffuse color 1				
				if(m_LineWords[2]=="objectdiffuse")
					PointType=OBJECT_DIFFUSE;
				else//	object specular color 2				
				if(m_LineWords[2]=="objectspecular")
					PointType=OBJECT_SPECULAR;
				else//	object ambient color 3
				if(m_LineWords[2]=="objectambient")
					PointType=OBJECT_AMBIENT;
				else//	object rotvel 4			
				if(m_LineWords[2]=="objectrotvel")
					PointType=OBJECT_ROTVEL;
				else//	object rotacc 5				
				if(m_LineWords[2]=="objectrotacc")
					PointType=OBJECT_ROTACC;
				else//	object transvel 6
				if(m_LineWords[2]=="objecttransvel")
					PointType=OBJECT_TRANSVEL;
				else//	object translational acceleration 7
				if(m_LineWords[2]=="objecttransacc")
					PointType=OBJECT_TRANSACC;
				else//	light position 8
				if(m_LineWords[2]=="lightposition")
					PointType=LIGHT_POSITION;
				else//	light direction or target 9
				if(m_LineWords[2]=="lightdirection")
					PointType=LIGHT_DIRECTION;
				else//	light diffuse
				if(m_LineWords[2]=="lightdiffuse")
					PointType=LIGHT_DIFFUSE;
				else//	light diffuse
				if(m_LineWords[2]=="lightspecular")
					PointType=LIGHT_SPECULAR;
				else//	light diffuse
				if(m_LineWords[2]=="lightambient")
					PointType=LIGHT_AMBIENT;
				else//	camera location 10
				if(m_LineWords[2]=="cameraposition")
					PointType=CAMERA_POSITION;
				else//	camera target 11
				if(m_LineWords[2]=="cameratarget")
					PointType=CAMERA_TARGET;
				else//	camera worldup 12
				if(m_LineWords[2]=="cameraup")
					PointType=CAMERA_UP;
				else//	particle system position
				if(m_LineWords[2]=="particlespos")
					PointType=PARTICLES_POSITION;
				else
					Application()->ReportError("Unknown point3d type");

				//	for anything that has a name..get the name
				if(PointType>=OBJECT_POSITION&&PointType<LIGHT_AMBIENT)
					Name=m_LineWords[3];
		}
	}

	//	now we can create the link, first read the dest id if the first 9 types (anything but the camera
	if(PointType>=OBJECT_POSITION&&PointType<=OBJECT_TRANSACC)
		DestID	=	GetObjectID((char*)Name.c_str());

	if(PointType>=LIGHT_POSITION&&PointType<=LIGHT_AMBIENT)
		DestID	=	GetLightID((char*)Name.c_str());

	if(PointType==PARTICLES_POSITION)
		DestID	=	GetParticleSystemID((char*)Name.c_str());

	
	//	make sure this is a vlid id before proceeding
	if(PointType>=OBJECT_POSITION&&PointType<=LIGHT_AMBIENT)
	if(DestID==-1)
	{
		Application()->ReportError("No object has such name");
	}


	//	now fill the destination point of the link
	switch(PointType)
	{
	case OBJECT_POSITION:	
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_ObjectsList[DestID]->m_WorldMatrix._41);
		break;
	case OBJECT_DIFFUSE:	
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_ObjectsList[DestID]->m_Material.diffuse);
		break;
	case OBJECT_SPECULAR:
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_ObjectsList[DestID]->m_Material.specular);
		break;
	case OBJECT_AMBIENT:
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_ObjectsList[DestID]->m_Material.ambient);
		break;
	case OBJECT_ROTVEL:
		m_pScene->m_VectorsList.push_back(&m_pScene->m_ObjectsList[DestID]->m_RotVel);
		break;
	case OBJECT_ROTACC:
		m_pScene->m_VectorsList.push_back(&m_pScene->m_ObjectsList[DestID]->m_RotAcc);
		break;
	case OBJECT_TRANSVEL:
		m_pScene->m_VectorsList.push_back(&m_pScene->m_ObjectsList[DestID]->m_TransVel);
		break;
	case OBJECT_TRANSACC:
		m_pScene->m_VectorsList.push_back(&m_pScene->m_ObjectsList[DestID]->m_TransAcc);
		break;
	case PARTICLES_POSITION:
		m_pScene->m_VectorsList.push_back(&m_pScene->m_ParticleSystems[DestID]->m_Origin);
		break;
	case LIGHT_POSITION:
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_LightsList[DestID]->dvPosition);
		break;
	case LIGHT_DIRECTION:
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_LightsList[DestID]->dvDirection);
		break;
	case LIGHT_DIFFUSE:
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_LightsList[DestID]->dcvDiffuse);
		break;
	case LIGHT_SPECULAR:
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_LightsList[DestID]->dcvSpecular);
		break;
	case LIGHT_AMBIENT:
		m_pScene->m_VectorsList.push_back((point3d*)&m_pScene->m_LightsList[DestID]->dcvAmbient);
		break;
	case CAMERA_POSITION:
		m_pScene->m_VectorsList.push_back(&m_pScene->m_Camera.m_Loc);
		break;
	case CAMERA_TARGET:
		m_pScene->m_VectorsList.push_back(&m_pScene->m_Camera.m_At);
		break;
	case CAMERA_UP:
		m_pScene->m_VectorsList.push_back(&m_pScene->m_Camera.m_Up);
		break;
	}

	//	now add the new path link to the local list
	return 1;
}
//	read the camera description
int cSceneLoader::ReadCamera()
{
	//	reading the camera requires reading 3 things
	//	the camera location, target and worlup vector

	//	set the world up default (0,1,0)
	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())				continue;
		//	if this is a new section, break from here
		if(NewSection())			break;

		//	if that we want to control the camera by something
		if(m_LineWords[0]=="controlby")
		{
			//	if controlled by mouse ( yaw and pitch only)
			if(m_LineWords[2]=="mouse")
			{
				SAFE_DELETE(m_pScene->m_pViewer);
				m_pScene->m_pViewer	=	new cMouseViewer;
			}
			else	//	if helmet ( yaw, pitch and roll )
			if(m_LineWords[2]=="helmet")
			{
				SAFE_DELETE(m_pScene->m_pViewer);
				m_pScene->m_pViewer	=	new cHelmetViewer;
			}
			else
				Application()->ReportError("Uknown camera controller type");

			//	if the response parameters were involved.. use them
			if(m_LineWords.size()>4)
				m_pScene->m_HeadResponse.SetParams(atof(m_LineWords[3]),atof(m_LineWords[4]));

		}
		else	//	if the canera target
		//	if that was the camera location
		if(m_LineWords[0]=="position")
			m_pScene->m_Camera.m_Loc=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else	//	if the canera speed
		if(m_LineWords[0]=="speed")
			m_pScene->m_Camera.m_Speed=atof(m_LineWords[2]);
		else	//	if the canera target
		if(m_LineWords[0]=="target")
			m_pScene->m_Camera.m_At=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else	//	if the camera world up vector
		if(m_LineWords[0]=="upvector")
			m_pScene->m_Camera.m_Up=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else	//	if the camera is controlled by the keyboard
		if(m_LineWords[0]=="keyboard"&&m_LineWords[2]=="yes")
		{
			m_pScene->m_bKeyboardControl=true;
			//	if the response parameters were involved.. use them
			if(m_LineWords.size()>4)
				m_pScene->m_KeysResponse.SetParams(atof(m_LineWords[3]),atof(m_LineWords[4]));
		}
		else
			Application()->ReportError("Unknown camera attribute!");
	}
	return 1;
}

//	read a path description
int cSceneLoader::ReadPath()
{
#define PATH_LINEAR		0x0000
#define PATH_EASEOUT	0x0001
#define PATH_EASEINOUT	0x0002
#define PATH_BEZIER		0x0003

	//	reading a path requires 2 things mainly
	//	a file name contaning the path description
	///	and the path type which is one of 4 types
	//	easein,easeinout,linear,bezier
	//	also, the number of steps for the path can be given

	int		PathType=PATH_LINEAR;	//	0=linear, 1=easeout , 2=easeinout , 3=bezier
	char	FileName[256];

	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())				continue;
		//	if this is a new section, break from here
		if(NewSection())			break;

		//	if this is the path type..
		if(m_LineWords[0]=="type")
		{
			if(m_LineWords[2]=="linear")
				PathType=PATH_LINEAR;
			else
			if(m_LineWords[2]=="easeout")
				PathType=PATH_EASEOUT;
			else
			if(m_LineWords[2]=="easeinout")
				PathType=PATH_EASEINOUT;
			else
			if(m_LineWords[2]=="bezier")
				PathType=PATH_BEZIER;
			else
			Application()->ReportError("Unknown path type!");
		}
		else	// the path name
		if(m_LineWords[0]=="name")
			m_PathsNames.push_back(m_LineWords[2]);
		else	// the filename
		if(m_LineWords[0]=="filename")
			strcpy(FileName,m_LineWords[2]);
		else
			Application()->ReportError("Uknown path attribute!");			
	}

	
	//	now create the path
	cPath* pNewPath;

	//	check the path type
	switch(PathType)
	{
		case PATH_LINEAR:
			pNewPath	=	new cPatternPath(FileName);
			break;
		case PATH_EASEOUT:
			pNewPath	=	new cEaseOutPath(FileName);
			break;
		case PATH_EASEINOUT:
			pNewPath	=	new cEaseInOutPath(FileName);
			break;
		case PATH_BEZIER:
			pNewPath	=	new cBezierPath(FileName);
			break;
	}

	//	add the new path to paths list
	m_pScene->m_PathsList.push_back(pNewPath);
	return 1;
}

//	read a marquee description from a file and create the marquee object
//	only one of those section is needed as the scene has only one marquee object
//	all we read about the marque is two things, the type and the filename
//	the filename is essential
int cSceneLoader::ReadMarquee()
{
	int MarqueeType	=	0;	//	0=normal, 1=matrix..2=scroll
	char	FileName[256];
	char	*SoundName=NULL;

	cString NextSectionName;	//	i need to sote the next section name temporarly

	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())
		{
			NextSectionName=m_LineWords[0];
			break;
		}

		//	if this is the path type..
		if(m_LineWords[0]=="type")
		{
			if(m_LineWords[2]=="normal")
				MarqueeType=0;
			else
			if(m_LineWords[2]=="matrix")
			{
				MarqueeType=1;
				if(m_LineWords.size()==4)
				{
					SoundName	=	new char[256];
					strcpy(SoundName,m_LineWords[3]);
				}
			}
			else
			if(m_LineWords[2]=="scroll")
				MarqueeType=2;
			else
				Application()->ReportError("unknown marquee type");

		}
		else	// the filename
			if(m_LineWords[0]=="filename")
				strcpy(FileName,m_LineWords[2]);
	}

	//	destroy the marque if it is already existing
	SAFE_DELETE(m_pScene->m_pMarquee);

	//	now create the marquee object
	switch(MarqueeType)
	{
	case 0:	//	normal
		m_pScene->m_pMarquee	=	new cMarquee(FileName);
		break;
	case 1://	matrix type
		m_pScene->m_pMarquee	=	new cMatrixMarquee(FileName,SoundName);
		break;

	case 2:
		m_pScene->m_pMarquee	=	new cScrollMarquee(FileName);
		break;
	}

	//	restore the next section name..the marquee has probably already read the file
	if(!feof(m_Fptr))
	{
		m_LineWords.clear();
		m_LineWords.push_back(NextSectionName);
	}

	if(SoundName)
		delete [] SoundName;
	return 1;
}


//	read a the fog description
//	only one fog ....
int cSceneLoader::ReadFog()
{
	//	fog parameters to be read
	float FogStart		=	0.0f		;
	float FogEnd		=	200.0f		;
	float Density		=	0.1f		;
	int Type			=	GL_LINEAR			;	//	1=linear, 2=exponential , 3= exponential2
	color3	FogColor	=	color3::Gray;	//	default fog color is gray

	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())			break;

		//	if this is the fog color
		if(m_LineWords[0]=="color")
		{
			FogColor.assign(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[3]));
		}
		else	//	the fog start
		if(m_LineWords[0]=="start")
		{
			FogStart	=	atof(m_LineWords[2]);
		}
		else	//	the fog end
		if(m_LineWords[0]=="end")
		{
			FogEnd		=	atof(m_LineWords[2]);
		}
		else	//	the fog density
		if(m_LineWords[0]=="density")
		{
			Density		=	atof(m_LineWords[2]);
		}
		else	//	the fog type
		if(m_LineWords[0]=="type")
		{
			if(m_LineWords[2]=="linear")
				Type=GL_LINEAR;
			else
			if(m_LineWords[2]=="exponential")
				Type=GL_EXP;
			else
			if(m_LineWords[2]=="exponential2")
				Type=GL_EXP2;
			else	//	unknown fog type...
				Application()->ReportError("Unknown fog type !");
		}
		else	//	error
			Application()->ReportError("Unkown fog attribute !");
	}
	//	create the scene fog
	SAFE_DELETE(m_pScene->m_pFog);
	m_pScene->m_pFog	=	new cFog(FogColor);
	m_pScene->m_pFog->Set(FogColor,FogStart,FogEnd,Type,Density);
	return 1;
}


//	a pth link lins a path which has a name
//	to a poin3d which also must have a name
//	i need to read the pathname and the point3d name only
int cSceneLoader::ReadPathLink()
{
	sPathLink	NewPathLink;	//	new path link to be filled
								//	will only be added to the added if both pointer are not nulls

	int	SrcID=-1;		//	the path id , index inside the paths vector
	int	DestID=-1;		//	the destid,  index inside the points vector

	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())			break;

		//	if this is the name of the path link
		//	add it to the pathlinksnames
		if(m_LineWords[0]=="name")
		{
			m_PathLinksNames.push_back(m_LineWords[2]);
		}
		else		//	first read the path id, starts from 1 to the number of paths
		if(m_LineWords[0]=="source")
			SrcID	=	GetPathID(m_LineWords[2]);
		else		//	if this is the type of link, may be one of 11 things
		if(m_LineWords[0]=="destination")
			DestID		=	GetPointID(m_LineWords[2]);
		else		//	the scale factor
		if(m_LineWords[0]=="scale")
			NewPathLink.m_Scale	=	atof(m_LineWords[2]);		
		else
			Application()->ReportError("Unknown pathlink attribute!");
	}

	//	check for valid names
	if(DestID==-1)
		Application()->ReportError("Unknown point3d name for pathlink");

	if(SrcID==-1)
		Application()->ReportError("Unknown path name for pathlink");


	NewPathLink.m_pPath	=	m_pScene->m_PathsList[SrcID];
	NewPathLink.m_pPoint=	m_pScene->m_VectorsList[DestID];

	//	now add the new path link to the local list
	m_pScene->m_PathLinksList.push_back(NewPathLink);
	return 1;
}

//	reading a particle system
int cSceneLoader::ReadParticleSystem()
{
	//	the pointer that will be added to the list once set
	cParticleSystem*	pNewParticleSystem=NULL;

	int		SystemType		=	1		;			//	1 for flame, 2 for snow, 3 for items
	int		NumParticles	=	50		;			//	default nuber of particles
	char	BitMapFileName[256]			;			//	name of the bitmapfile containg th skin for the particle system
	float	TimeScale		=	1.0f	;
	point3d	Source(0,0,0)				;			//	the source of the particles
	float	MinSize	=	0.0f;
	float	MaxSize	=	1.0f;

	//	attributes for the snow storm particle system
	float Width=0,Height=0,Depth=0;		//	width and height and depth ( required for snow storms )


	//	attributes for the flame partivle system
	color4	InitialColor				=	color4(1,1,1,1)		;
	color4  DeltaColor					=	color4(-0.01f,-0.01f,-0.01f,0);
	point3d	ParticleVelocity(0,0,0)								;
	point3d	ParticleAcceleration(0,0,0)								;
	point3d	ParticleVelocityVariation(0.2f,0.2f,0.2f)			;
	float	StartSize	=	0.1f								;
	float	DeltaSize	=	0.01f								;
	

	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())			break;

		//	if this is the name of the path link
		//	add it to the pathlinksnames
		if(m_LineWords[0]=="name")
		{
			m_ParticlesNames.push_back(m_LineWords[2]);
		}
		else		//	type of the particle system
		if(m_LineWords[0]=="type")
		{
			if(m_LineWords[2]=="snow")
				SystemType	=	2;
			else
			if(m_LineWords[2]=="flame")
				SystemType	=	1;
			else
			if(m_LineWords[2]=="items")
				SystemType	=	3;
			else
				Application()->ReportError("Unknown Particle System Type !");
		}
		else		//	position of the particle system
		if(m_LineWords[0]=="position")
			Source		=	point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else		//	bitmap of the particle
		if(m_LineWords[0]=="bitmap")
			strcpy(BitMapFileName,m_LineWords[2].c_str());
		else		//	width of the snow storm
		if(m_LineWords[0]=="width")
			Width=atof(m_LineWords[2]);
		else		//	width of the snow storm
		if(m_LineWords[0]=="height")
			Height=atof(m_LineWords[2]);
		else		//	width of the snow storm
		if(m_LineWords[0]=="depth")
			Depth=atof(m_LineWords[2]);
		else		//	number of particles
		if(m_LineWords[0]=="count")
			NumParticles=atoi(m_LineWords[2]);
		else		//	color of particles
		if(m_LineWords[0]=="color")
			InitialColor=color4(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]),1);
		else		//	velocity of particles
		if(m_LineWords[0]=="velocity")
			ParticleVelocity=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else		//	acceleration of particles
		if(m_LineWords[0]=="acceleration")
			ParticleAcceleration=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else		//	velocity variation of particles
		if(m_LineWords[0]=="velocityvariation")
			ParticleVelocityVariation=point3d(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else		//	start size of the patricles
		if(m_LineWords[0]=="startsize")
			StartSize	=	atof(m_LineWords[2]);
		else		//	delta size of the patricles
		if(m_LineWords[0]=="deltasize")
			DeltaSize	=	atof(m_LineWords[2]);
		else		//	delta color of particles
		if(m_LineWords[0]=="deltacolor")
			DeltaColor	=	atof(m_LineWords[2]);
		else		//	time scale for the system mentioned
		if(m_LineWords[0]=="timescale")
			TimeScale	=	atof(m_LineWords[2]);
		else		//	minimum size
		if(m_LineWords[0]=="minsize")
			MinSize	=	atof(m_LineWords[2]);
		else		//	maximum size
		if(m_LineWords[0]=="maxsize")
			MaxSize	=	atof(m_LineWords[2]);
		else		//	if this is the type of link, may be one of 11 things
			Application()->ReportError("Unknown particle system attribute!");
	}


	//	create the particle system
	switch(SystemType)
	{
	case 1:		//	a flame system
		pNewParticleSystem	=	new cFlame(NumParticles,Source,InitialColor,BitMapFileName);
		((cFlame*)pNewParticleSystem)->SetParams(InitialColor,ParticleVelocity,ParticleVelocityVariation,StartSize,DeltaSize);
		((cFlame*)pNewParticleSystem)->m_MinSize=MinSize;
		((cFlame*)pNewParticleSystem)->m_MaxSize=MaxSize;
		break;

	case 2:		//	a snow storm system
		pNewParticleSystem	=	new cSnowStorm(NumParticles,Source,Width,Height,Depth,BitMapFileName);
		((cSnowStorm*)pNewParticleSystem)->SetAttributes(point3d(0,-0.5f,0),point3d(0.5f,0.3f,0),0.1f,20);
		break;
	case 3:		//	a items system
		pNewParticleSystem	=	new cItems(NumParticles,Source,InitialColor,BitMapFileName);
		((cItems*)pNewParticleSystem)->SetParams(InitialColor,ParticleVelocity,ParticleVelocityVariation,StartSize,DeltaSize);
		((cItems*)pNewParticleSystem)->m_MinSize=MinSize;
		((cItems*)pNewParticleSystem)->m_MaxSize=MaxSize;
		((cItems*)pNewParticleSystem)->m_ParticleAcceleration=ParticleAcceleration;
		
		break;
	}
	//	set the time time for the system
	pNewParticleSystem->m_TimeScale=TimeScale;	

	//	add it to the scene particle systems
	m_pScene->m_ParticleSystems.push_back(pNewParticleSystem);


	return 1;
}


//	reading a pointlink section
int	cSceneLoader::ReadPointLink()
{
	sPointLink	NewPointLink;	//	new path link to be filled
								//	will only be added to the added if both pointer are not nulls
	int	SrcID=-1;		//	the path id , index inside the paths vector
	int	DestID=-1;		//	the destid,  index inside the points vector

	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())			continue;
		//	if this is a new section, break from here
		if(NewSection())			break;

		//	if this is the name of the point link
		//	add it to the pointslinksnames
		if(m_LineWords[0]=="name")
		{
			m_PointLinksNames.push_back(m_LineWords[2]);
		}
		else		//	first read the point id, starts from 1 to the number of paths
		if(m_LineWords[0]=="source")
			SrcID		=	GetPointID(m_LineWords[2]);
		else		//	if this is the type of link, may be one of 11 things
		if(m_LineWords[0]=="destination")
			DestID		=	GetPointID(m_LineWords[2]);
		else
		if(m_LineWords[0]=="scale")
			NewPointLink.m_Scale	=	atof(m_LineWords[2]);		
		else
			Application()->ReportError("Unknown pointlink attribute!");
	}

	//	check for valid names
	if(DestID==-1)
		Application()->ReportError("Unknown point3d name for pathlink");

	if(SrcID==-1)
		Application()->ReportError("Unknown path name for pathlink");


	NewPointLink.m_pSrcPoint	=	m_pScene->m_VectorsList[SrcID];
	NewPointLink.m_pDestPoint	=	m_pScene->m_VectorsList[DestID];

	//	now add the new path link to the local list
	m_pScene->m_PointLinksList.push_back(NewPointLink);
	return 1;

}


//	use those to encode/decode scripts
void cSceneLoader::Encode(char* infilename,char* outfilename)
{
	FILE*	infile;
	FILE*	outfile;

	char Counter=0x00;

	infile=fopen(infilename,"r+b");

	outfile=fopen(outfilename,"w+b");

	char c;

	while(1)
	{
		c=fgetc(infile);

		if(c==EOF)
			break;

		//	encode the character
		c	=	c^Counter;

		//	if the result is EOF, put F followed by L
		if(c==EOF)
		{
			fputc('F',outfile);
			fputc('L',outfile);
		}
		else	//	if an F..put 2 Fz
			if(c=='F')
			{
				fputc('F',outfile);
				fputc('F',outfile);
			}
			else	//	else.. put the character
			fputc(c,outfile);

		Counter++;		//	in crement counter
	}
	fclose(infile);
	fclose(outfile);
}


void cSceneLoader::Decode(char* infilename,char* outfilename)
{
	FILE*	infile;
	FILE*	outfile;

	char Counter=0x00;

	infile=fopen(infilename,"r+b");

	outfile=fopen(outfilename,"w+b");

	char c;

	while(1)
	{
		c=fgetc(infile);

		if(c==EOF)
			break;


		//	if we found an F
		if(c=='F')
		{
			//	get another charcater
			c=fgetc(infile);

			//	if this is L...make c=EOF...
			if(c=='L')
				c=EOF;
		}
		c	=	c^Counter;


		fputc(c,outfile);
		Counter++;		//	in crement counter
	}
	fclose(infile);
	fclose(outfile);
}




