#ifndef _SIMULATORS_H
#define _SIMULATORS_H
/****************************************************************
*	simulators.h												*
*	header file for classes representing blocks of systems		*
*	part of the flulib class library							*
****************************************************************/

#define g_hr 0.002f 					// error in calculation...

/****************************************************************
*	Class cGain													*
*	very simple block in cointrol system, just a multply value	*
*	used later to aggreagte into more comples constructs		*
****************************************************************/
template <class X> class cGain
{
private:
	// gain.. input.. output.. very simple
	float	m_K		;
	X		m_Output	;

public:
	// the main constructor.. make a passive gain of 1
	cGain(float val=1.0f){Set(val);}
	// this is to set the gain value
	void Set(float val){m_K=val;}
	// if we apply an input.. the out is it mulitplied by k..
	void Apply(X in){m_Output=m_K*in;}
	// this is to return the out_put
	X Output(){return m_Output;}
};

/*------------------end of class cGain--------------------------*/


/****************************************************************
*	Class cIntegrator											*
*	a block that accumulates all its inputs						*
*	represent a block diagram with equation 1/S					*
*	it has a reste to emty it out again							*
****************************************************************/
template <class X> class cIntegrator
{
private:
	float	m_H;				//	height of curve (factor)
	X	m_ThisInput;			//	previous input value
	X	m_LastInput;			//	input applied at the moment
	X	m_Output;				//	output of the block

public:

	// main constructor
	cIntegrator()	{SetZero();}

	// initial condition settings
	void SetZero()
	{
		m_H			=	g_hr;						// by default this is 1/100
		memset(&m_LastInput,0,sizeof(X));			//	no input there
		memset(&m_Output,0,sizeof(X));				//	output is still zero
	}

	// this is to apply input.. using the trapezium rule to implement integration
	void Apply(X Input)
	{
		m_ThisInput		=	Input;
		m_Output		=	m_Output+m_H/2*(m_ThisInput+m_LastInput);
		m_LastInput		=	m_ThisInput;
	}
	// this method is used after applying input to read the output
	X Output()
	{
		return m_Output;
	}
};

/*-------------end of class cIntegrator declaration-------------*/


/****************************************************************
*	Class cDifferetiator										*
*	represent a block of S										*
****************************************************************/
template <class X> class cDifferentiator
{
private:
	float	m_H;
	X	m_ThisInput;
	X	m_LastInput;
	X	m_Output;

public:

	// main constructor
	cDifferentiator()
	{	SetZero();	}

	// to set the initial condition
	void SetZero()
	{
		m_H			=	g_hr;
		memset(&m_LastInput,0,sizeof(X));			//	no input there
		memset(&m_Output,0,sizeof(X));				//	output is still zero
	}

	// this is to apply input.subtract last two inputs to get the output
	void Apply(X Input)
	{
		m_ThisInput		=	Input;
		m_Output		=	(m_ThisInput-m_LastInput)/m_H;
		m_LastInput		=	m_ThisInput;
	}
	// this method is used after applying input to read the output
	X Output(){	return m_Output;}
};
/*--------------end of class cDifferetiator --------------------*/



/****************************************************************
*	class cPole													*
*	represents a block with the equation	  1/(AS+B)			*
*	using the previous 2 blocks...								*
****************************************************************/
template <class X> class cPole
{
private:
	cGain<X>			m_Gain;			//	gain of 1/A
	cIntegrator<X>		m_Integrator;	//	1/S

public:

	//	The Pole parameters (1/AS+B)
	float			m_A;
	float			m_B;

	// main constructor
	cPole()
	{
		m_A=1;								// default time constant
		m_B=1;								// default  1/S+1
		m_Gain.Set(1/m_A);					// this will set the gain value
		m_Integrator.SetZero();				// set the integrator
	}
	// a function to shange the time constant
	// u send the parameters J,B of the equation 1/Js+B
	void SetParams(float A,float B=1)
	{
		m_A=A;						// default time constant
		m_Gain.Set(1/m_A);			// this will set the gain value
		m_B=B;
	}
	// resetting it.. only the integrator need to be reset
	void SetZero()
	{
		m_Integrator.SetZero();
	}
	// here apply input
	void Apply(X Input)
	{
		m_Gain.Apply(Input - m_Integrator.Output()*m_B);
		// out put of gain enters the integrator
		m_Integrator.Apply(m_Gain.Output());
		// and closes the loop back to the feedback path
	}
	// read the out put
	X Output()
	{
		return m_Integrator.Output();
	}

};

/*------------------end of class dtrem---------------------*/

/****************************************************************
*	class cZero													*
*	represents a block with the equation	  (AS+B)			*
*	using the previous  blocks...								*
****************************************************************/
template <class X> class cZero
{
private:

	X					m_Input;
	cGain<X>			m_Gain;
	cDifferentiator<X>	m_Differentiator;

public:

	//	Zero Parameters (AS+B)
	float			m_A;
	float			m_B;

	// main constructor
	cZero()
	{
		m_A=1;							//	default time constant
		m_B=1;
		m_Gain.Set(m_A);				//	this will set the gain value
		SetZero();
	}
	// a function to shange the term parameters
	void SetParams(float a,float b=1)
	{
		m_A=a;// default time constant
		m_gain.Set(m_A);// this will set the gain value
		m_B=b;
	}

	// reseeting it..
	void SetZero()
	{
		memset(&m_Input,0,sizeof(X));	//	input is still zero
		m_Differentiator.SetZero();
	}

	// here apply input
	void Apply(X in)
	{
		m_Input=in;
		m_Gain.Apply(m_input);
		// out put of gain enters the integrator
		m_Diffirentiator.Apply(m_Gain.Output());
	}

	// read the out put
	X Output()
	{
		return m_Differentiator.Output()+(m_Input*B);
	}
};

/*----------------end of class nterm---------------------------*/

/****************************************************************
*	class secondorder											*
*	this is a class which represents a second order system		*
*	it is mainly a closed loop of an integrator,				*
*	a gain, and a dterm											*	
*	the equation as open loop is  Wn^2 /  s ( s + 2 * Z * Wn)	*
*	but closed loop is  Wn^2 / (S^2 + 2 *Z * Wn + Wn^2)			*
****************************************************************/
template <class X> class cSecondOrder
{
private:

	cIntegrator<X>	m_Integrator;
	cPole<X>		m_Pole;

	float m_Wn;				// this is omega N
	float m_Zeta;			// this is Z
public:

	bool m_bOpenLoop;

	// default constructor...
	cSecondOrder()
	{
		m_Wn=1;
		m_Zeta=1;
		m_Pole.SetParams(1,2*m_Zeta*m_Wn);
		m_bOpenLoop=false;
	}

	// this sets the zeta and omega of the system
	void SetParams(float WN,float Z)
	{
		m_Wn=WN;
		m_Zeta=Z;
		m_Pole.SetParams(1,2*m_Zeta*m_Wn);
	}
	// reset the system..
	void SetZero()
	{
		m_integ1.SetZero();
		m_Pole.SetZero();
	}
	// applying input...
	void Apply(X in)
	{
		// this is a closed loop one
		if(!m_bOpenLoop)
		m_Integrator.Apply((in-m_Pole.Output())*m_Wn*m_Wn);
		else
			m_Integrator.Apply((in)*m_Wn*m_Wn);
		m_Pole.Apply(m_Integrator.Output());
	}
	// reading output..
	X Output()
	{
		return m_Pole.Output();
	}

};
/*----------------end of class cSecondOrder---------------------*/


#endif //_FLULIBSYSTEM_H