#include "special.h"

/************************************************************************
*	special.cpp															*
*	By Shady el Mously													*
*	part of the flulib class library									*
*	this file has the implementation for the speciall effects classes	*
************************************************************************/



//	bill boarding means drawing a single quad that is always facing the viewer..in any location
void BillBoardedQuad(point3d& pos,float size)
{
	float mat[16];		//	get the model view matrix here
	
	glGetFloatv(GL_MODELVIEW_MATRIX,mat);		//	get the model view matrix
	point3d right(mat[0],mat[4],mat[8]);		//	extract the 2 needed vector to make the trick
	point3d up(mat[1],mat[5],mat[9]);

	glBegin(GL_QUADS);
		//	bottom left corner
		glTexCoord2f(0,0);glVertex3fv((pos+(right+up)*-size).v);
		//bottom right corner
		glTexCoord2f(1,0);glVertex3fv((pos+(right-up)*size).v);
		//top right corner
		glTexCoord2f(1,1);glVertex3fv((pos+(right+up)*size).v);
		//top left corner
		glTexCoord2f(0,1);glVertex3fv((pos+(up-right)*size).v);
	glEnd();
}




/************************************************************************
*	class cParticleSystem implementation								*
************************************************************************/

//	particle system constructor, sets the pointer to NULL
//	and store the maximum number of particles and the origin of particles
cParticleSystem::cParticleSystem(int maxParticles,point3d& origin)
{
	m_MaxParticles	=	maxParticles	;
	m_Origin		=	origin			;
	m_ParticleList	=	NULL			;
	m_TimeScale		=	1.0f			;
}

//	creates the number of new particles specified by the parameter
//	using the genral particle system values with some radnom element
//	note that only initial values will be randomized, final values will not
//	this may be changed later
int cParticleSystem::Emit(int numparticles)
{
	//	create numparticles new particles (if ther is room)
	while(numparticles&&(m_NumParticles < m_MaxParticles))
	{
		InitializeParticle(m_NumParticles++);
		--numparticles;
	}
	return numparticles;
}

//	allocate memory for the maximum number of particles in the system
void cParticleSystem::InitializeSystem()
{
	//	if this is just a reset, free the memory
	if(m_ParticleList)
	{
		delete [] m_ParticleList;
		m_ParticleList	=	NULL;
	}


	//	allocate the maximum number of particles
	m_ParticleList	=	new sParticle[m_MaxParticles];

	//	reset the number of particles and accumulated time
	m_NumParticles		=	0;
	m_AccumulatedTime	=	0;
}

//	will free up the memory and tells the emitter to stop emitting
void cParticleSystem::KillSystem()
{
	if(m_ParticleList)
	{
		delete [] m_ParticleList;
		m_ParticleList	=	NULL;
	}

	m_NumParticles	=	0;
}
/*--------------end of class cParticleSystem implementation-----------*/

/************************************************************************
*	class cSnowStorm Implementation										*
************************************************************************/



//	construct the snow storm and set all its parameters
cSnowStorm::cSnowStorm(int numparticles,point3d& origin,float height,float width,float depth,char* filename)
:m_Height(height),m_Width(width),m_Depth(depth),cParticleSystem(numparticles,origin)
{
	m_pTexture					=	new cTexture(filename)		;
	m_FlakeVelocity				=	point3d(0,-0.7f,0)			;
	m_FlakeVelocityVariation	=	point3d(0.1f,0.5f,0.1f)		;
	m_SnowFlakeSize				=	0.2f						;
	m_SnowFlakesRate			=	2000						;
	m_TimeScale					=	1.0f/400.0f					;
}


//	use this to set the snowstorm attributes, call initialize after calling it for effect
void cSnowStorm::SetAttributes(point3d& FlakeVelocity,point3d& VelocityVariation,float FlakeSize,float Rate)
{
	m_FlakeVelocity				=	FlakeVelocity;
	m_FlakeVelocityVariation	=	VelocityVariation;
	m_SnowFlakeSize				=	FlakeSize	;
	m_SnowFlakesRate			=	Rate	;

}


//	set the initial particle properties for the snow storm
void cSnowStorm::InitializeParticle(int index)
{
	//	start the particle in the sky at a random location in the emission zone
	m_ParticleList[index].m_Loc.y	=	m_Height;
	m_ParticleList[index].m_Loc.x	=	m_Origin.x+FRAND*m_Width;
	m_ParticleList[index].m_Loc.z	=	m_Origin.z+FRAND*m_Depth;

	//	set the size of the particle
	m_ParticleList[index].m_Size	=	m_SnowFlakeSize;

	//	give the particle a random velocity
	m_ParticleList[index].m_Velocity.x	=	m_FlakeVelocity.x+FRAND*m_FlakeVelocityVariation.x;
	m_ParticleList[index].m_Velocity.y	=	m_FlakeVelocity.y+FRAND*m_FlakeVelocityVariation.y;
	m_ParticleList[index].m_Velocity.z	=	m_FlakeVelocity.z+FRAND*m_FlakeVelocityVariation.z;
}



//	update the existing particlees, killing them and creating new ones when needed
void cSnowStorm::Update(float elapsedtime)
{
	elapsedtime*=m_TimeScale;
	for(int i=0;i<m_NumParticles;)
	{
		//	update the particle position based on elapsed time and velocity
		m_ParticleList[i].m_Loc+=elapsedtime*m_ParticleList[i].m_Velocity;

		
		//	if the particle hit the ground, kill it
		if(m_ParticleList[i].m_Loc.y<=m_Origin.y)
		{
			//	move the last particle to the current position and decrease the count
			m_ParticleList[i]=m_ParticleList[--m_NumParticles];
		}
		else
		{
			++i;
		}
	}

	//	store the accumulated time 
	m_AccumulatedTime+=elapsedtime;

	//	determine how many particles are needed
	int newParticles=m_SnowFlakesRate* m_AccumulatedTime;

	//	save the remaining time for after the new particles are released
	m_AccumulatedTime-=1.0f/(float)m_SnowFlakesRate*newParticles;

	Emit(newParticles);
}


void cSnowStorm::Render()
{
	//	enable alpha blending and disable zbuffer

	//	set the blend mode
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	m_pTexture->Activate();
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

	//	avoid constant dereferencing
	point3d partpos;
	float size;

	//	set up billboarding
	float mat[16];		//	get the model view matrix here
	glGetFloatv(GL_MODELVIEW_MATRIX,mat);		//	get the model view matrix
	point3d right(mat[0],mat[4],mat[8]);		//	extract the 2 needed vector to make the trick
	point3d up(mat[1],mat[5],mat[9]);

	//	draw the quads
	glBegin(GL_QUADS);
	for(int i=0;i<m_NumParticles;++i)
	{
		partpos	=	m_ParticleList[i].m_Loc;
		size	=	m_ParticleList[i].m_Size;
		glTexCoord2f(0,0);glVertex3fv((partpos+(right+up)*-size).v);
		glTexCoord2f(1,0);glVertex3fv((partpos+(right-up)*size).v);
		glTexCoord2f(1,1);glVertex3fv((partpos+(right+up)*size).v);
		glTexCoord2f(0,1);glVertex3fv((partpos+(up-right)*size).v);
	}
	glEnd();

	m_pTexture->Deactivate();
}
/*----------------end of class cSnowStorm implementation-----------*/

/********************************************************************
*	class cFlame implementation										*
********************************************************************/
//	use this to set the parameters
//	some useful values are :
//	smoke : gray,point3d(0,0.03f,0),point3d(0.04f,0.05f,0.04f)
//	fast smoke : point3d(0,0.1,0),point3d(0.1,0.1,0.1)
void cFlame::SetParams(color4 InitialColor,point3d& ParticleVelocity,point3d& ParticleVelocityVariation,float StartSize,float DeltaSize)
{
	//m_InitialColor				=	InitialColor				;
	m_ParticleVelocity			=	ParticleVelocity			;
	m_ParticleVelocityVariation	=	ParticleVelocityVariation	;
	m_Size						=	StartSize					;
	m_DeltaSize					=	DeltaSize					;
}

//	this will initialize a smoke particle
void cFlame::InitializeParticle(int index)
{
	//	start the particle in the source position
	m_ParticleList[index].m_Loc			=	m_Origin;
		
	//	give the particle a random velocity
	m_ParticleList[index].m_Velocity.x	=	m_ParticleVelocity.x+FRAND*m_ParticleVelocityVariation.x;
	m_ParticleList[index].m_Velocity.y	=	m_ParticleVelocity.y+FRAND*m_ParticleVelocityVariation.y;
	m_ParticleList[index].m_Velocity.z	=	m_ParticleVelocity.z+FRAND*m_ParticleVelocityVariation.z;

	//	set the color of the particle to the initial color
	m_ParticleList[index].m_Color		=	m_InitialColor;

	//	set size and size delta
	m_ParticleList[index].m_Size		=	m_Size;
	m_ParticleList[index].m_SizeDelta	=	m_DeltaSize-((0.5f+FRAND)/10.0f);

}



//	update the existing particlees, killing them and creating new ones when needed
void cFlame::Update(float elapsedtime)
{
	elapsedtime*=m_TimeScale;
	for(int i=0;i<m_NumParticles;)
	{
		//	update the particle position based on elapsed time and velocity
		m_ParticleList[i].m_Loc+=elapsedtime*m_ParticleList[i].m_Velocity;

		//	increase its size
		m_ParticleList[i].m_Size+=m_ParticleList[i].m_SizeDelta;
				
		//	if the particle size is less than zero kill it
		if(m_ParticleList[i].m_Size<m_MinSize||m_ParticleList[i].m_Size>m_MaxSize)
		{
			//	move the last particle to the current position and decrease the count
			m_ParticleList[i]=m_ParticleList[--m_NumParticles];
		}
		else
		{
			++i;
		}
	}

	//	store the accumulated time 
	m_AccumulatedTime+=elapsedtime;

	//	determine how many particles are needed
	int newParticles=m_ParticlesRate* m_AccumulatedTime;

	//	save the remaining time for after the new particles are released
	m_AccumulatedTime-=1.0f/(float)m_ParticlesRate*newParticles;

	//	emit the required particles
	Emit(newParticles);
}

//	this will draw the smoke
void cFlame::Render()
{

	//	set the blend mode
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	m_pTexture->Activate();
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

	//	avoid constant dereferencing
	point3d partpos;
	float size;

	//	set up billboarding
	float mat[16];		//	get the model view matrix here
	glGetFloatv(GL_MODELVIEW_MATRIX,mat);		//	get the model view matrix
	point3d right(mat[0],mat[4],mat[8]);		//	extract the 2 needed vector to make the billboarding
	point3d up(mat[1],mat[5],mat[9]);

	glColor4fv(m_InitialColor.c);

	//	draw the quads
	glBegin(GL_QUADS);
	for(int i=0;i<m_NumParticles;++i)
	{
		partpos	=	m_ParticleList[i].m_Loc;
		size	=	m_ParticleList[i].m_Size;
		glTexCoord2f(0,0);glVertex3fv((partpos+(right+up)*-size).v);
		glTexCoord2f(1,0);glVertex3fv((partpos+(right-up)*size).v);
		glTexCoord2f(1,1);glVertex3fv((partpos+(right+up)*size).v);
		glTexCoord2f(0,1);glVertex3fv((partpos+(up-right)*size).v);
	}
	glEnd();

	m_pTexture->Deactivate();	//	finish using the texture
}
/*----------------end of class cFlame implementation--------------*/





/********************************************************************
*	class cItems implementation										*
*	By Shady El Mously												*
********************************************************************/

cItems::cItems(int numparticles,point3d& origin,color4 InitialColor,char* BmpFileName) :cFlame(numparticles,origin,InitialColor,BmpFileName)
{
	m_ParticleAcceleration=point3d(0,0,0);
	
}


//	initialize a single particle
void cItems::InitializeParticle(int Index)
{
	cFlame::InitializeParticle(Index);
	m_ParticleList[Index].m_Acceleration=m_ParticleAcceleration;
}

void cItems::Update(float elapsedtime)
{
	elapsedtime*=m_TimeScale;

	for(int i=0;i<m_NumParticles;)
	{
		//	update the particle position based on elapsed time and velocity
		m_ParticleList[i].m_Velocity+=elapsedtime*m_ParticleList[i].m_Acceleration;

		//	update the particle position based on elapsed time and velocity
		m_ParticleList[i].m_Loc+=elapsedtime*m_ParticleList[i].m_Velocity;

		//	if the particle size is less than zero kill it
		if(m_ParticleList[i].m_Loc.y<-1)
		{
			//	move the last particle to the current position and decrease the count
			m_ParticleList[i]=m_ParticleList[--m_NumParticles];
		}
		else
		{
			++i;
		}
	}

	//	store the accumulated time 
	m_AccumulatedTime+=elapsedtime;

	//	determine how many particles are needed
	int newParticles=m_ParticlesRate* m_AccumulatedTime;

	//	save the remaining time for after the new particles are released
	m_AccumulatedTime-=1.0f/(float)m_ParticlesRate*newParticles;

	//	emit the required particles
	Emit(newParticles);
}


/*----------------end of class cItems implementation---------------*/