/****************************************************************
*	flulibio.cpp												*
*	By Shady El Mously											*
*	this is the implementation for the methods and functions	*
*	of classes related to text input/input and processing		*
*	and the threaded command line interpreter					*
*	part of the flutey library									*
*	it is mainly the implementation of the classes				*
*	BFONT,TEXTIO												*
*	and global string functions									*
****************************************************************/

#include	"application.h"
#include	"graphics.h"			//	Graphics layer used here to make the text I/O interface
#include	"text.h"

/********************************************************************
*	Definition of cTextSubSystem methods							*
********************************************************************/

//	constructor, a graphics layer pointer  of an existing  one must be sent
cTextSubSystem::cTextSubSystem() :m_CurLine(Graphics()->m_CurLine),m_CurCharPos(Graphics()->m_CurCharPos)

{
	m_CurLine			=	m_CurCharPos			=	0	;	//	initialize cursor position
	m_NumLines			=	Graphics()->m_NumLines			;	//	number of lines
	m_NumCharsPerLine	=	Graphics()->m_NumCharsPerLine	;	//	number os max charcaters per line
	m_Buffer			=	Graphics()->m_ScreenText		;	//	the pointer to screen text buffer
	m_DelayOnAll		=	false							;	//	no delay on all keys
	BuildKeysMap();
	Application()->m_ReportFile<<"Textlayer   SubSystem Successfully Initialized"<<endl;
}

// this small function will clear the primary surface when used for text output
void cTextSubSystem::Clear()
{
	//	reset the cursor position
	m_CurLine		=	0;
	m_CurCharPos	=	0;
	Graphics()->ClearText();
}


// the same one but this one works directly on the back buffer.. or any other surface
// if u wanna write somewhwre on screen...send the coordinates..
// elsewhere.. the text cursor moves...
void cTextSubSystem::PutString(char * string,int x_pos,int y_pos)
{
	//	if the application is dead.. go back quickly..
	if(Application()->m_IsDead) return;

	//	store the cursor position in here
	point2d tempcur;

	// check if naything was sent to the function
	if(x_pos!=-1 && y_pos !=-1)
	{
		// save the cursor position...
		tempcur.y		=	m_CurLine;
		tempcur.x		=	m_CurCharPos;
		// move to the new position required..			
		m_CurCharPos	=	x_pos;
		m_CurLine		=	y_pos;
	}


		
	// put the string in the screen text array..
	for(int x=0;x<(int)strlen(string);x++)
	{	
		SendChar(string[x]);
	}
	// uncomment this to gain everything back
	if(!Application()->m_InThread)
	{
		Graphics()->Clear();
		Graphics()->DrawTextBuffer();
		Graphics()->Flip();
	}
	// restore the original cursor position before leaving...
	if(x_pos!=-1 && y_pos !=-1)
	{
		m_CurLine		=	tempcur.y;
		m_CurCharPos	=	tempcur.x;
	}
}



//	this function will be used to get a string
//	send FALSE if u dont want it to make a new new when exiting
void cTextSubSystem::GetString(char * string)
{
	
	string[0]=0	;		//	set up the string first.
	char ch		;		//	the character to get to

	//	this while loop is thread enabled,
	while(1)
	{		
		if(!Application()->m_InThread)			// a thread should'nt handle messages
		if(Application()->ProcessMessages())break;
		// never process string when the window is not active
		if(Application()->m_IsActive)
		{
			ch=GetKey();
			//	this sleep is very useful in threads
			Sleep(50);
			if(ch==13)break;
			else
			//	do not handle back space for zero length strings
			if(ch==8)
			{
				if(strlen(string)>0)		//	back space should cut out last charcater in da string
					string[strlen(string)-1]=0;
				else
					continue;	//	else i should not go further
			}
			else
			if(ch)
			{
				sprintf(string,"%s%c",string,ch);
			}
			SendChar(ch);
			if(Application()->m_IsDead)		return;
		}
	}
}

//	will build the map at startup between keuboard and ascii codes
//	keys with shift must be sent being ored with 0x0800
void cTextSubSystem::BuildKeysMap(void)
{
	m_VKtoAsciiMap[VK_SPACE |0x0000]		=	32	;	//	space bar
	m_VKtoAsciiMap[VK_TAB   |0x0000]		=	9	;	//	tab key
	m_VKtoAsciiMap[VK_BACK	|0x0000]		=	8	;	//	back space
	m_VKtoAsciiMap[VK_RETURN|0x0000]		=	13	;	//	back space

	m_VKtoAsciiMap[VK_SPACE	|0x0800]		=	32	;	//	space bar
	m_VKtoAsciiMap[VK_TAB	|0x0800]		=	9	;	//	tab key
	m_VKtoAsciiMap[VK_BACK	|0x0800]		=	8	;	//	back space
	m_VKtoAsciiMap[VK_RETURN|0x0800]		=	13	;	//	back space

	//	add the alphabetic low case characters
	for(int x=62;x<92;x++)// loop through the ascii characters
	{
		m_VKtoAsciiMap[x]			=	x+32	;
		m_VKtoAsciiMap[x|0x0800]	=	x		;
	}


	for(x=48;x<58;x++)// loop numbers 0,1,2...
	{
		m_VKtoAsciiMap[x]	=	x;
	}

	m_VKtoAsciiMap[186]		=	';';
	m_VKtoAsciiMap[187]		=	'=';
	m_VKtoAsciiMap[188]		=	',';
	m_VKtoAsciiMap[189]		=	'-';
	m_VKtoAsciiMap[190]		=	'.';
	m_VKtoAsciiMap[191]		=	'/';
	m_VKtoAsciiMap[192]		=	'`';
	m_VKtoAsciiMap[219]		=	'[';
	m_VKtoAsciiMap[220]		=	'\\';
	m_VKtoAsciiMap[221]		=	']';
	m_VKtoAsciiMap[222]		=	'\'';

	m_VKtoAsciiMap[48 |0x0800]	=')';
	m_VKtoAsciiMap[49 |0x0800]	='!';
	m_VKtoAsciiMap[50 |0x0800]	='@';
	m_VKtoAsciiMap[51 |0x0800]	='#';
	m_VKtoAsciiMap[52 |0x0800]	='$';
	m_VKtoAsciiMap[53 |0x0800]	='%';
	m_VKtoAsciiMap[54 |0x0800]	='^';
	m_VKtoAsciiMap[55 |0x0800]	='&';
	m_VKtoAsciiMap[56 |0x0800]	='*';
	m_VKtoAsciiMap[57 |0x0800]	='(';

	m_VKtoAsciiMap[186|0x0800]	=':';
	m_VKtoAsciiMap[187|0x0800]	='+';
	m_VKtoAsciiMap[188|0x0800]	='<';
	m_VKtoAsciiMap[189|0x0800]	='_';
	m_VKtoAsciiMap[190|0x0800]	='>';
	m_VKtoAsciiMap[191|0x0800]	='?';
	m_VKtoAsciiMap[192|0x0800]	='~';
	m_VKtoAsciiMap[219|0x0800]	='{';
	m_VKtoAsciiMap[220|0x0800]	='|';
	m_VKtoAsciiMap[221|0x0800]	='}';
	m_VKtoAsciiMap[222|0x0800]	='"';
}

//	this is my own version of getch
//	getch should enter in an infinite loop
//	waiting for a charcater to be pressed
//	or waiting that the application terminates
//	if there is a thread runnning
//	it should not handles windows messages
//	i usually should use this function inside threads
char cTextSubSystem::Getch()
{
	char res=0;
	static DWORD lastchartime	=	GetTickCount();
	while(1)
	{

		if(!Application()->m_InThread)
			if(!Application()->ProcessMessages())
				break;

		if(Application()->m_IsDead)
			break;

		//	to synchronize
		Sleep(50);

		for(int i=0;i<256;i++)
		{
			if(GetTickCount()-lastchartime>100)
			{
			if(KEYDOWN(i)&&i!=VK_SHIFT)
				{
					lastchartime	=	GetTickCount();

					if(KEYDOWN(VK_SHIFT))
						i=i|0x0800;
					return m_VKtoAsciiMap[i];
				}
			}
		}
	}
	return res;
}

//	return the ascii code of a touched keyboard key
//	returns 0 if nothing was touched
char cTextSubSystem::HitKey()
{
	for(int i=1;i<256;i++)
	{
		if(KEYDOWN(i)&&i!=VK_SHIFT)
		{
			if(KEYDOWN(VK_SHIFT))
				return Text()->m_VKtoAsciiMap[i|0x0800];
			else
				return Text()->m_VKtoAsciiMap[i];
		}
	}
	return 0;
}


//	this will get a key from the keyboard, using delay checks
//	it is essentially just like HitKey
char	cTextSubSystem::GetKey		(void)
{
	char c;
	c=HitKey();

	//	first check if we are still in the delay period
	if(GetTickCount()-m_CurTime<200)
	{
		//	if true, check if we could check for the same charcater or not
		if(m_DelayOnAll)
			return 0;
		else	//	make sure this is a new character, if it is the same old, go back
			if(m_LastCharacter==c)
				return 0;
	}

	//	update the time for next delay
	m_LastCharacter	=	c					;
	m_CurTime		=	GetTickCount()		;
	return	c;
}
//	cursor control functions

//	sending a single characater
void cTextSubSystem::SendChar(char c)
{
	if(c==13||c=='\n')	//	if received an enter
	{
		NewLine();
		return ;
	}
	else
	if(c==8)		//	if back space.. go back one step
	{
		BackSpace();
		return;
	}
	else
		if(c==0)return;
	else
	{
		if(c=='\t')c=' ';
		//	if i reach here, this is a valid character
		m_Buffer[m_CurLine][m_CurCharPos]=c;
		m_CurCharPos++;
		if(m_CurCharPos==m_NumCharsPerLine-1)
			NewLine();
	}

}

//	new line
void cTextSubSystem::NewLine(void)
{
	m_CurCharPos=0;					//	set cursor x to zero
	if(m_CurLine==m_NumLines-1)		//	if this is the last line
	{
		ScrollUp();				//	scroll up the characters
		return;					//	and return
	}
	m_CurLine++;					//	else, increment the current cursor y by 1
}



//	scrolls up the screen
void cTextSubSystem::ScrollUp(void)
{
	for(int i=1;i<m_NumLines;i++)		//	scroll all lines up
		for(int j=0;j<m_NumCharsPerLine;j++)	//	except the first line
		{
			m_Buffer[i-1][j]=m_Buffer[i][j];
		}
	//	fill last line with zeros
	for(int j=0;j<m_NumCharsPerLine;j++)	//	except the first line
	{
		m_Buffer[m_NumLines-1][j]=0;
	}
}


//	back space handling
void cTextSubSystem::BackSpace(void)
{
	if(m_CurCharPos>0)				//	if not on the begining of a line
		m_CurCharPos--;				//	decrement cursor xby 1
	else						//	else (we are at the begining of a line)
	if(m_CurLine>0)				//	make sure this isnt the first line
	{
		m_CurCharPos=strlen(m_Buffer[m_CurLine-1])-1;	//	reset the cursor x position to the end of a line
		m_CurLine--;				//	decrement cursor y position by 1
	}
	m_Buffer[m_CurLine][m_CurCharPos]=0; //	put a zero in the cursor position (delete a character)
}

/*-------------------------------------------------------------*/
//				End of cTextSubSystem implementation


/****************************************************************
*	class shell implementation									*
*	By Shady El Mously											*
*	this class is used to create a shell						*
*	new commands may be added using inhertance and rewriting	*
*	the function (HandleMoreCommands							*
*	a shell with 3d commands is inherited from this shell in	*
*	the 3d section												*
****************************************************************/




//	this will start the shell as a normal function
void cShell::Run(void)
{
	Text()->Clear();

	cString		Command;		//	the command taken
	Command.reserve(100);
	// a welcome message
	Text()->PutString(m_WelcomeMessage);
	while(1)
	{
		//	if application is dead break this loop quickly..
		if(Application()->m_IsDead)		return;
		// the thread dont handle messages
		if(!InThread())if(Application()->ProcessMessages())break;
		//	put the prompt
		Text()->PutString(">");
		//	and get a string
		Text()->GetString(Command);
		Text()->PutString("\n");
		if(Application()->m_IsDead)		return;
		//	now split the string into a series of small strings..
		//	the arguments
		cString::Split(Command);
		if(cString::arg_v.size()==0)	
			continue;
		else
			if(cString::arg_v[0]=="exit")break;
		else
			if(!HandleCommands())
				Text()->PutString("Unrecognized command, type help for help\n");
	}
	if(Graphics())	Text()->Clear();
}
//	the default commands remain here
bool cShell::HandleCommands()
{
		if(cString::arg_v[0]=="cls")
		{
			Text()->Clear();
			return true;
		}
		// time command writes back the time..
		else
		if(cString::arg_v[0]=="time")
		{
			Text()->PutString(Application()->CurTime(),-1,-1);
			return true;
		}
		//quit ends program
		else
		if(cString::arg_v[0]=="quit")
		{
			Application()->m_IsDead=true;
			return true;
		}
		else 
		return HandleMoreCommands();

}

//*-------------------------------------------------------------*/
//				End of class shell implementation

