#include "textwindow.h"
#include "opengl.h"


//	the constructor makes a window with the dimentions sent to it
cTextWindow::cTextWindow(int width, int height,char* label)
{
	
	m_Label			=	"Chat Window"	;	//	set the window label
	m_CurLine		=	0				;	//	set the cursor position
	m_CurChar		=	0				;	//	set the cursor position
	m_Color			=	color3::Cyan	;	//	window is white
	m_Buffer		=	NULL			;	//	no buffer allocated yet
	SetPos(0,0)							;	//	initially window is in top left
	SetDims(width,height)				;	//	set The Window Dimensions
	AllocateCharacters()				;	//	allocate the charcaters
	m_ReceivedChar	=	0				;	//	set received character to null	
	m_Cursor		=	'_'				;	//	set cursor 
	m_CurTime		=	GetTickCount()	;	//	current time
	m_KeyDelay		=	0				;	//	the delay before repeting
	m_LastCharacter	=	0				;	//	last characer is initially zero
	m_DelayOnAll	=	false			;	//	delay is only on the same character
	if(label)
		m_Label=label;
}


//	this will draw the window, should be called in 2D mode
void cTextWindow::Draw(void)
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);						//	save the state machine
	glBegin(GL_LINE_LOOP);
		glVertex3f(m_Xpos,m_Ypos,0);
		glVertex3f(m_Xpos+m_Width-2,m_Ypos,0);
		glVertex3f(m_Xpos+m_Width-2,m_Ypos-m_Height,0);
		glVertex3f(m_Xpos,m_Ypos-m_Height,0);
	glEnd();

	glListBase(GraphicsGL()->m_BitMapFontBase-32);				//	set new list base
	glDisable(GL_DEPTH_TEST);									//	disable zbuffer test
	glColor3f(m_Color.r,m_Color.g,m_Color.b);


	//	write the label
	glRasterPos2f((m_Width+(strlen(m_Label)*GraphicsGL()->m_FontWidth))/2,m_Ypos+10);
	glCallLists(strlen(m_Label),GL_UNSIGNED_BYTE,m_Label);
	

	// now draw all the screen text on the back buffer and flip it forward
	for(int y=0;y<m_NumLines;y++)
	{
		glRasterPos2f(m_Xpos,(m_Ypos-((y+1)*(GraphicsGL()->m_FontHeight))));
		glCallLists(strlen(m_Buffer[y]),GL_UNSIGNED_BYTE,m_Buffer[y]);
	}

	glRasterPos2f(m_Xpos+(strlen(m_Buffer[m_CurLine])*(GraphicsGL()->m_FontWidth-1)),(m_Ypos-((m_CurLine+1)*(GraphicsGL()->m_FontHeight))));
	glCallLists(1,GL_UNSIGNED_BYTE,&m_Cursor);
	glPopAttrib();											//	restore state machine
}

//	this will allocate characters exactly to the size of the window
void cTextWindow::AllocateCharacters()
{
	int i,j;
	FreeCharacters();
	m_NumLines			=	m_Height/GraphicsGL()->m_FontHeight;
	m_NumCharsPerLine	=	m_Width/(GraphicsGL()->m_FontWidth-1);

	m_Buffer	=	new char*[m_NumLines];
	for(i=0;i<m_NumLines;i++)
		m_Buffer[i]	=	new char[m_NumCharsPerLine];

	for(i=0;i<m_NumLines;i++)
		for(j=0;j<m_NumCharsPerLine;j++)
			m_Buffer[i][j]=0;
}


//	this will free up the memory allocated by the window characters
void cTextWindow::FreeCharacters()
{
	if(m_Buffer)
	{
		for(int i=0;i<m_NumLines;i++)
			delete [] m_Buffer[i];
		delete [] m_Buffer;
	}
}

//	use this to externaly send characters to the chat window
void cTextWindow::SendCharacter(char c)
{
	//	first check if we are still in the delay period
	if(GetTickCount()-m_CurTime<m_KeyDelay)
	{
		//	if true, check if we could check for the same charcater or not
		if(m_DelayOnAll)
			return;
		else	//	make sure this is a new character, if it is the same old, go back
			if(m_LastCharacter==c)
				return;
	}

	

	//	update the time for next delay
	m_LastCharacter	=	c					;
	m_CurTime		=	GetTickCount()		;

	if(c==13)	//	if received an enter
	{
		NewLine();
		return ;
	}
	else
	if(c==8)		//	if back space.. go back one step
	{
		BackSpace();
		return;
	}
	else
		if(c==0)return;
	else
	{
		if(c=='\t')c=' ';
		//	if i reach here, this is a valid character
		m_Buffer[m_CurLine][m_CurChar]=c;
		m_CurChar++;
		if(m_CurChar==m_NumCharsPerLine-1)
			NewLine();
	}
}


//	called when a new line character comes
void cTextWindow::NewLine()
{
	m_CurChar=0;					//	set cursor x to zero
	if(m_CurLine==m_NumLines-2)		//	if this is the last line
	{
		ScrollUp();				//	scroll up the characters
		return;					//	and return
	}
	m_CurLine++;					//	else, increment the current cursor y by 1
}

//	called when a back space comes
void cTextWindow::BackSpace()
{
	if(m_CurChar>0)				//	if not on the begining of a line
		m_CurChar--;				//	decrement cursor xby 1
	else						//	else (we are at the begining of a line)
	if(m_CurLine>0)				//	make sure this isnt the first line
	{
		m_CurChar=strlen(m_Buffer[m_CurLine-1]);	//	reset the cursor x position to the end of a line
		m_CurLine--;				//	decrement cursor y position by 1
	}
	m_Buffer[m_CurLine][m_CurChar]=0; //	put a zero in the cursor position (delete a character)
}

//	scroll up, the upper most line will be lost
void cTextWindow::ScrollUp()
{
	for(int i=1;i<m_NumLines;i++)		//	scroll all lines up
		for(int j=0;j<m_NumCharsPerLine;j++)	//	except the first line
		{
			m_Buffer[i-1][j]=m_Buffer[i][j];
		}
}
