#ifndef _WATERMESH_H
#define _WATERMESH_H
/****************************************************************
*	water.h														*
*	By Shady El Mously											*
*	header for the water mesh object							*
*	most of this code is extracted form the code in the book	*
*	Game Programming Gems, the article about the simulation		*
*	of water surfaces in containers								*
*	the mesh implemented here represents a water container		*
*	and it accurately simulates the refraction and the			*
*	reflection of it, this is a heavy code, and needs a real	*
*	good accelerator for use									*
****************************************************************/


/****************************************************************
*	struct sWaterContainer										*
*	represents a water container which has a depth				*
*	two textures are required for this mesh						*
****************************************************************/



struct sRealWater : public sMesh
{
	cTexture*	m_pRefractionTexture;			//	texture of refraction
	cTexture*	m_pReflectionTexture;			//	texture of reflection

	int		m_Width;
	int		m_Height;
	int		m_NumVertices;
	int		m_NumIndices;
	float	m_Depth;
	float	m_DampFactor;						//	damping factor
	bool	m_Fresnel		;					//	calcuate fresnel terms..


	vector<unsigned char>	m_Colors;			//	the colors used
	vector<float>			m_ForceArray;		//	the forces in each point
	vector<float>			m_VelocityArray;	//	velocities at each point
	vector<int>				m_Indices;			//	triangle indices

	void UpdateSurface(void);
	void ComputeNormals(void);
	void CalculateRefractionCoordinates(void);
	void Draw(void);
	void DrawMultiTex(void);
	void DrawMultiPass(void);
	void InitSurface(void);

	//	construct with the number of fertices on width and height, deph of container and a damping factor
	sRealWater(int NumRows,int NumColumns,float Depth=0.5f,float Damp=0.0f);


	//	destructor
	~sRealWater();
};

/*--------------end of sWaterContainer declaration---------------*/

#endif //_WATERMESH_H