#include "balls.h"



/****************************************************************
*	class cSteelBall implementation								*
*	many euations here repsent the motion of rigid balls,		*
*	simple harmonic motion and collision response				*
****************************************************************/

//	constructor..make the ball, using a radius and a string length
cSteelBall::cSteelBall(float rad,float len)
{
	m_StringLength	=	len	;
	m_Angle			=	0	;	
	m_AngVel		=	0	;
	m_Rad			=	rad	;
	m_Mass			=	1		;
	m_Pos			=	m_Loc+point3d(4,-len,0);			//	position of the ball in the world
}


//	apply a force on the ball
void cSteelBall::ApplyForce(float Force)
{
	//	the force applit is tangetial, it will simply givea velocity
	m_AngVel+=Force;
}



//	update the ball, using simple pendulum equations
void cSteelBall::UpDate(float DeltaTime)
{
	m_PrevAngVel	=	m_AngVel;
	m_PrevAngle		=	m_Angle;

	//	and i want to calculate the new position of the ball and its new tangetial velocity
	float TangAcc;
	float AngAcc;


	//	find the tangetial acceleration from the angle
	TangAcc=sinf(m_Angle)*9.8f;

	//	convert it into a tangential angular acceleration
	AngAcc=asin(TangAcc/m_StringLength);

	//	change the angular velocity
	m_AngVel+=AngAcc*DeltaTime;

	//	change the angle
	m_Angle-=m_AngVel*DeltaTime;

	//	find the world location of the ball
	m_Pos.x	=	m_Loc.x+m_StringLength*sin(m_Angle);
	m_Pos.y=	m_Loc.y-m_StringLength*cos(m_Angle);

}


//	draw the steel ball and the string
void cSteelBall::Draw()
{
	//	activate the material
	m_Material.Activate();


	point3d pos;
	pos.x=m_StringLength*sin(m_Angle);;
	pos.y=-m_StringLength*cos(m_Angle);
	pos.z=0;
	
	glPushMatrix();
	glMultMatrixf(m_WorldMatrix.d);


	//	draw the string
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
		glVertex3f(0,0,0);
		glVertex3fv(pos.v);
	glEnd();

	//	draw the ball
	glEnable(GL_LIGHTING);


	glTranslatef(pos.x,pos.y,pos.z);
	m_Mesh->Draw();

	glPopMatrix();

}
/*-------------end of class cSteelBall implementation----------*/


/****************************************************************
*	class cCollision implementation								*
*	represnts a collision between 2 steel balls					*
*	can be used to check and handle collisions					*
****************************************************************/

//	collision test .. using bounding spheres
bool cCollision::Check()
{

	float angvel=fabs(A->m_AngVel-B->m_AngVel);

	//	find the distance between both of the balls
	float dist=(A->m_Pos-B->m_Pos).mag();

	//	if it is less than the sum of their radii....return true
	if(dist<(A->m_Rad+B->m_Rad))
	{
		return true;
	}
	else
		return false;



}

//	handling the collision using momentum and conservation of energy equations
void cCollision::Handle()
{
	//	go back to angles before collision
	A->m_AngVel=A->m_PrevAngVel;
	B->m_AngVel=B->m_PrevAngVel;

	//	before starting to apply the momentum..
	//	move the balls to an angle at which they do are just touching
	if(fabs(A->m_Angle)<fabs(B->m_Angle))
		B->m_Angle=A->m_Angle;
	else
		A->m_Angle=B->m_Angle;

	//	and update both balls again
	A->UpDate(0.2f);
	B->UpDate(0.2f);


	B->m_Angle=A->m_Angle=A->m_PrevAngle;
	
	
	float vai=A->m_AngVel;
	float vbi=B->m_AngVel;

	//	masses
	float ma=A->m_Mass;
	float mb=B->m_Mass;

	A->m_AngVel=(2*mb*vbi+vai*(ma-mb))/(ma+mb);
	B->m_AngVel=(2*ma*vai-vbi*(ma-mb))/(ma+mb);

}




/*------------end of class cCollision implementation-----------*/

/****************************************************************
*	class cSteelSystem implementation							*
****************************************************************/

//	create the system
void cSteelSystem::Create(int NumBalls,double Radii,double L)
{

	if(size())
		Destroy();

	m_BallMesh.CreateSphere(Radii,20,20);

	float start=(NumBalls-1)*Radii;
	for(int i=0;i<NumBalls;i++)
	{
		push_back(new cSteelBall(Radii,L));
		back()->SetPos(start-i*2*Radii,L+2,0);
		SAFE_DELETE(back()->m_Mesh);
		back()->m_Mesh=&m_BallMesh;
	}

	//	create the collision objects
	for(i=0;i<NumBalls-1;i++)
	{
		cCollision newcol((*this)[i],(*this)[i+1]);
		m_Collisions.push_back(newcol);
	}


	m_Bar.CreateCylinder(Radii/2.0f,Radii/2.0f,1+(2*Radii*NumBalls));
	m_Bar.m_WorldMatrix.makeident();
	m_Bar.Rotate(0,0,PI/2);
	m_Bar.SetPos(0,front()->m_StringLength+2,0);

}

//	destroy the system
void cSteelSystem::Destroy()
{
	//	safely remove the mesh from the object and destroy it
	for(int i=0;i<size();i++)
	{
		(*this)[i]->m_Mesh=NULL;
		SAFE_DELETE((*this)[i]);
	}
	clear();
	m_Collisions.clear();
}


//	update the whole system
void cSteelSystem::Update()
{
	//	update all balls physics
	for(int i=0;i<size();i++)
	{
		(*this)[i]->UpDate(0.2f);
	}
	//	check and handle collisions
	for(i=0;i<m_Collisions.size();i++)
	{
		if(m_Collisions[i].Check())
		{
			if(i==0||i==m_Collisions.size()-1)
			m_Sound.Play();
			m_Collisions[i].Handle();

		}
	}
}



//	render he system and the bar
void cSteelSystem::Draw()
{
	m_Bar.Draw();
	glDisable(GL_TEXTURE);
	for(int i=0;i<size();i++)
	{
		(*this)[i]->Draw();
	}
	glEnable(GL_TEXTURE);
}



/*-----------end of class cSteelSystem implementation----------*/