#ifndef _BALLS_H
#define _BALLS_H

#include "./flulib/library.h"

/****************************************************************
*	balls.h														*
*	By Shady el Mously											*
*	part of the flulib 3d engine								*
*	this is the declaration file for the classes used to make	*
*	steel balls system											*
****************************************************************/


/****************************************************************
*	class cSteelBall											*
*	represents a single Steel Ball								*
****************************************************************/
class cSteelBall : public c3DComplexObject
{
public:


	float	m_Rad;				//	radius if the ball
	float	m_Mass			;	//	mass of the ball
	float	m_StringLength;		//	length of the string

	float	m_Angle;			//	angle in degrees
	float	m_PrevAngle;		//	previous angle in degrees
	float	m_AngVel;			//	angular velocity
	float	m_PrevAngVel;		//	previoud angular velocity
	//float	m_TangVel;			//	tangential velocity
	

	point3d	m_Pos	;			//	position of the ball


	//	constructor
	cSteelBall(float rad,float len);


	//	draw the ball with its string
	void Draw(void);

	//	update the ball..using the rotational equation
	void UpDate(float DeltaTime);

	//	this will apply a force on the ball
	void ApplyForce(float);
};
/*---------------end of class cSteelBall declaration--------*/


/*************************************************************
*	class cCollision decalration							*
*	represnts a collision between2 steel balls				*
*	using a boolean flag for it								*
************************************************************/

class cCollision
{
private:


	cSteelBall* A;
	cSteelBall* B;



public:

	bool		m_IsActive;
	//	always construct with the adrress of two existing balls
	cCollision(cSteelBall* b1,cSteelBall* b2):A(b1),B(b2)
	{	m_IsActive=false;	}


	//	collision test .. using bounding spheres
	bool Check();


	//	handling collision
	void Handle();
};
/*------------end of class cCollision declaration----------*/


/************************************************************
*	class cSteelSystem										*
*	represents a system of steel balls and the bar			*
*	from which they are syspended							*
************************************************************/
class cSteelSystem : public vector<cSteelBall*>
{
private:

	sStridedMesh	 m_BallMesh;			//	a single mesh is used for all balls
	cSound			m_Sound;				//	collision sound
	cRigidBody		m_Bar;					//	the bar where the balls are suspended
	vector<cCollision>	m_Collisions;		//	collisions objects

public:

	//	constructor with number of balls, their radii and the the strings length
	cSteelSystem(int NumBalls,double Radii,double L):m_Sound("collide.wav")
	{	Create(NumBalls,Radii,L);	}

	//	destructor !
	virtual ~cSteelSystem()
	{	Destroy();	}

	//	theis creates the system
	void Create(int NumBalls,double Radii,double L);

	//	destroy the system 
	void Destroy();

	//	update the  system
	void Update();

	//	render he system
	void Draw();
};
/*-------------end of class cSteelSystem declaration--------*/

#endif //_BALLS_H