/****************************************************************
*	main.cpp													*
*	main application file										*
*	here lies the main code of any specific app					*
*	everything in the library may be used here					*
****************************************************************/
#include "./flulib/library.h"

#include "balls.h"

void DisplayHelp()
{
	Graphics()->WriteText(0,30,0xFFFFFF,"Press SPACE to push the first ball");
	Graphics()->WriteText(0,50,0xFFFFFF,"Press F2 to Add a ball");
	Graphics()->WriteText(0,70,0xFFFFFF,"Press F3 to remove a ball");
	Graphics()->WriteText(0,90,0xFFFFFF,"Press F4 to stop the simulation");
	Graphics()->WriteText(0,110,0xFFFFFF,"Mouse + UP/DOWN control the camera");
	Graphics()->WriteText(0,130,0xFFFFFF,"E to toggle environment map");
}

//	this is the application main function
void cApplication::main(void)
{
	SetLabel("Marble Balls Simulator....By Shady El Mously (www.geocities.com/fluteyeg)");
	
	//	SetUp Scene and create objects
	if(!m_IsWindowed)ShowCursor(FALSE);

	bool	help			=	false	;
	bool	env				=	true	;
	int		NumBalls		=	2		;
	float	BallsRad		=	0.5f	;

	cCamera Camera;
	Camera.m_Loc=point3d(0,0,20);
	Camera.m_Yaw=0.3f;
	Camera.m_Pitch=-0.9f;
	float CameraLength=30.0f;

	sDirectionalLight l1(point3d(0,0,1));
	l1.Activate();

	cSteelSystem balls(NumBalls,BallsRad,9);
	cEnvPassTexture Env("env.bmp");

	cRigidBody Table;
	Table.CreateBox(1,1,1);
	Table.Translate(0,-1,0);
	Table.Scale(24,24,1);
	Table.Rotate(PI/2,0,0);

	cMusic anthem("sentmood.mid");
	anthem.Play();

	cTexture Marble("marbte2.bmp");
	cLinearTexture Granite("marble1.bmp");

	balls[0]->ApplyForce(-0.4f);		


	//	camera spring response
	cSecondOrder<point3d> CamRes;
	CamRes.SetParams(200,0.8f);


	//	create the world
	while(!Application()->ProcessMessages())
	{
		if(FrameController()->IsReady())
		{
			//	read input
			Input()->Catch();

			CamRes.Apply(point3d(Input()->m_MouseState.lX/30.0f,Input()->m_MouseState.lY/30.0f,0));

			Camera.m_Yaw		+=	CamRes.Output().x;
			Camera.m_Pitch	+=	CamRes.Output().y;


			if(Camera.m_Pitch>PI/4)Camera.m_Pitch=PI/4;
			if(Camera.m_Pitch<-PI/2)Camera.m_Pitch=-PI/2;

			if(KEYDOWN(VK_UP))				CameraLength+=0.2f;
			if(KEYDOWN(VK_DOWN))					CameraLength-=0.2f;


			if(KEYHIT(VK_SPACE)&&!m_KeyPressed)
			{
				balls[0]->ApplyForce(-0.3f);			m_KeyPressed=true;		

			}


			if(KEYHIT(VK_F2)&&!m_KeyPressed)
			{	
				NumBalls++;
				balls.Create(NumBalls,BallsRad,9);
				m_KeyPressed=true;
			}


			if(KEYHIT(VK_F3)&&NumBalls>1&&!m_KeyPressed)
			{
				NumBalls--;
				balls.Create(NumBalls,BallsRad,9);
				m_KeyPressed=true;
			}

			if(KEYHIT(VK_F4)&&!m_KeyPressed)
			{
				balls.Create(NumBalls,BallsRad,9);
				m_KeyPressed=true;
			}

			if(KEYDOWN(VK_F1)&&!m_KeyPressed)
			{
				help=!help;
				m_KeyPressed=true;
			}
			if(KEYDOWN('E')&&!m_KeyPressed)
			{
				env=!env;
				m_KeyPressed=true;
			}




			if(KEYUP(VK_F2)&&KEYUP(VK_F3)&&KEYUP(VK_SPACE)&&KEYUP(VK_F4)&&KEYUP(VK_F1)&&KEYUP('E'))
				m_KeyPressed=false;



			Camera.ComputeInfo();
			Camera.m_At=point3d(0,3,0);
			Camera.m_Loc=-CameraLength*Camera.m_Fwd+Camera.m_At;

			//	update the world
			Camera.Activate();

			//	update all balls physics
			balls.Update();

			//	clear previous frame
			glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
			

			//	draw new frame
			Marble.Activate();
			balls.Draw();
			Marble.Deactivate();

			Granite.Activate();
			Table.Draw();
			Granite.Deactivate();


			//	the env pass
			if(env)
			{
				Env.Activate();
				balls.Draw();
				Table.Draw();
				Env.Deactivate();
			}


			Granite.Activate();
			Table.Draw();
			Granite.Deactivate();
			

			//	enter 2D Mode
			Graphics()->InitDrawing2D();
			Graphics()->WriteFPS();
			Graphics()->WriteText(0,0,0xFFFFFF,"Steel Balls Simulation, By Shady El Mously");

			if(help)
				DisplayHelp();
			else
			Graphics()->WriteText(0,20,0xFFFFFF,"Press F1 for help");


			Graphics()->EndDrawing2D();

			
			//	Flush the buffers
			glFlush();
			//	and flip the frame
			Graphics()->Flip();
			//	update the timing
			FrameController()->Update();
		}
	}


}